#include <QString>

class KMessage;
class KSelectAction;
class QWidget;

class KMessageStyle
{
public:
    KMessageStyle();
    virtual ~KMessageStyle();

    const QString &currentStyle() const;

    /** Stringlist containing all available styles by accessable names
    This includes HD lookups, so you might want to cache it if using in loops etc.*/
    static QStringList list();

    /** Returns a html string - including css - for the current messsage, calls protected
    functions makeUpCss() and makeUpHtml(), which you want to reimplement in case
    NOTICE: the html string has a opening \<html\> and \<body\> tag, but NOT the closing tags*/
    QString makeUp( const KMessage &msg ) const;

    /** Returns a SelectAction including all available styels. To be used in toolbars, popups etc. */
    static KSelectAction *selector(const QString &title, QWidget *parent);

    /** Set the current style. If @p style does not (case sensitively) match any available style
    (see list() and selector()) a fancy fallback style will be set */
    bool set( const QString &style );
    
protected:
    /** Substitutes KMessage field placeholders and toggles classes visible dependend on KMessage
    usage. You want to reimplement this to toggle html classes beyond the KMessage default.
    NOTICE: To provide more variables, reimplement substitute() */
    QString makeUpCss( const KMessage &msg ) const;

    /** Substitutes KMessage field placeholders.
    NOTICE: In general, you do NOT want to reimplement this, as long as you don't want to inject
    additional html code into the header
    To provide more variables, reimplement substitute() */
    QString makeUpHtml( const KMessage &msg ) const;

    QString imgToDataUrl( const QImage &image ) const;

    const QString &senderPicURL( const KMessage *msg ) const;

    /** Replaces a variable by actual content. @Returns QString()(.isNull()) in case of no match.
    You want to reimplement this to provide variables beyond the KMessage default */
    QString substitute( const QString &variable, const KMessage &message ) const;
    
private:
    void tokenize( const QString &string, QStringList &tokens );
private:
    class Private;
    Private *const d;
};
