/*
 *   Copyright (c) 2009 Rudy Commenge <rudywi@gmail.com>
 *
 *   This library is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU Library General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or (at your
 *   option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful, but WITHOUT
 *   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *   FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 *   License for more details.
 *
 *   You should have received a copy of the GNU Library General Public License
 *   along with this library; see the file COPYING.LIB. If not, write to the
 *   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 *   02110-1301, USA.
 */

#ifndef KDEFORUM_RESOURCE_H
#define KDEFORUM_RESOURCE_H

#include <akonadi/resourcebase.h>
#include <attica/providermanager.h>
#include <QList>

#define CPT_TRY	3

//class KDEForum;

using namespace Akonadi;

class KDEForum_Resource : public Akonadi::ResourceBase,
                          public Akonadi::AgentBase::Observer
{
  Q_OBJECT

  public:
    KDEForum_Resource( const QString &id );
    ~KDEForum_Resource();
    void init();
    void getContacts(const int page);
    void getForums(const int page);
    void getPmFolders();
    void getTopics(const int page, const QString idForum);
    void getMessages(const int page);
    void getPm(const QString idFolder);
    void writeLog(const QString &text);

  public Q_SLOTS:
    virtual void configure( WId windowId );
    void contactsReceived(Attica::BaseJob* baseJob);
    void forumsReceived(Attica::BaseJob* baseJob);
    void topicsReceived(Attica::BaseJob* baseJob);
    void messagesReceived(Attica::BaseJob* baseJob);
    void pmFoldersReceived(Attica::BaseJob* baseJob);
    void pmReceived(Attica::BaseJob* baseJob);
    void checkItemRead(Attica::BaseJob* baseJob);


  protected Q_SLOTS:
    void retrieveCollections();
    void retrieveItems( const Akonadi::Collection &col );
    bool retrieveItem( const Akonadi::Item &item, const QSet<QByteArray> &parts );

  protected:
    virtual void aboutToQuit();
    virtual void itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection );
    virtual void itemChanged( const Akonadi::Item &item, const QSet<QByteArray> &parts );
    virtual void itemRemoved( const Akonadi::Item &item );

    
  private:
    void generateConfigFile();
    Attica::ProviderManager* m_pm; // Provide Manager
    int m_cptRequest; // Nb requests remaining before ERROR
    Item::List m_items; // Items retrieved
    Collection::List m_list; // Collections retrieved
    int m_nbCollec; // Nb Collections (Not include Forums Collections)
    Item m_item; // Current item retrieve    
    QString m_currentForum;
    QString m_currentPmFolder;
    Item m_currentItem;

};

#endif
