/*
 *   Copyright (c) 2009 Rudy Commenge <rudywi@gmail.com>
 *
 *   This library is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU Library General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or (at your
 *   option) any later version.
 * 
 *   This library is distributed in the hope that it will be useful, but WITHOUT
 *   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *   FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public
 *   License for more details.
 *
 *   You should have received a copy of the GNU Library General Public License
 *   along with this library; see the file COPYING.LIB. If not, write to the
 *   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 *   02110-1301, USA.
 */

#include "kdeforumsettings.h"

#include "settings.h"

#include <KConfigDialogManager>
#include <KWindowSystem>
#include <KDE/KMessageBox>
#include <QtCore/QTimer>




using namespace Akonadi;

KDEForumSettings::KDEForumSettings( WId windowId )
  : KDialog()
{
  ui.setupUi( mainWidget() );
  setButtons( Ok | Cancel );
  enableButtonOk(false);
  if ( windowId )
    KWindowSystem::setMainWindow( this, windowId );
  connect( this, SIGNAL( okClicked () ), SLOT( save() ) );
  connect( ui.kcfg_Username, SIGNAL( textChanged( QString ) ), SLOT( validate() ) );
  connect( ui.kcfg_Password, SIGNAL( textChanged( QString ) ), SLOT( validate() ) );
  connect( ui.kcfg_Urlforum, SIGNAL( textChanged( QString ) ), SLOT( validate() ) );
  connect( ui.kcfg_Visitor, SIGNAL( stateChanged ( int ) ), SLOT( validate() ) );

  //QTimer::singleShot( 0, this, SLOT( validate() ) );

  ui.kcfg_Urlforum->setText( QString( Settings::self()->urlforum() ) );
  if(isUrlValid())
      ui.lbUrlWarning->setVisible(false);
  else
      ui.lbUrlWarning->setVisible(true);
  if(bool(Settings::self()->visitor()) == true)
  {
      ui.kcfg_Visitor->setChecked(true);
      ui.kcfg_Username->clear();
      ui.kcfg_Username->setEnabled(false);
      ui.lbAccount->setEnabled(false);
      ui.kcfg_Password->clear();
      ui.kcfg_Password->setEnabled(false);
      ui.lbPass->setEnabled(false);
  }
  else
  {
      ui.kcfg_Visitor->setChecked(false);
      ui.kcfg_Username->setEnabled(true);
      ui.kcfg_Username->setText( QString( Settings::self()->username() ) );
      ui.kcfg_Password->setEnabled(true);
      ui.lbAccount->setEnabled(true);
      ui.kcfg_Password->setText( QString( Settings::self()->password() ) );
      ui.lbPass->setEnabled(true);
  }
  mManager = new KConfigDialogManager( this, Settings::self() );
  mManager->updateWidgets();
}

void KDEForumSettings::save()
{
      mManager->updateSettings();
      Settings::self()->setUsername( ui.kcfg_Username->text() );
      Settings::self()->setPassword( ui.kcfg_Password->text() );
      Settings::self()->setUrlforum( ui.kcfg_Urlforum->text() );
     Settings::self()->writeConfig();
}

void KDEForumSettings::validate()
{

  enableButtonOk(false);
  ui.lbUrlWarning->setVisible(true);
  if(ui.kcfg_Visitor->checkState() == Qt::Unchecked)
  {
      if(isUrlValid())
         ui.lbUrlWarning->setVisible(false);
      if(!ui.kcfg_Username->text().isEmpty() && !ui.kcfg_Password->text().isEmpty() && isUrlValid())
        enableButtonOk(true);

  }
  else
  {
      if(isUrlValid())
      {
          enableButtonOk(true);
          ui.lbUrlWarning->setVisible(false);
      }
  }

}

bool KDEForumSettings::isUrlValid() const
{
  QString res(ui.kcfg_Urlforum->text());
  return (res.length()>10);
}
