/*
 *  Copyright (C) 2010 Nokia Corporation
 *  Copyright (c) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
 *  Author: Rafael Roquetto <rafael.roquetto@kdab.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) version 3, or any
 *  later version accepted by the membership of KDE e.V. (or its
 *  successor approved by the membership of KDE e.V.), which shall
 *  act as a proxy defined in Section 6 of version 3 of the license.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public 
 *  License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef AKONADITELEPATHYRESOURCE_H
#define AKONADITELEPATHYRESOURCE_H

#include <akonadi/resourcebase.h>

#include "telepathyclient.h"

class TelepathyAccount;
class TelepathyContact;

class AkonadiTelepathyResource : public Akonadi::ResourceBase,
                           public Akonadi::AgentBase::Observer
{
  Q_OBJECT

  public:
    AkonadiTelepathyResource( const QString &id );
    ~AkonadiTelepathyResource();

  protected Q_SLOTS:
    void retrieveCollections();
    void retrieveItems( const Akonadi::Collection &col );
    void accountGotReady( const TelepathyAccount *account, const Akonadi::Collection &collection );
    void contactGotReady( const TelepathyContact *contact, const Akonadi::Item &item );
    void contactsAdded( QList<TelepathyContact*> contacts);
    void contactAdded( TelepathyContact *contact);
    void contactChanged( const TelepathyContact *contact );
    void contactRemoved( TelepathyContact *contact );
    void itemCreateJobFinished( KJob *job );
    void itemModifyJobFinished( KJob *job );
    void itemDeleteJobFinished( KJob *job );
    void itemFetchedForChange( KJob *job );
    void itemFetchedForRemoval( KJob *job );
    void collectionFetchedForAdd( KJob *job );

    bool retrieveItem( const Akonadi::Item &item, const QSet<QByteArray> &parts );

  protected:
    virtual void itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection );
    virtual void itemChanged( const Akonadi::Item &item, const QSet<QByteArray> &parts );
    virtual void itemRemoved( const Akonadi::Item &item );
    virtual void doSetOnline(bool online);

  private:
    void fillInItem( const TelepathyContact *contact, Akonadi::Item &item );

    TelepathyAccount *accountForCollection( const Akonadi::Collection &collection );
    TelepathyContact *contactForItem( const Akonadi::Item &item );

    Akonadi::Collection collectionForContact( const TelepathyContact *contact );

    TelepathyClient mTelepathy;

    QList<Akonadi::Collection> mCollectionList;
};

#endif
