/*
 *  Copyright (C) 2010 Nokia Corporation
 *  Copyright (c) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
 *  Author: Rafael Roquetto <rafael.roquetto@kdab.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) version 3, or any
 *  later version accepted by the membership of KDE e.V. (or its
 *  successor approved by the membership of KDE e.V.), which shall
 *  act as a proxy defined in Section 6 of version 3 of the license.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public 
 *  License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "booleanattribute.h"

BooleanAttribute::BooleanAttribute( const QString &key, bool value ) :
    mKey( key ),
    mValue( value )
{
}

void BooleanAttribute::setKey( const QString &key )
{
    mKey = key;
}

void BooleanAttribute::setValue( bool value )
{
    mValue = value;
}

QString BooleanAttribute::key() const
{
    return mKey;
}

bool BooleanAttribute::value() const
{
    return mValue;
}

QByteArray BooleanAttribute::type() const
{
    return mKey.toUtf8();
}

QByteArray BooleanAttribute::serialized() const
{
    return ( mValue ) ? "true" : "false";
}

void BooleanAttribute::deserialize( const QByteArray &data )
{
    mKey = type();
    mValue = ( data == "true" );
}

Akonadi::Attribute * BooleanAttribute::clone() const
{
    return new BooleanAttribute( mKey, mValue );
}
