/*
 *  Copyright (C) 2010 Nokia Corporation
 *  Copyright (c) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
 *  Author: Rafael Roquetto <rafael.roquetto@kdab.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) version 3, or any
 *  later version accepted by the membership of KDE e.V. (or its
 *  successor approved by the membership of KDE e.V.), which shall
 *  act as a proxy defined in Section 6 of version 3 of the license.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public 
 *  License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <akonadi/resourcebase.h>
#include "telepathyaccount.h"

#include "telepathyaccountfetchjob.h"

TelepathyAccountFetchJob::TelepathyAccountFetchJob( const TelepathyAccount *acc,
        const Akonadi::Collection &col, QObject *parent) :
    QObject( parent ),
    account( acc ),
    collection( col ),
    fetched( false )
{
    connect( account,
            SIGNAL(gotReady(const TelepathyAccount*)),
            SLOT(accountGotReady(const TelepathyAccount*)));
}

TelepathyAccountFetchJob::~TelepathyAccountFetchJob()
{
}

void TelepathyAccountFetchJob::fetch()
{
    if ( account->isReady() )
        accountGotReady( account );

    fetched = true;
}

void TelepathyAccountFetchJob::accountGotReady( const TelepathyAccount *acc )
{
    Q_UNUSED( acc );

    emit finished( account, collection );

    if (fetched)
        deleteLater();
}
