/*
 *  Copyright (C) 2010 Nokia Corporation
 *  Copyright (c) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
 *  Author: Rafael Roquetto <rafael.roquetto@kdab.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) version 3, or any
 *  later version accepted by the membership of KDE e.V. (or its
 *  successor approved by the membership of KDE e.V.), which shall
 *  act as a proxy defined in Section 6 of version 3 of the license.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public 
 *  License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef TELEPATHYACCOUNT_H
#define TELEPATHYACCOUNT_H

#include <QObject>
#include <TelepathyQt4/Account>

class TelepathyContact;

class TelepathyAccount : public QObject
{
    Q_OBJECT

    public:
        TelepathyAccount(const Tp::AccountPtr &account, QObject *parent = 0);
        ~TelepathyAccount();

        QString id() const;

        bool isReady() const;
        bool isOnline() const;

        QList<TelepathyContact*> contactList() const;

        QString displayName() const;

    Q_SIGNALS:
        void accountReady();
        void connectionReady();
        void gotReady(const TelepathyAccount *);
        void contactsAdded(QList<TelepathyContact*> contacts);
        void contactChanged(const TelepathyContact *contact);
        void contactAdded(TelepathyContact *contact);
        void contactRemoved(TelepathyContact *contact);

    private Q_SLOTS:
        void onAccountReady(Tp::PendingOperation *op);
        void handleAccountReady();
        void connectionChanged(bool online);
        void onConnectionReady(Tp::PendingOperation *op);
        void handleConnectionReady();
        void onUpgradeContacts(Tp::PendingOperation *op);
        void onUpgradeNewContacts(Tp::PendingOperation *op);
        void clearContactList();
        void setReady(bool ready);
        void onContactAvatarRetrieved(uint contact, const QString &token,
                const QByteArray &avatar, const QString &);
        void onContactAvatarUpdated(uint contact, const QString &token);
        void onAllKnownContactsChanged(const Tp::Contacts &added, const Tp::Contacts &removed);
        void onGroupMembersChanged(const QString &group, const Tp::Contacts &added, const Tp::Contacts &removed);
        void onContactDestroyed(const Tp::ContactPtr &contact);
        void onContactSubscriptionChanged(TelepathyContact *contact);
        void onContactInfoUpdated(TelepathyContact *contact);
        void onContactPresenceChanged(TelepathyContact *contact);
        void onContactCapabilitiesChanged(TelepathyContact *contact);
        
        void contactInfoStub(uint contact, const Tp::ContactInfoFieldList &contactInfo);

    private:
        void addContact(Tp::ContactPtr &contact);
        void handleNewContacts(const Tp::Contacts &added);
        void handleRemovedContacts(const Tp::Contacts &removed);
        void retrieveAvatars(const Tp::UIntList &handles);

        TelepathyContact *getContactByHandle(uint handle);
        TelepathyContact *getContactByPtr(const Tp::ContactPtr &tpContact);
        
        bool hasContact(Tp::ContactPtr contact) const;

        Tp::AccountPtr mAccount;
        Tp::ConnectionPtr mConnection;

        QList<TelepathyContact*> mContactList;

        QString mDisplayName;

        bool mReady;
};

#endif /* TELEPATHYACCOUNT_H */
