/*
 *  Copyright (C) 2010 Nokia Corporation
 *  Copyright (c) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
 *  Author: Rafael Roquetto <rafael.roquetto@kdab.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) version 3, or any
 *  later version accepted by the membership of KDE e.V. (or its
 *  successor approved by the membership of KDE e.V.), which shall
 *  act as a proxy defined in Section 6 of version 3 of the license.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public 
 *  License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <TelepathyQt4/Debug>
#include <TelepathyQt4/Constants>
#include <TelepathyQt4/Types>
#include <TelepathyQt4/ConnectionManager>
#include <TelepathyQt4/PendingStringList>
#include <TelepathyQt4/AccountManager>
#include <TelepathyQt4/PendingReady>

#include <QDebug>

#include "telepathyclient.h"
#include "telepathyaccount.h"

int TelepathyClient::mRefCount = 0;

void TelepathyClient::init()
{
    Tp::registerTypes();
    Tp::enableDebug(true);
    Tp::enableWarnings(true);
}

TelepathyClient::TelepathyClient(QObject *parent) : 
    QObject(parent),
    mReady(false)
{
    if (mRefCount == 0) {
        init();
        mRefCount++;
    }

    mAccountManager = Tp::AccountManager::create();

    connect(mAccountManager->becomeReady(),
            SIGNAL(finished(Tp::PendingOperation*)),
            SLOT(onAccountManagerReady(Tp::PendingOperation*)));
}

TelepathyClient::~TelepathyClient()
{
    mRefCount--;
}

void TelepathyClient::roster()
{
    return;
    Tp::PendingStringList *l = Tp::ConnectionManager::listNames();

    connect(l, SIGNAL(finished(Tp::PendingOperation*)),
            this, SLOT(gotConnManagerList(Tp::PendingOperation*)));
}

QList<TelepathyAccount *> TelepathyClient::accountList() const
{
    return mAccountList;
}

bool TelepathyClient::isReady() const
{
    return mReady;
}

void TelepathyClient::gotConnManagerList(Tp::PendingOperation *op)
{
    if (op->isError()) {
        qDebug() << "Failed to get connection managers list";
        return;
    }

    Tp::PendingStringList *psl = qobject_cast<Tp::PendingStringList*>(op);

    if (!psl) {
        qDebug() << "Operation is not of Tp::PendingStringList type";
        return;
    }

    foreach (QString manager, psl->result()) {
        handleConnectionManager(manager);
    }
    
}

void TelepathyClient::handleConnectionManager(const QString &name)
{
    Tp::ConnectionManagerPtr manager = Tp::ConnectionManager::create(name);

    qDebug() << manager;
}

void TelepathyClient::onAccountManagerReady(Tp::PendingOperation *op)
{
    if (op->isError()) {
        qDebug() << "Error: " << op->errorName() << op->errorMessage();
        return;
    }

    QList<Tp::AccountPtr> accounts = mAccountManager->allAccounts();

    foreach (Tp::AccountPtr account, accounts) {
        TelepathyAccount *telepathyAccount = new TelepathyAccount(account, this);

        mAccountList << telepathyAccount;
    }

    setReady(true);
}

void TelepathyClient::setReady(bool r)
{
    mReady = r;
}
