/*
 *  Copyright (C) 2010 Nokia Corporation
 *  Copyright (c) 2010 Klarälvdalens Datakonsult AB, a KDAB Group company <info@kdab.com>
 *  Author: Rafael Roquetto <rafael.roquetto@kdab.com>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) version 3, or any
 *  later version accepted by the membership of KDE e.V. (or its
 *  successor approved by the membership of KDE e.V.), which shall
 *  act as a proxy defined in Section 6 of version 3 of the license.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public 
 *  License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef TELEPATHYCONTACT_H
#define TELEPATHYCONTACT_H

#include <QObject>
#include <QImage>
#include <TelepathyQt4/Contact>
#include <TelepathyQt4/Types>

class TelepathyContact : public QObject
{
    Q_OBJECT

    public:
        enum Subscription {
            Unsubscribed,
            Subscribed,
            Ask
        };

        TelepathyContact(Tp::ContactPtr contact,
                QObject *parent = 0);
        ~TelepathyContact();

        QString id() const;
        QString alias() const;
        QString fullName() const;
        QString email() const;
        QString url() const;
        QString tel() const;
        QString org() const;
        QString birthday() const;
        QString productId() const;
        QString title() const;
        QString street() const;
        QString city() const;
        QString state() const;
        QString zip() const;
        QString country() const;

        QString presenceStatus() const;
        QString presenceMessage() const;

        uint presenceType() const;

        Subscription subscription() const;

        bool isReady() const;
        bool supportsTextChats() const;
        bool supportsMediaCalls() const;
        bool supportsAudioCalls() const;
        bool supportsVideoCalls() const;
        bool supportsUpgradingCalls() const;

        void setAvatar(const QString &token, const QByteArray &avatar);
        void setAccountId(const QString &accountId);
        void updateInfo();

        const Tp::ContactPtr tpContact() const;
        const QString avatarToken() const;
        const QString accountId() const;

        const QImage &avatar() const;

    Q_SIGNALS:
        void gotReady(const TelepathyContact *);
        void subscriptionChanged(TelepathyContact *);
        void infoUpdated(TelepathyContact *);
        void presenceChanged(TelepathyContact *);
        void capabilitiesChanged(TelepathyContact *);

    private Q_SLOTS:
        void handleInfoChanged(const Tp::ContactInfoFieldList &);
        void onRefreshInfo(Tp::PendingOperation *op);
        void onSubscriptionStateChanged(Tp::Contact::PresenceState state);
        void onAliasChanged(const QString &alias);
        void onPresenceChanged(const QString &status, uint type,
                const QString &message);
        void onCapabilitiesChanged(Tp::ContactCapabilities *);

    private:
        void setId(const QString &id);
        void setAlias(const QString &alias);
        void setFullName(const QString &fullName);
        void setEmail(const QString &email);
        void setUrl(const QString &url);
        void setTel(const QString &tel);
        void setOrg(const QString &org);
        void setBirthday(const QString &bday);
        void setProductId(const QString &prodId);
        void setTitle(const QString &title);
        void setStreet(const QString &street);
        void setCity(const QString &city);
        void setState(const QString &state);
        void setZip(const QString &zip);
        void setCountry(const QString &country);
        void setReady(bool ready);
        void setAddressInfo(const QStringList &addr);
        void setSubscription(Subscription subscription);
        void setPresenceStatus(const QString &status);
        void setPresenceMessage(const QString &message);
        void setPresenceType(uint type);
        void setSupportsTextChats(bool supportsTextChats);
        void setSupportsMediaCalls(bool supportsMediaCalls);
        void setSupportsAudioCalls(bool supportsAudioCalls);
        void setSupportsVideoCalls(bool supportsVideoCalls);
        void setSupportsUpgradingCalls(bool supportsUpgradingCalls);

        Tp::ContactPtr mContact;
        Tp::ContactInfoFieldList mInfo;

        QString mId;
        QString mAlias;
        QString mFullName;
        QString mEmail;
        QString mUrl;
        QString mTel;
        QString mOrg;
        QString mBirthday;
        QString mProductId;
        QString mTitle;
        QString mStreet;
        QString mCity;
        QString mState;
        QString mZip;
        QString mCountry;

        QString mAvatarToken;
        QString mAccountId;

        QString mPresenceStatus;
        QString mPresenceMessage;

        uint mPresenceType;

        QImage mAvatar;

        Subscription mSubscription;

        bool mSupportsTextChats;
        bool mSupportsMediaCalls;
        bool mSupportsAudioCalls;
        bool mSupportsVideoCalls;
        bool mSupportsUpgradingCalls;

        bool mReady;
};

Q_DECLARE_METATYPE( TelepathyContact* );
Q_DECLARE_METATYPE( const TelepathyContact* );

#endif /* TELEPATHYCONTACT_H */
