/*
 * Copyright (c) 2008  Igor Trindade Oliveira <igor_trindade@yahoo.com.br>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "retrievecollectionsscript.h"

#include <akonadi/collectionfetchjob.h>

#include <kdebug.h>

#include <QStringList>

RetrieveCollectionsScript::RetrieveCollectionsScript(scriptableResource *resource)
{
  mResource = resource;
}

void RetrieveCollectionsScript::setMimeType(QString mimeType)
{
  mMimeType = mimeType;
}

QString RetrieveCollectionsScript::getMimeType()
{
  return mMimeType;
}

Akonadi::Collection::List RetrieveCollectionsScript::getRetriviedCollections()
{
  return mColList;
}

void RetrieveCollectionsScript::setRetriviedCollections(Akonadi::Collection::List colList)
{
  mColList = colList;
}

void RetrieveCollectionsScript::setCollectionName(QString collectionName)
{
  mCollectionName = collectionName;
}

void RetrieveCollectionsScript::setCollectionPath(QString path)
{
 mCollectionPath = path;
}

QString RetrieveCollectionsScript::getCollectionPath()
{
  return mCollectionPath;
}

QString RetrieveCollectionsScript::getCollectionName()
{
  return mCollectionName;
}

void RetrieveCollectionsScript::configureCollection(QString name, QString remoteId, QString parent, QStringList mimetypes)
{
  kDebug() << "name=" << name << ", remoteId=" << remoteId
           << ", parent=" << parent << "mimetype=" << mimetypes;
  Akonadi::Collection parentCollection = getCollectionByName(parent);
  if ( !parentCollection.isValid() && parentCollection.remoteId().isEmpty() ) {
    kError() << "No parent collection with name" << parent;
    return;
  }

  Akonadi::Collection c;
  c.setParent( parentCollection );
  c.setRemoteId( remoteId );
  c.setName( name );

  c.setContentMimeTypes( mimetypes );

  mColList << c;

  mResource->setSynchronizePass( QString("collection"));
  mResource->synchronize();  
}

Akonadi::Collection::List  RetrieveCollectionsScript::showCollections(){
  Akonadi::CollectionFetchJob *job = new Akonadi::CollectionFetchJob(
                                        Akonadi::Collection::root(),
                                        Akonadi::CollectionFetchJob::Recursive);

    job->exec();
    Akonadi::Collection::List collections = job->collections();
    return collections;
}

Akonadi::Collection  RetrieveCollectionsScript::getCollectionByName(const QString &colname){
  // first check in locally created ones
  foreach ( const Akonadi::Collection &collection, mColList ) {
    if( colname == collection.name()  ) return collection;
  }

  if( colname == QString("root") )
    return Akonadi::Collection::root();

  Akonadi::Collection::List colist = showCollections();
  foreach(const Akonadi::Collection &col, colist){
    if( colname == col.name()  ) return col;
  }

  return Akonadi::Collection();
}

