/*
 * Copyright (c) 2008  Igor Trindade Oliveira <igor_trindade@yahoo.com.br>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "scriptableresource.h"

#include "settings.h"
#include "settingsadaptor.h"

#include "itemfactory.h"
#include "item.h"

#include "configreader.h"

#include "scriptable.h"

#include <QtDBus/QDBusConnection>

#include <kross/core/manager.h>
#include <kross/core/action.h>

#include <kfiledialog.h>
#include <klocalizedstring.h>

#include <kdebug.h>
#include <kstandarddirs.h>

scriptableResource::scriptableResource( const QString &id )
  : ResourceBase( id )
{
  new SettingsAdaptor( Settings::self() );
  QDBusConnection::sessionBus().registerObject( QLatin1String( "/Settings" ),
                            Settings::self(), QDBusConnection::ExportAdaptors );

  script = new Scriptable(this);
  action = new Kross::Action(this,"Scriptable");
  configureAction();
  
  action->trigger();
}

scriptableResource::~scriptableResource()
{
  delete action;
  delete script;
}

void scriptableResource::retrieveCollections()
{
  //if( synchronizePass == QString("collection") ){ 
    collectionsRetrieved( script->getRetrieveCollectionsScript()->getRetriviedCollections() );
//  } 
}

void scriptableResource::retrieveItems( const Akonadi::Collection &collection )
{
  Q_UNUSED( collection );

  if( synchronizePass == QString("items") ){
   
    const QString path = script->getRetrieveItemsScript()->getRemoteId();

    QDir dir( path );

    QStringList filters;
    filters << QLatin1String( "*.vcf" );
    const QStringList fileList = dir.entryList( filters, QDir::Files );

    Akonadi::Item::List items;
    foreach( const QString &file, fileList ) {
      Akonadi::Item item( script->getRetrieveItemsScript()->getMimeType()  );
      item.setRemoteId( path + QLatin1Char( '/' ) + file );

      items << item;
      script->getRetrieveItemsScript()->setRetriviedItems( item );
    }
    kDebug()<<"teste:"<<fileList;
    itemsRetrieved( items );
  }
}

bool scriptableResource::retrieveItem( const Akonadi::Item &item, const QSet<QByteArray> &parts )
{
  Q_UNUSED(item);
  Q_UNUSED(parts);
 
  if( synchronizePass == QString("item") ){

    Akonadi::Item nItem;
    const QString fileName = script->getRetrieveItemScript()->getFileName();
    ItemFactory factory;

    nItem.setMimeType("text/vcard"); //right now only works with vcard items
    nItem.setRemoteId(fileName);

    Item *i = factory.createItem("vcard");
    Akonadi::Item newitem = i->loadItem( fileName, nItem);

    script->getRetrieveItemScript()->setReadItem( newitem );

    itemRetrieved( newitem );
    delete i;
  }
  return true;
}

void scriptableResource::aboutToQuit()
{
}

void scriptableResource::configure( WId windowId )
{
  Q_UNUSED( windowId );
  status(Idle, QString());
}

void scriptableResource::itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection )
{
  Q_UNUSED(item);
  Q_UNUSED(collection);
}

void scriptableResource::itemChanged( const Akonadi::Item &item, const QSet<QByteArray> &parts )
{
  Q_UNUSED(item);
  Q_UNUSED(parts);
}

void scriptableResource::itemRemoved( const Akonadi::Item &item )
{
  Q_UNUSED(item);
}

void scriptableResource::configureAction()
{
  const QString pathToConfig = KStandardDirs::locate("config","scriptableconf.xml");
  //ConfigReader config( pathToConfig );
  ConfigReader config( QString("/usr/share/kde4/config/scriptableconf.xml"));
  action->setFile(config.getScriptLocation("main"));
  
  action->addObject(script->getRetrieveCollectionsScript(),"RetrieveCollections");
  action->addObject(script->getRetrieveItemScript(),"RetrieveItem");
  action->addObject(script->getRetrieveItemsScript(),"RetrieveItems");
}

void scriptableResource::setSynchronizePass(QString synchronizePass)
{
  this->synchronizePass = synchronizePass;
}

AKONADI_RESOURCE_MAIN( scriptableResource )

#include "scriptableresource.moc"
