/*
 * Copyright (c) 2008  Igor Trindade Oliveira <igor_trindade@yahoo.com.br>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef SCRIPTABLERESOURCE_H
#define SCRIPTABLERESOURCE_H

#include <akonadi/resourcebase.h>

#include <kross/core/manager.h>
#include <kross/core/action.h>

class Scriptable;

class scriptableResource : public Akonadi::ResourceBase,
                           public Akonadi::AgentBase::Observer
{
  Q_OBJECT
 
  friend class RetrieveCollectionsScript;
  friend class RetrieveItemsScript;  

  public:
    scriptableResource( const QString &id );
    ~scriptableResource();

    void setSynchronizePass(QString synchronizePass);

  private:
    Kross::Action *action;
    Scriptable *script;
    QString synchronizePass;

    void configureAction();     

  public Q_SLOTS:
    virtual void configure( WId windowId );

  public Q_SLOTS:
    void retrieveCollections();
    void retrieveItems( const Akonadi::Collection &col );
    bool retrieveItem( const Akonadi::Item &item, const QSet<QByteArray> &parts );
   
  protected:
    virtual void aboutToQuit();

    virtual void itemAdded( const Akonadi::Item &item, const Akonadi::Collection &collection );
    virtual void itemChanged( const Akonadi::Item &item, const QSet<QByteArray> &parts );
    virtual void itemRemoved( const Akonadi::Item &item );
  
};

#endif
