/*
 * Copyright (c) 2008  Igor Trindade Oliveira <igor_trindade@yahoo.com.br>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "item.h"
#include "vcarditem.h"
#include <krandom.h>
#include <kabc/addressee.h>
#include <kabc/vcardconverter.h>
//#include <QFile>

VCardItem::VCardItem()
{}

Akonadi::Item VCardItem::loadItem(const QString &filename , Akonadi::Item item)
{
  QFile file( filename );
  file.open( QFile::ReadOnly ); 

  const QByteArray data = file.readAll();

  convertVCard( data );
  
  Akonadi::Item newItem( item );
  newItem.setPayload<KABC::Addressee>( addressee );
 
  return newItem;
}

Akonadi::Item VCardItem::addToCollection(const Akonadi::Item &item, const QString &path)
{
  addPayload( item );
  addUID();

  QFile file( path + QLatin1Char( '/' ) + addressee.uid() + QLatin1String( ".vcf" ) );

  file.open( QFile::WriteOnly );

  writeVCard( file );

  Akonadi::Item newItem( item );
  newItem.setRemoteId( file.fileName() );
  newItem.setPayload<KABC::Addressee>( addressee );

  return newItem;
}
  
Akonadi::Item VCardItem::modifyItem(const Akonadi::Item &item, const QString &filename)
{
  addPayload( item );
  addUID();
  
  QFile file( filename );
 
  file.open( QFile::WriteOnly );

  writeVCard( file );

  Akonadi::Item newItem( item );
  newItem.setPayload<KABC::Addressee>( addressee );

  return newItem;
}

bool VCardItem::convertVCard(const QByteArray &data)
{
  KABC::VCardConverter converter;
  this->addressee = converter.parseVCard( data );
  if ( addressee.isEmpty() )
    return false;
  return true;
}

bool VCardItem::writeVCard(QFile &file)
{
  KABC::VCardConverter converter;
  file.write( converter.createVCard( addressee ) );
  if ( file.error() != QFile::NoError )
    return false;
  return true;
}

void VCardItem::addPayload(const Akonadi::Item &item)
{
 
  KABC::Addressee addressee;
  if ( item.hasPayload<KABC::Addressee>() )
    addressee = item.payload<KABC::Addressee>();
}

void VCardItem::addUID()
{
  if ( addressee.uid().isEmpty() )
    addressee.setUid( KRandom::randomString( 10 ) );
}

