#!/bin/bash

echo "Automated test run of KABC -> Akonadi migration"

if [ ! -s config.xml ]; then
  echo "Testrunner configuration file config.xml missing or empty"
  exit 1
elif [ ! -r config.xml ]; then
  echo "No read permission for testrunner configuration file config.xml"
  exit 1
fi

TESTDATADIR=$(pwd)/testdata

echo "Setting TESTDATADIR to '" $TESTDATADIR "'"

if [ ! -d $TESTDATADIR ]; then
  echo "Directory containing the test data does not exist"
  exit 1;
fi

rm -rf tmp 2> /dev/null
mkdir tmp 2> /dev/null

rm -rf log 2> /dev/null
mkdir log 2> /dev/null

echo "Starting testrunner..."

akonaditest config.xml >&2 2>> log/akonaditest.log &
TESTRUNNER_PID=$!

# wait for the testenvironment.sh
while [ ! -e testenvironment.sh ]; do
  sleep 1
done

source testenvironment.sh

# wait for the akonadiconnectionrc if necessary
while [ ! -e $XDG_CONFIG_HOME/akonadi/akonadiconnectionrc ]; do
  sleep 1
done

echo "Done"
echo

echo "Extracting pre-test reference data"

kabcclient -L -of uid 2>> log/kabcclient.log | sort > tmp/pretest.uids
echo >> log/kabcclient.log
echo >> log/kabcclient.log
kabcclient -L -if uid -of vcard 2>> log/kabcclient.log < tmp/pretest.uids > tmp/pretest.vcf
echo >> log/kabcclient.log
echo >> log/kabcclient.log

echo "Done. " $(wc -l tmp/pretest.uids) "contact UIDs"
echo

echo "Starting migration tool"

kres-migrator --contacts-only >&2 2>> log/kres-migrator.log

echo "Done"
echo

echo "Extracting post-test reference data"

kabcclient -L -of uid 2>> log/kabcclient.log | sort > tmp/posttest.uids
echo >> log/kabcclient.log
echo >> log/kabcclient.log
kabcclient -L -if uid -of vcard 2>> log/kabcclient.log < tmp/posttest.uids > tmp/posttest.vcf

echo "Done. " $(wc -l tmp/posttest.uids) "contact UIDs"
echo

echo "Comparing reference data, output of diff follows"
echo "------diff uids-------"

diff -u tmp/pretest.uids tmp/posttest.uids

echo "------diff vcards-------"
diff -u tmp/pretest.vcf tmp/posttest.vcf

echo "------------------------"
echo

echo "Checking if the Akonadi bridge is the new standard resource"

RESOURCE_KEYS=$(kreadconfig --file kresources/contact/stdrc --group General --key ResourceKeys)

STANDARD_KEY=$(kreadconfig --file kresources/contact/stdrc --group General --key Standard)

if [ "$RESOURCE_KEYS" != "$STANDARD_KEY" ]; then
  echo "ResourceKeys (" $RESOURCE_KEYS ") not equal Standard (" $STANDARD_KEY ")"
fi

STANDARD_TYPE=$(kreadconfig --file kresources/contact/stdrc --group "Resource_"$STANDARD_KEY --key ResourceType)

if [ "$STANDARD_TYPE" != "akonadi" ]; then
  echo "ResourceType of standard resource is" $STANDARD_TYPE "and not akonadi"
fi

echo
echo "Test keeps running in case you want to perform manual checks"
echo "To get a suitable shell environment, source testenvironment.sh"
echo "This also contains a shell function to stop the test, called 'shutdown-testenvironment'"

