/* GroupDAV Akonadi Resource
 * Copyright (C) 2009 Marco Nelles (marco.nelles@credativ.de)
 * <http://www.credativ.de/>
 * <http://opensource.maniatek.com/kgroupdav/>
 *
 * credativ GmbH
 * Hohenzollernstr. 133
 * 41061 Mönchengladbach
 * E-Mail: info@credativ.de
 *
 * RFC 2445 (iCalendar)
 * RFC 2518 (WebDAV)
 * RFC 2616 (HTTP)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GROUPDAVRESOURCE_H
#define GROUPDAVRESOURCE_H

//Qt
#include <QtDBus/QDBusConnection>

//KDE
#include <akonadi/resourcebase.h>
#include <akonadi/changerecorder.h>
#include <akonadi/itemfetchscope.h>
#include <akonadi/monitor.h>
#include <akonadi/itemcreatejob.h>
#include <akonadi/itemfetchjob.h>
#include <akonadi/itemmodifyjob.h>
#include <akonadi/itemdeletejob.h>
#include <kcal/icalformat.h>
#include <kcal/calendar.h>
#include <kcal/calendarlocal.h>
#include <kcal/incidence.h>


#include "kgroupdavproxy/kgroupdavproxy.h"

#include "configdialog.h"

#include "settings.h"
#include "settingsadaptor.h"

class GroupDAVResource : public Akonadi::ResourceBase, public Akonadi::AgentBase::Observer {

Q_OBJECT

public:
  GroupDAVResource(const QString &id);
  ~GroupDAVResource();

public Q_SLOTS:
  virtual void configure(WId windowId);

protected Q_SLOTS:
  void retrieveCollections();
  void retrieveItems(const Akonadi::Collection &col);
  bool retrieveItem(const Akonadi::Item &item, const QSet<QByteArray> &parts);
  void synchronize();

  void proxyItemAdd(const quint32 id);
  void proxyItemMod(const quint32 id);
  void proxyItemDel(const quint32 id);

protected:
  virtual void aboutToQuit();

  virtual void itemAdded(const Akonadi::Item &item, const Akonadi::Collection &collection);
  virtual void itemChanged(const Akonadi::Item &item, const QSet<QByteArray> &parts);
  virtual void itemRemoved(const Akonadi::Item &item);

  virtual void doSetOnline(bool online);

private:
  KGroupDAVProxy p_proxy;
  //Akonadi::Monitor *p_monitor;

  bool p_load_configuration();
  void p_shutdown();

};

#endif
