/*
  This file is part of the Grantlee template system.

  Copyright (c) 2011 Stephen Kelly <steveire@gmail.com>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either version
  2.1 of the Licence, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "kgrantleeintegration.h"

#include <QDebug>
#include <QDir>

#include <kstandarddirs.h>
#include "kdelocalizer.h"

#include <grantlee/engine.h>
#include <grantlee/templateloader.h>

KGrantleeIntegration::KGrantleeIntegration()
{
}

void KGrantleeIntegration::setType(const QString& type)
{
  m_type = type;
}

void KGrantleeIntegration::setTheme(const QString& theme)
{
  m_loader->setTheme(theme);
}

QStringList KGrantleeIntegration::availableThemes() const
{
  QStringList list;

  Q_FOREACH(const QString &templateDir, m_loader->templateDirs()) {
    QDir themeDir(templateDir);
    list << themeDir.entryList(QDir::Dirs | QDir::NoDotAndDotDot);
  }
  list.removeDuplicates();
  return list;
}

Grantlee::Template KGrantleeIntegration::loadByName(const QString& name)
{
  return m_engine->loadByName(name);
}

void KGrantleeIntegration::initializeEngine(QObject* parent)
{
  initLocalizer();
  Q_ASSERT(m_localizer);
  m_loader = Grantlee::LocalizedFileSystemTemplateLoader::Ptr(new Grantlee::LocalizedFileSystemTemplateLoader(m_localizer));

  KStandardDirs standardDirs;

  m_loader->setTemplateDirs(standardDirs.findDirs("data", "contacts_kde/grantlee/" + m_type));

  m_engine = new Grantlee::Engine(parent);

  // TODO: Consider adding API to the builder to configure these.
  m_engine->addDefaultLibrary( "grantlee_i18n" );
  m_engine->addDefaultLibrary( "grantlee_scriptabletags" );

  m_engine->addTemplateLoader(m_loader);
  m_engine->setPluginPaths(standardDirs.findDirs("lib", ""));
}

void KGrantleeIntegration::initLocalizer()
{
  if (!m_localizer)
    m_localizer = KDELocalizer::Ptr(new KDELocalizer);
}

Grantlee::Context KGrantleeIntegration::createContext()
{
  Grantlee::Context context;
  initLocalizer();
  context.setLocalizer(m_localizer);

  return context;
}
