=begin
  This is the class to hold information about an entry in the 
  holidays file. That can be either a holiday, a comment, an empty
  line, or another include file.

  @short Holiday Entry
  @author Reinhold Kainhofer <reinhold@kainhofer.com>
  @version 0.1
=end

=begin    
    mEntryType: Holiday | Comment | Empty | Include
    mComment: String
    mName: String
    mHolidayType: Holiday | Weekend | BankHoliday | Specialday | Information
    mDateBy: date | monthpos | special
        mDateDay: 1-31
        mDateMonth: 1-12
        mDateYear: 1900- or -1
        mDatePos: 1./2./3./4./5./last
        mDateWDay: mtwtfss
        mDatePosMonth: 1-12
        mDateSpecial: Easter|Pessach|Ramadan
    
    mHasWeekday: bool
        mWeekday:+-mtwtfss
    mHasOffset: bool
    	mOffset: int
    mHasLength:
        mLength: uint
    mHasShift:
    	mShiftFrom:mtwtfss (or'ed)
	mShiftTo: mtstfss (or'ed)
=end


class EntryType
  Unknown = 0
  Holiday = 1
  Comment = 2
  Empty   = 3
  Include = 4
end

class HolidayType
  Holiday = 1
  Weekend = 2
  BankHoliday = 3
  SpecialDay = 4
  Information = 5
end

class DateBy
  Date = 1
  MonthPos = 2
  Special = 3
end

class SpecialDate
  Easter = 1
  Pessach = 2
end

class Weekday
  Monday    = 1
  Tuesday   = 2
  Wednesday = 3
  Thursday  = 4
  Friday    = 5
  Saturday  = 6
  Sunday    = 7
end

class KHoliday_Entry
   attr_accessor :EntryType, :Comment, :Include, :Name, :HolidayType
   attr_accessor :DateBy, :DateDay, :DateMonth, :DateYear, :DatePos
   attr_accessor :DateWDay, :DatePosMonth, :DateSpecial
   attr_accessor :HasWeekDay, :HasOffset, :HasLength, :HasShift
   attr_accessor :Weekday, :Offset, :ShiftFrom, :Length, :ShiftTo
   attr_accessor :WholeString
 
    def initialize()
	@EntryType=EntryType::Holiday
	@Comment=@Include=@WholeString=""
	@Name="New holiday"
	@HolidayType=HolidayType::Holiday
	@DateBy=DateBy::Date
	@DateDay=@DateMonth=1
	@DateYear=-1
	
	@DatePos=1
	@DateWDay=0
	@DatePosMonth=1
	@DateSpecial=SpecialDate::Easter
	
	@HasWeekDay = @HasOffset = @HasLength = @HasShift = false
	@Weekday = @Offset = @ShiftFrom = 0
	@Length = @ShiftTo = 1
    end

    def shortText()
      case @EntryType
          when EntryType::Empty 
              return "Empty line"
          when EntryType::Comment
              return "Comment: "+@Comment
          when EntryType::Include
              return "Include: "+@Include
          when EntryType::Holiday
              return "Holiday: "+@Name
          else
              return "UNKNOWN entry type"
      end
    end


    def dump()
      puts "Holiday Entry: "+@EntryType.to_s+" "
      case @EntryType
          when EntryType::Empty
              puts "Empty line \n";
          when EntryType::Comment
              puts "Comment: "+@Comment+"\n";
          when EntryType::Include
              puts "Include: "+@Include+"\n";
          when EntryType::Holiday
              puts "Holiday: "+@Name+", type: "+@HolidayType.to_s+"\n";
          else
              puts "UNKNOWN entry type \n";
      end
    end


end

