
=begin
  This is the class to parse a given string in the plan holiday format. 

  @short Plan Holiday Format
  @author Reinhold Kainhofer <reinhold@kainhofer.com>
  @version 0.1
=end


require 'holiday_entry.rb'

class Holiday_Format_Plan

    def initialize()
    end

    def interpretOredWDays( wdaystring )
      res = 0
      # Remove all leading and trailing spaces and the braces
      wdaystr = wdaystring.downcase.gsub( /^\s*(\(|)\s*(.*?)\s*(\)|)\s*$/, '\2' )
      wdays = wdaystr.split( /\s*\|\|\s*/ )
      wdays.each { |day| 
        res = res | ( 1 << wdayFromString(day) )
      }
      return res
    end

    def parseHolidayLine(line) 
      origline = line
      res = KHoliday_Entry.new();

      if ( line =~ /^\s*(small|)\s*(black|red|green|yellow|blue|magenta|cyan|white|weekend|)\s*"([^"]*)"\s*(black|red|green|yellow|blue|magenta|cyan|white|weekend|)\s*(on|)\s*([^\s][^\n]*)\n*$/i )
        res.EntryType = EntryType::Holiday
        
        color=$2;
        res.Name = $3;
        if ( $4 && $4.length > 0 )
          color=$4
        end
        if ( color == "weekend" )
          res.HolidayType = HolidayType::Holiday
        elsif ( color && color.length>0 )
          res.HolidayType = HolidayType::SpecialDay
        else 
          res.HolidayType = HolidayType::Information
        end
        line=$6;


        if ( line =~ /^\s*(.*?)\s*(|(plus|minus)\s*([0-9]+)\s*(|days?))\s*(|shift\s*(to)*\s*(.*)\s*if\s*([^\s]*|\(.*\)))\s*(|length\s*([0-9]*)\s*(|days?))\s*$/i )
          line = $1; # Remaining date 

          # Possible offset
          if ( $2.length()>0) 
            res.HasOffset = true
            res.Offset = $4.to_i
            if ( $3.downcase == "minus" )
              res.Offset = -res.Offset
            end
          end

          # Possible shift:
          if ( $6.length()>0 )
            res.HasShift = true
            res.ShiftTo = interpretOredWDays( $8 );
            res.ShiftFrom = interpretOredWDays( $9 );
          end

          # Possible length:
          if ( $10.length()>0 ) 
            res.HasLength = true;
            res.Length = $11.to_i
          end


          # Interpret the date part

          if ( line =~ /^\s*(monday|tuesday|wednesday|thursday|friday|saturday|sunday)\s*(before|after)\s*(.*)\s*$/i )
            res.HasWeekDay = true
            wday=wdayFromString( $1 )
            rel = $2
            if ( rel == "before" )
              wday = -wday
            end
            res.Weekday = wday
            line = $3
          end

          # line  now only contains the actual date spec
          if ( line =~ /^\s*easter\s*$/i )
            res.DateSpecial = SpecialDate::Easter
            res.DateBy = DateBy::Special
          elsif ( line =~ /^\s*([0-9]+)\.\s*([0-9]+)\.?(|([0-9]+))\s*$/i )
            # day.month.[year]
            res.DateBy = DateBy::Date
            res.DateDay = $1.to_i
            res.DateMonth = $2.to_i
            res.DateYear = ($4 && $4.length>0 )?($4.to_i):(-1)
          elsif ( line =~ /^\s*([0-9]+)\/\s*([0-9]+)(|\/([0-9]+))\s*$/i )
            # month/day[/year]
            res.DateBy = DateBy::Date
            res.DateMonth = $1.to_i
            res.DateDay = $2.to_i
            res.DateYear = ($4 && $4.length>0 )?($4.to_i):(-1)
          elsif ( line =~ /^\s*(january|february|march|april|may|june|july|august|september|october|november|december)\s*([0-9]+)\s*(|([0-9]+))\s*$/i )
            # monthname date [year]
            res.DateBy = DateBy::Date
            res.DateDay = $2.to_i
            res.DateMonth = monthFromString( $1 )
            res.DateYear = ($4 && $4.length>0 )?($4.to_i):(-1)
          elsif ( line =~ /^\s*([0-9]+)\.?\s*(january|february|march|april|may|june|july|august|september|october|november|december)\s*(|([0-9]+))\s*$/i )
            # date monthname [year]
            res.DateBy = DateBy::Date
            res.DateDay = $1.to_i
            res.DateMonth = monthFromString( $2 )
            res.DateYear = ($4 && $4.length>0 )?($4.to_i):(-1)
          elsif ( line =~ /^\s*(every|)\s*(|first|second|third|fourth|fifth|last|1st|2nd|3rd|4th|5th|any)\s*(monday|tuesday|wednesday|thursday|friday|saturday|sunday)\s*(|in)\s*(january|february|march|april|may|june|july|august|september|october|november|december)\s*$/i  )
            # [every] pos weekday [in] month
            res.DateBy = DateBy::MonthPos
            res.DatePos = posFromString( $2 )
# puts "POS: "+res.DatePos.to_s+ " for line "+line.to_s
            res.DateWDay = wdayFromString( $3 )
            res.DatePosMonth = monthFromString( $5 )
          else
puts "UNKNOWN holiday date spec... Preserving the line, "+origline+", line: "+line
            res.EntryType = EntryType::Unknown
            res.WholeString = origline
          end



        else
          res.EntryType = EntryType::Unknown
          res.WholeString = origline
        end

      else
        res.EntryType = EntryType::Unknown
        res.WholeString = origline
      end

      return res
    end

    def parseLine(line)
      res = KHoliday_Entry.new();
      if ( line.length == 0 || line =~ /^\s*$/ )
        res.EntryType =  EntryType::Empty
      elsif ( line =~ /^\s*#include\s*(.*)$/i )
        res.EntryType = EntryType::Include
        res.Include = line.gsub( /^\s*#include\s*(.*)\n*$/, '\1' )
      elsif ( line =~ /^\s*[#:](.*)$/ )
        res.EntryType = EntryType::Comment
        res.Comment = line.gsub( /^\s*[#:](.*)\n*$/, '\1' )
      else 
        res=parseHolidayLine( line )
      end

      return res
    end

    def openFile( file )
      n=0;
      @holidays = [];
      file = Qt::File.new(file);
      if ( file.open( Qt::IO_ReadOnly ) )
        stream = Qt::TextStream.new( file )
        while ( !stream.atEnd() )
            line = stream.readLine()
            if ( line )
              @holidays[n] = parseLine(line)
              n=n+1
            end
        end
        file.close()
      end

      return @holidays;
    end

    def fromString( contents )
      n=0;
      @holidays = [];
      for line in contents
        @holidays[n]=parseLine(line)
        n=n+1
      end
      return holidays;
    end

    def posFromString( pos )
      case pos.downcase
          when "first"; return 1
          when "1st"; return 1
          when "second"; return 2
          when "2nd"; return 2
          when "third"; return 3
          when "3nd"; return 3
          when "fourth"; return 4
          when "4th"; return 4
          when "fifth"; return 5
          when "5th"; return 5
          when "any"; return 0
          when "every"; return 0
          when "last"; return -1
          else return 0
      end
    end

    def wdayString( day )
      case day
          when Weekday::Monday;    return "monday"
          when Weekday::Tuesday;   return "tuesday"
          when Weekday::Wednesday; return "wednesday"
          when Weekday::Thursday;  return "thursday"
          when Weekday::Friday;    return "friday"
          when Weekday::Saturday;  return "saturday"
          when Weekday::Sunday;    return "sunday"
      end
      return ""
    end

    def wdayFromString( day )
      case day.downcase
          when "monday"; return Weekday::Monday
          when "tuesday"; return Weekday::Tuesday
          when "wednesday"; return Weekday::Wednesday
          when "thursday"; return Weekday::Thursday
          when "friday"; return Weekday::Friday
          when "saturday"; return Weekday::Saturday
          when "sunday"; return Weekday::Sunday
      end
      return 0
    end

    def monthName( month )
      case month
        when 1; return "january"
        when 2; return "february"
        when 3; return "march"
        when 4; return "april"
        when 5; return "may"
        when 6; return "june"
        when 7; return "july"
        when 8; return "august"
        when 9; return "september"
        when 10;return "october"
        when 11;return "november"
        when 12;return "december"
      end
    end

    def monthFromString( month )
      case month.downcase
        when "january"; return 1
        when "february"; return 2
        when "march"; return 3
        when "april"; return 4
        when "may"; return 5
        when "june"; return 6
        when "july"; return 7
        when "august"; return 8
        when "september"; return 9
        when "october"; return 10
        when "november"; return 11
        when "december"; return 12
      end
    end

    def entryToString( holiday )
# puts "Entry type: "+holiday.EntryType.to_s+" for "+ holiday.Name.to_s
      case holiday.EntryType
          when EntryType::Empty
              return "";
          when EntryType::Comment
              return ":" + holiday.Comment.to_s();
          when EntryType::Include
              return "#include \"" + holiday.Include.to_s()+ "\""
          when EntryType::Holiday
              str = "\"" + holiday.Name.to_s() + "\" ";
              case holiday.HolidayType
                when HolidayType::Holiday; str += "weekend ";
                when HolidayType::Information; str += "";
                else str += "blue ";
              end
              
              str += "on "

              if ( holiday.HasWeekDay )
                if ( holiday.Weekday < 0 )
                  str += wdayString( -holiday.Weekday ) + " before "
                else 
                  str += wdayString( holiday.Weekday ) + " after " 
                end
              end

              case holiday.DateBy
                  when DateBy::Date
                      str += holiday.DateDay.to_s + "." + holiday.DateMonth.to_s + "."
                      if ( holiday.DateYear>0 ) 
                        str += holiday.DateYear.to_s
                      end
                      str += " "
                  when DateBy::MonthPos
                      case holiday.DatePos
                        when 1
                          str += "1st "
                        when 2
                          str += "2nd "
                        when 3
                          str += "3rd "
                        when 4
                          str += "4th "
                        when 5
                          str += "5th "
                        when -1
                          str += "last "
                        when 0
                          str += "any "
                      end
                      str += wdayString( holiday.DateWDay ) + " in "+ monthName( holiday.DatePosMonth ) + " "
                  
                  when DateBy::Special
                      if ( holiday.DateSpecial == SpecialDate::Easter )
                        str += "easter "
                      else 
                        str += "UNKNOWN SPECIAL DATE "
                      end
              end
              

              if ( holiday.HasOffset )
                off = holiday.Offset
                if ( off<0 )
                  str += "minus "+(-off).to_s()
                else
                  str += "plus "+off.to_s();
                end
                if ( off.abs == 1 )
                  str += " day "
                else
                  str += " days "
                end
              
              end
              if ( holiday.HasShift )
                from = weekdayStringFromMask(holiday.ShiftFrom)
                to = weekdayStringFromMask(holiday.ShiftTo)
                if ( from.length>0 && to.length>0 )
                  str += "shift to " + to + " if " + from + " "
                end
              end
              if ( holiday.HasLength )
                str += "length "+holiday.Length.to_s()
                if ( holiday.Length == 1 )
                  str += " day "
                else
                  str += " days "
                end
              end
              return str
          when EntryType::Unknown:
              return holiday.WholeString
          else 
              return ""
      end
    end
    
    def weekdayStringFromMask( mask ) 
# puts "Mask: "+mask.to_s
      n = 0
      days = [];
      if ( (mask & (1 << Weekday::Monday) ) != 0 )
        days[n] = "monday"
        n = n + 1
      end
      if ( (mask & (1 << Weekday::Tuesday) ) != 0 )
        days[n] = "tuesday"
        n = n + 1
      end
      if ( (mask & (1 << Weekday::Wednesday) ) != 0 )
        days[n] = "wednesday"
        n = n + 1
      end
      if ( (mask & (1 << Weekday::Thursday) ) != 0 )
        days[n] = "thursday"
        n = n + 1
      end
      if ( (mask & (1 << Weekday::Friday) ) != 0 )
        days[n] = "friday"
        n = n + 1
      end
      if ( (mask & (1 << Weekday::Saturday) ) != 0 )
        days[n] = "saturday"
        n = n + 1
      end
      if ( (mask & (1 << Weekday::Sunday) ) != 0 )
        days[n] = "sunday"
        n = n + 1
      end
      res = days.join( "||" )
# puts "Res: "+res.to_s
      if ( n>1 ) 
        res = "( "+res+" )"
      end
# puts "Res1: "+res.to_s
      return res;
    end
    
    def toString( holidays )
      res="";
      holidays.each { |hday| 
        hdaystring = entryToString( hday );
# puts hdaystring.to_s()+"\n";
        res += hdaystring+"\n";
      }
      return res;
    end
    
    def saveFile( file, holidays )
# puts "===================================\n\n";
# puts res;
# puts "===================================\n\n";
# puts "url: "+file+"\n"
      file = Qt::File.new(file);
      if ( file.open( Qt::IO_WriteOnly ) )
        res = toString( holidays )
        stream = Qt::TextStream.new( file )
        stream << res
        file.close()
      end
    end

end
