
=begin
  This is the main view class for holiday_editor1.  Most of the non-menu,
  non-toolbar, and non-statusbar (e.g., non frame) GUI code should go
  here.

  This holiday_editor1 uses an HTML component as an example.

  @short Main view
  @author Reinhold Kainhofer <reinhold@kainhofer.com>
  @version 0.1
=end
require 'kholiday_editor_base.rb'
require 'holiday_format_plan.rb'
require 'holiday_entry.rb'

# FIXME: Before quitting, as for saving...
class KHoliday_Editor_View < HolidayEditor_Base

    #
    # Use this signal to change the content of the statusbar
    #
    signals 'signalChangeStatusbar(const QString&)'

    #
    # Use this signal to change the content of the caption
    signals 'signalChangeCaption(const QString&)'
    #
    # Use this signal to change the content modified indicator in the caption
    signals 'signalModified(bool)'
    
    slots 'slotOnURL(const QString&)',
          'slotSetTitle(const QString&)',
          'slotHolidaySelected(int)',
          'slotAddHoliday()', 'slotDelHoliday()',
          'slotHolidayUp()', 'slotHolidayDown()',
          'slotNameChanged( const QString & )', 'slotChanged()'


    def initialize(parent)
        super(parent)
        @holidays = []
        @curHDay = -1;
        @Url= ""
        @dirty = false
        format = holidayFormat()

        # keep a reference to the DCOP Interface so it doesn't get gc'd
        @dcop = KHoliday_Editor_Iface.new(self)
        connect(mHolidayList,SIGNAL('highlighted(int)'), self,   SLOT('slotHolidaySelected(int)')) 
        connect(mAddHoliday, SIGNAL('clicked()'), self, SLOT('slotAddHoliday()') )
        connect(mDelHoliday, SIGNAL('clicked()'), self, SLOT('slotDelHoliday()') )
        connect(mHolidayUp,  SIGNAL('clicked()'), self, SLOT('slotHolidayUp()') )
        connect(mHolidayDown,SIGNAL('clicked()'), self, SLOT('slotHolidayDown()') )
        connect(mHolidayName,SIGNAL('textChanged(const QString &)'), 
                self,        SLOT('slotNameChanged(const QString &)') )


        connect( mTypeHoliday, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mHolidayClass, SIGNAL('activated(int)'), self, SLOT('slotChanged()') )


        connect( mDateDate, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mDateDay, SIGNAL('valueChanged(int)'), self, SLOT('slotChanged()') )
        connect( mDateDayMonth, SIGNAL('activated(int)'), self, SLOT('slotChanged()') )
        connect( mDateYearCheck, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mDateYear, SIGNAL('valueChanged(int)'), self, SLOT('slotChanged()') )

        connect( mDateMonth, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mDateMonthDay, SIGNAL('activated(int)'), self, SLOT('slotChanged()') )
        connect( mDateMonthWeekday, SIGNAL('activated(int)'), self, SLOT('slotChanged()') )
        connect( mDateMonthMonth, SIGNAL('activated(int)'), self, SLOT('slotChanged()') )

        connect( mDateSpecialCheck, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mDateSpecial, SIGNAL('activated(int)'), self, SLOT('slotChanged()') )

        connect( mWeekdayCheck, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mWeekday, SIGNAL('activated(int)'), self, SLOT('slotChanged()') )
        connect( mWeekdayRelation, SIGNAL('activated(int)'), self, SLOT('slotChanged()') )


        connect( mHasOffset, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mOffset, SIGNAL('valueChanged(int)'), self, SLOT('slotChanged()') )

        connect( mHasLength, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mLength, SIGNAL('valueChanged(int)'), self, SLOT('slotChanged()') )

        connect( mHasShift, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mShiftToDay, SIGNAL('activated(int)'), self, SLOT('slotChanged()') )
        connect( mMoveFromMo, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mMoveFromTu, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mMoveFromWe, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mMoveFromTh, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mMoveFromFr, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mMoveFromSa, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mMoveFromSu, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )




        connect( mTypeComment, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mComment, SIGNAL('textChanged(const QString&)'), self, SLOT('slotChanged()') )
        connect( mTypeInclude, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mInclude, SIGNAL('textChanged(const QString&)'), self, SLOT('slotChanged()') )
        connect( mTypeEmpty, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mTypeUnknown, SIGNAL('toggled(bool)'), self, SLOT('slotChanged()') )
        connect( mUnparsableString, SIGNAL('textChanged(const QString&)'), self, SLOT('slotChanged()') )


        initializeUI()
    end


    def print(p, height, width)
        # do the actual printing, here
        # p.drawText(etc..)
        p.drawText 
        
    end

    def currentURL()
        return @Url
    end
     
    def holidayFormat()
       return Holiday_Format_Plan.new()
    end

    def openURL(url)
       @Url = url
       format = holidayFormat()
       @holidays = format.openFile( url )
       initializeUI()
       mHolidayList.setSelected( 0, true )
       slotHolidaySelected( 0 )
       unsetDirty()
    end
    
    def initializeUI() 
      mHolidayList.clear()
      mEntryEditGroup.setEnabled( @holidays.length>0 )
    
       @holidays.each {|hday|
#           hday.dump 
          addListBoxEntry(hday, -1)
        }
        emit signalChangeCaption(@Url)
    end
    
    def saveDefaultURL()
      if ( @Url.length <= 0 ) 
        @Url = KDE::FileDialog::getSaveFileName( "", "", self )
      end
      saveURL( currentURL() )
      unsetDirty()
    end
    
    def saveURL(url)
      format = holidayFormat()
      format.saveFile( url, @holidays );
      @Url = url
      slotOnURL( @Url )
      unsetDirty()
      emit signalChangeCaption(@Url)
    end

    def slotOnURL(url)
      emit signalChangeStatusbar(url)
    end

    def slotSetTitle(title)
      emit signalChangeCaption(title)
    end

    def listBoxEntryName( holiday ) 
      case holiday.EntryType 
        when EntryType::Empty
          return "-"
        when EntryType::Comment
          return "C: "+holiday.Comment
        when EntryType::Include
          return "I: " + holiday.Include
        when EntryType::Holiday
          return holiday.Name
        else
          return "UNKNOWN "
      end
    end

    def addListBoxEntry( holiday, pos )
      mHolidayList.insertItem( listBoxEntryName(holiday), pos );
    end

    def hasBitSet( field, bit )
      return ( field & (1 << bit) ) != 0
    end

    def addBit( field, bit, value ) 
      if ( value ) 
        field = field | ( 1 << bit )
      end
      return field
    end

    def setHolidayData( holiday )
      mEntryEditGroup.setEnabled( true )
      case holiday.EntryType
        when EntryType::Empty
          mTypeEmpty.setChecked( true )
        when EntryType::Comment
          mTypeComment.setChecked( true )
          mComment.setText( holiday.Comment )
        when EntryType::Include
          mTypeInclude.setChecked( true )
          mInclude.setText( holiday.Include )
        when EntryType::Holiday
          mTypeHoliday.setChecked( true )
          mHolidayName.setText( holiday.Name )

          case holiday.DateBy
            when DateBy::Date
              mDateDate.setChecked( true )
              mDateDay.setValue( holiday.DateDay )
              mDateDayMonth.setCurrentItem( holiday.DateMonth - 1 )
              mDateYearCheck.setChecked( holiday.DateYear > 0 )
              if ( holiday.DateYear>0 ) 
                mDateYear.setValue( holiday.DateYear )
              end
            when DateBy::MonthPos
              mDateMonth.setChecked( true )
              if ( holiday.DatePos < 0 )
                mDateMonthDay.setCurrentItem( 5 ) # Last
              elsif ( holiday.DatePos == 0 )
                mDateMonthDay.setCurrentItem( 6 ) # Every
              else 
                mDateMonthDay.setCurrentItem( holiday.DatePos-1 )
              end
              mDateMonthWeekday.setCurrentItem( holiday.DateWDay-1 )
              mDateMonthMonth.setCurrentItem( holiday.DatePosMonth - 1 )

            when DateBy::Special
              mDateSpecialCheck.setChecked( true )
              mDateSpecial.setCurrentItem( holiday.DateSpecial - 1 )
          end

          mWeekdayCheck.setChecked( holiday.HasWeekDay )
          if ( holiday.Weekday < 0 )
            mWeekdayRelation.setCurrentItem( 0 )
            mWeekday.setCurrentItem( -holiday.Weekday-1 )
          else
            mWeekdayRelation.setCurrentItem( 1 )
            mWeekday.setCurrentItem( holiday.Weekday-1 )
          end


          mHasOffset.setChecked( holiday.HasOffset )
          if ( holiday.HasOffset )
            mOffset.setValue( holiday.Offset )
          end

          mHasLength.setChecked( holiday.HasLength )
          if ( holiday.HasLength )
            mLength.setValue( holiday.Length )
          end

          mHasShift.setChecked( holiday.HasShift )
          if ( holiday.HasShift )
            if ( hasBitSet(holiday.ShiftTo, Weekday::Monday) )
              mShiftToDay.setCurrentItem( 0 )
            elsif ( hasBitSet(holiday.ShiftTo, Weekday::Tuesday) )
              mShiftToDay.setCurrentItem( 1 )
            elsif ( hasBitSet(holiday.ShiftTo, Weekday::Wednesday) )
              mShiftToDay.setCurrentItem( 2 )
            elsif ( hasBitSet(holiday.ShiftTo, Weekday::Thursday) )
              mShiftToDay.setCurrentItem( 3 )
            elsif ( hasBitSet(holiday.ShiftTo, Weekday::Friday) )
              mShiftToDay.setCurrentItem( 4 )
            elsif ( hasBitSet(holiday.ShiftTo, Weekday::Saturday) )
              mShiftToDay.setCurrentItem( 5 )
            elsif ( hasBitSet(holiday.ShiftTo, Weekday::Sunday) )
              mShiftToDay.setCurrentItem( 6 )
            else
              mShiftToDay.setCurrentItem( 0 )
            end
            mMoveFromMo.setChecked( hasBitSet(holiday.ShiftFrom, Weekday::Monday) )
            mMoveFromTu.setChecked( hasBitSet(holiday.ShiftFrom, Weekday::Tuesday) )
            mMoveFromWe.setChecked( hasBitSet(holiday.ShiftFrom, Weekday::Wednesday) )
            mMoveFromTh.setChecked( hasBitSet(holiday.ShiftFrom, Weekday::Thursday) )
            mMoveFromFr.setChecked( hasBitSet(holiday.ShiftFrom, Weekday::Friday) )
            mMoveFromSa.setChecked( hasBitSet(holiday.ShiftFrom, Weekday::Saturday) )
            mMoveFromSu.setChecked( hasBitSet(holiday.ShiftFrom, Weekday::Sunday) )
          end

        when EntryType::Unknown
          mTypeUnknown.setChecked( true )
          mUnparsableString.setText( holiday.Include )

        else
          mEntryEditGroup.setDisabled( true )
      end
    end

    def slotHolidaySelected( pos )
      # prevent the name and the other settings from triggering assignment 
      # while we are updating all controls with the new values
      @processingNewSelection = true
      if ( !@assigningFromControls ) 
        if ( 0 <= pos && pos < @holidays.length() )
          setHolidayData( @holidays[pos] );
          @curHDay = pos;
        else
          @curHDay = -1;
          mEntryEditGroup.setEnabled( false )
        end
      end
      enableButtons()
      @processingNewSelection = false
    end

    def enableButtons()
      mHolidayUp.setEnabled( @curHDay > 0 )
      mHolidayDown.setEnabled( @curHDay >=0 && @curHDay < @holidays.length()-1 )
      mAddHoliday.setEnabled( true )
      mDelHoliday.setEnabled( @holidays.length() > 0 && @curHDay >= 0 && @curHDay < @holidays.length() )
    end
    
    def slotAddHoliday()
      hday = KHoliday_Entry.new()
      pos = @curHDay;
      if ( pos >= 0 ) 
        @holidays.insert( pos, hday )
        addListBoxEntry( hday, pos )
      else 
        @holidays << hday
        addListBoxEntry( hday, -1 )
        @curHDay = 0
      end
      mHolidayList.setSelected( @curHDay, true )
      slotHolidaySelected( @curHDay )
      setDirty()
    end

    def slotDelHoliday()
      if ( @curHDay >=0 && @curHDay < @holidays.length() ) 
        hday = @holidays[@curHDay];
        
        if ( hday && KDE::MessageBox::questionYesNo(nil, i18n("<qt>Do you really want to delete the following entry at position "+@curHDay.to_s()+":<br>"+@holidays[@curHDay].shortText()+ "</qt>") ) == KDE::MessageBox::Yes ) 
          @holidays.delete_at(@curHDay);
          mHolidayList.removeItem(@curHDay);
#           if ( mHolidayList.
          if ( @curHDay >= @holidays.length() )
            @curHDay = @holidays.length()-1
          end
        mHolidayList.setSelected( @curHDay, false )
        mHolidayList.setSelected( @curHDay, true )
          slotHolidaySelected( @curHDay )
        end
        setDirty()
      end
    end
    
    def slotHolidayUp()
      # @curHDay will be reset by takeItem if we are removing the last item => store its current value
      cur = @curHDay
      if ( cur >0 && cur < @holidays.length() )
        hday = @holidays[cur]
        @holidays[cur] = @holidays[cur-1]
        @holidays[cur-1] = hday
        item = mHolidayList.item( cur )
        mHolidayList.takeItem( item )
        mHolidayList.insertItem( item , cur - 1 )
        @curHDay = cur - 1
        mHolidayList.setSelected( @curHDay, false )
        mHolidayList.setSelected( @curHDay, true )
        setDirty()
      end
    end
    
    def slotHolidayDown()
      if ( @curHDay >=0 && @curHDay < @holidays.length()-1 )
        hday = @holidays[@curHDay]
        @holidays[@curHDay] = @holidays[@curHDay+1]
        @holidays[@curHDay+1] = hday
        item = mHolidayList.item( @curHDay )
        mHolidayList.takeItem( item )
        mHolidayList.insertItem( item , @curHDay+1 )
        @curHDay = @curHDay + 1
        mHolidayList.setSelected( @curHDay, false )
        mHolidayList.setSelected( @curHDay, true )
        setDirty()
      end
    end
    
    def isDirty()
      return @dirty
    end

    def setDirty()
      @dirty = true
      emit signalModified(@dirty)
    end
    
    def unsetDirty()
      @dirty = false
      emit signalModified(@dirty)
    end
    
    def slotNameChanged( newname )
      if ( !@processingNewSelection )
        pos = @curHDay
        setDirty()
        if ( pos >=0 && pos < @holidays.length() )
          @assigningFromControls = true
          @holidays[pos].Name=newname
          mHolidayList.changeItem( listBoxEntryName(@holidays[pos]), pos )
          @assigningFromControls = false
        end
      end
    end
    
    def slotChanged()
      if ( !@processingNewSelection )
        assignValuesFromControls( @curHDay )
        setDirty()
      end
    end
    
    def assignValuesFromControls( pos )
      @assigningFromControls = true
      if ( pos >=0 && pos < @holidays.length() )
        hday = @holidays[pos]
        if ( mTypeHoliday.isChecked() ) 
          hday.EntryType = EntryType::Holiday

          if ( mDateDate.isChecked() )
            hday.DateBy = DateBy::Date
            hday.DateDay = mDateDay.value()
            hday.DateMonth = mDateDayMonth.currentItem()+1
            if ( mDateYearCheck.isChecked() )
              hday.DateYear = mDateYear.value()
            else
              hday.DateYear = -1
            end

          elsif ( mDateMonth.isChecked() )
            hday.DateBy = DateBy::MonthPos
            hday.DatePos = mDateMonthDay.currentItem() + 1
            if ( mDateMonthWeekday.currentItem() == 5 )
              hday.DateWDay = -1 # last
            elsif ( mDateMonthWeekday.currentItem() == 6 )
              hday.DateWDay = 0 # every
            else 
              hday.DateWDay = mDateMonthWeekday.currentItem() + 1
            end
            hday.DatePosMonth = mDateMonthMonth.currentItem() + 1

          elsif ( mDateSpecialCheck.isChecked() )
            hday.DateBy = DateBy::Special
            hday.DateSpecial = mDateSpecial.currentItem() + 1

          end

          hday.HasWeekDay = mWeekdayCheck.isChecked()
          if ( mWeekdayCheck.isChecked() )
            day = mWeekday.currentItem() + 1
            if ( mWeekdayRelation.currentItem() == 0 ) 
              day = -day
            end
            hday.Weekday = day
          end

          hday.HasOffset = mHasOffset.isChecked();
          if ( mHasOffset.isChecked() )
            hday.Offset = mOffset.value()
          end
          hday.HasLength = mHasLength.isChecked()
          if ( mHasLength.isChecked() )
            hday.Length = mLength.value()
          end
          hday.HasShift = mHasShift.isChecked()
          if ( mHasShift.isChecked() )
            hday.ShiftTo = 1 << (mShiftToDay.currentItem()+1)
puts "Shift to: "+hday.ShiftTo.to_s
            from = 0;
            from = addBit( from, Weekday::Monday, mMoveFromMo.isChecked() )
            from = addBit( from, Weekday::Tuesday, mMoveFromTu.isChecked() )
            from = addBit( from, Weekday::Wednesday, mMoveFromWe.isChecked() )
            from = addBit( from, Weekday::Thursday, mMoveFromTh.isChecked() )
            from = addBit( from, Weekday::Friday, mMoveFromFr.isChecked() )
            from = addBit( from, Weekday::Saturday, mMoveFromSa.isChecked() )
            from = addBit( from, Weekday::Sunday, mMoveFromSu.isChecked() )
            from = hday.ShiftFrom = from
puts "Shift from: "+hday.ShiftFrom.to_s
          end
          
        elsif ( mTypeComment.isChecked() )
          hday.EntryType = EntryType::Comment
          hday.Comment = mComment.text()
        elsif ( mTypeInclude.isChecked() )
          hday.EntryType = EntryType::Include
          hday.Include = mInclude.text()
        elsif ( mTypeEmpty.isChecked() )
          hday.EntryType = EntryType::Empty
        else
          hday.EntryType = EntryType::Unknown
          hday.WholeString = mUnparsableString.text()
        end
        mHolidayList.changeItem( listBoxEntryName(@holidays[pos]), pos )
      end
      @assigningFromControls = false
    end

end
