/*
    KIckPim

    Copyright (c) 2002      by Bert Speckels <bert.speckels@web.de>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include <string>
// #include <iostream>
// #include <fstream>

// #include <qlayout.h>
// #include <qlabel.h>

#include <kcrash.h>
#include <kdebug.h>
#include <klocale.h>
//#include <kmessagebox.h>

#include "kickpim.h"
#include "kickpimwidget.h"

using namespace std;

KickPimRepository* KickPim::s_repository = 0;

extern "C"
{
  KPanelApplet* init( QWidget *parent, const QString& configFile)
  {
    KGlobal::locale()->insertCatalogue("kickpim");
    return new KickPim(configFile, KPanelApplet::Normal,
                      KPanelApplet::About/* | KPanelApplet::Help | KPanelApplet::Preferences*/,
                      parent, "KickPim-Applet");
  }
}

void KickPimMessageHandler( QtMsgType type, const char *msg )
{
  #ifdef DEBUG_OUTPUT
  switch ( type )
  {
    case QtDebugMsg:
      break;
    case QtWarningMsg:
      if (strstr(msg,"connect")>0)
        kdWarning() << "  QT(Warn): " << msg << endl;
      break;
    case QtFatalMsg:
      kdFatal() << "  QT(Fatal): " << msg << endl;
      break;
  }
  #endif
}

KickPim::KickPim(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)
       : KPanelApplet(configFile, type, actions, parent, name)
{
  KCrash::setCrashHandler( crashHandler );
  setAcceptDrops(true);

  kdDebug() << "KickPim - Version " << VERSION << endl << endl;

  qInstallMsgHandler(KickPimMessageHandler);

  // setBackgroundMode(X11ParentRelative);

  // Create Repository-Object early, but initialize its data later ...
  if (s_repository!=0) delete s_repository;
  s_repository = new KickPimRepository(this);

  // main widget
  m_kickPimWidget = new KickPimWidget( this , "KickPim");

  // create icon
  m_iconKickPim = new QPixmap( KickPim::rep()->dirOfIcons()+"hi32-kickpim-icon.png" );
  setIcon( *m_iconKickPim );

  // create global keys before initializing repository data (i.e. loading the configuration)
  opt()->globalAccel->insert("MenuToggle",      i18n("Menu Toggle"),              i18n("Toggles the KickPim menu"),                0, 0, m_kickPimWidget, SLOT(toggleMenu()));
  opt()->globalAccel->insert("MenuShow",        i18n("Menu: Show"),               i18n("Displays the KickPim menu"),              0, 0, m_kickPimWidget, SLOT(showMenu()));
  opt()->globalAccel->insert("MenuHide",        i18n("Menu: Hide"),               i18n("Hides the KickPim menu"),                 0, 0, m_kickPimWidget, SLOT(hideMenu()));
  opt()->globalAccel->insert("ContactAdd",      i18n("Contacts: Add Contact"),    i18n("Adds a new contact to the addressbook"),  0, 0, m_kickPimWidget, SLOT(addContact()));
  opt()->globalAccel->insert("MailCheck",       i18n("Emails: Check Now"),        i18n("Checks the mails accounts."),             0, 0, m_kickPimWidget, SLOT(checkMailAccounts()));
  opt()->globalAccel->insert("MailCheckToggle", i18n("Emails: Toggle Autocheck"), i18n("Toggles automatic mail polling."),        0, 0, m_kickPimWidget, SLOT(toggleMailCheck()));
  opt()->load();

  m_kickPimWidget->init();

  LogService::setLogCategoryFlag ( opt()->logCategories );
  LogService::setLogLevel        ( opt()->logLevel      );

  s_repository->initData();
  kdDebug() "Repo data initialized" << endl;
  setAcceptDrops(TRUE);
}

KickPim::~KickPim()
{
  kdDebug() << endl;

  delete m_iconKickPim;   m_iconKickPim   = 0;
  delete m_kickPimWidget; m_kickPimWidget = 0;

  delete s_repository;    s_repository    = 0;
}

/*
bool KickPim::process(const QCString &fun, const QByteArray &data, QCString &replyType, QByteArray &replyData)
{
  cout << "DCOP-Message: "+fun << endl;

  if (fun == "help()")
  {
    QString result="functions:\n"
                   "  show: display the KickPim menu\n"
                   "  hide: hide the KickPim menu\n";
    QDataStream reply(replyData, IO_WriteOnly);
    reply << result;
    replyType = "QString";
    return true;
  }
  else if (fun == "show()")
  {
    QString result="KickPim menu is already visible";
    if (setMenuVisible(true)) result="KickPim menu displayed!";
    QDataStream reply(replyData, IO_WriteOnly);
    reply << result;
    replyType = "QString";
    return true;
  }
  else if (fun == "hide()")
  {
    QString result="KickPim menu is not visible";
    if (setMenuVisible(false)) result="KickPim menu hidden!";
    QDataStream reply(replyData, IO_WriteOnly);
    reply << result;
    replyType = "QString";
    return true;
  }
  else if (fun == "toggle()")
  {
    QString result="KickPim menu toggled: now visible";

    if (!setMenuVisible(true))
    {
      setMenuVisible(false);
      result="KickPim menu toggled: now hidden";
    }

    QDataStream reply(replyData, IO_WriteOnly);
    reply << result;
    replyType = "QString";
    return true;
  }
  else
  {
    qDebug("unknown function call to KickPim::process()");
    return false;
  }
}
*/

bool  KickPim::close( bool alsoDelete )
{
  LogService::call("KickPim","close");
  
  delete m_iconKickPim;   m_iconKickPim   = 0;
  delete m_kickPimWidget; m_kickPimWidget = 0;
  delete s_repository;    s_repository    = 0;

  kdDebug() << endl;

  return KPanelApplet::close( alsoDelete );
}

void KickPim::about()
{
  if (m_kickPimWidget) m_kickPimWidget->displayAboutDialog();
}

void KickPim::help()
{
   //KMessageBox::information(0, i18n("This is a help box"));
}

void KickPim::preferences()
{
   //KMessageBox::information(0, i18n("This is a preferences box"));
}

int KickPim::widthForHeight( int /*height*/ ) const
{
  return 24;
  // return (height > 40) ? 24 : 42;
}

int KickPim::heightForWidth( int /*width*/ ) const
{
   return 24;  
   // return (width > 40) ? 24 : 42;  
}

bool KickPim::isVertical() const
{
   return orientation() == Qt::Vertical;
}


void KickPim::resizeEvent( QResizeEvent*)
{
  if (m_kickPimWidget)
  {
    m_kickPimWidget->setFixedSize(width(),height() );
  }
}

void KickPim::crashHandler( int signal ) {
  fprintf(stderr,"KickPim: Crash :-[ \n");
  rep()->emergencySave();

  KCrash::defaultCrashHandler( signal );
}

#include "kickpim.moc"
