/*
    KickPimContactViewItem

    Copyright (c) 2002      by Bert Speckels <bert@speckels.de>
    Copyright (c) 2005      by Duncan Mac-Vicar P. <duncan@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include <qpainter.h>
#include <kdebug.h>
#include "kpcontact.h"
#include "kickpim.h"
#include "kickpimoptions.h"
#include "kickpimcontactview.h"
#include "kickpimcontactviewitem.h"
#include "kickpimlistviewitem.h"

class KickPimContactViewItem::Private
{
public:
  Private() : contactPhoto( 0L ), contactIcon( 0L ), nameText( 0L ), extraText( 0L ), photoSize(KickPim::opt()->photoSize) {}
  KickPimListViewImageComponent *contactPhoto;
  KickPimListViewImageComponent *contactIcon;
  KickPimListViewTextComponent *nameText;
  KickPimListViewTextComponent *extraText;
  KickPimListViewBoxComponent *spacerBox;
  //std::auto_ptr<KickPimListViewItemToolTipSource> toolTipSource;
  int photoSize;
  KPContact*        contact;
  DistributionList* distList; 
  QString           label; 
};

KickPimContactViewItem::KickPimContactViewItem( QListView* parent, QListViewItem* after, QString label )
    : KickPimListViewItem(parent,after)
{
  initLVI();
  d->label = label;
}

void KickPimContactViewItem::initLVI()
{
 d = new Private;
}

KickPimContactViewItem::~KickPimContactViewItem()
{
  initLVI();
}

void KickPimContactViewItem::setContact(KPContact* contact)
{
  d->contact  = contact;
  d->distList = 0;
}

QString KickPimContactViewItem::key( int columnt, bool ascending) const
{
  switch (KickPim::opt()->sortContactsBy)
  {
    case KickPimOptions::ADDR_SORT_FAMILYNAME:
      return d->contact->names()["familyName"];
    break;
    case KickPimOptions::ADDR_SORT_NAME:
      return d->contact->names()["givenName"];
    break;
  }
  return KickPimListViewItem::key(columnt, ascending);
}

QString KickPimContactViewItem::text( int /*column*/ ) const
{
  return d->nameText->text();
}

void KickPimContactViewItem::setDistributionList(DistributionList* list)
{
  d->distList = list;
}

KPContact* KickPimContactViewItem::contact()
{
  return d->contact;
}

DistributionList* KickPimContactViewItem::distributionList()
{
  return d->distList;
}

bool KickPimContactViewItem::isDistributionList()
{
  return d->distList!=0;
}

void KickPimContactViewItem::setDisplayMode()
{
  while ( component( 0 ) )
    delete component( 0 );

  d->nameText = 0L;
  d->contactPhoto = 0L;
  d->extraText = 0L;
  //d->buddyIcon = 0L;
  //d->iconSize = IconSize( KIcon::Small );
  d->photoSize =  KickPim::opt()->photoSize;

  KickPimListViewComponent *hbox = new KickPimListViewBoxComponent( this, KickPimListViewBoxComponent::Horizontal );
  //hbox->setMinWidth(300);
  //d->spacerBox = new KickPimListViewBoxComponent( hbox, KickPimListViewBoxComponent::Horizontal )
  
  // set the photo for the item 
  if ( KickPim::opt()->showContactPhoto )
  {
    KickPimListViewComponent *imageBox = new KickPimListViewBoxComponent( hbox, KickPimListViewBoxComponent::Vertical );
    new KickPimListViewVSpacerComponent( imageBox );
    // include borders in size
    d->contactPhoto = new KickPimListViewImageComponent( imageBox, d->photoSize, d->photoSize );
    new KickPimListViewVSpacerComponent( imageBox );
    
    new KickPimListViewHSpacerComponent( hbox );
    KickPimListViewComponent *vbox = new KickPimListViewBoxComponent( hbox, KickPimListViewBoxComponent::Vertical );
    d->nameText = new KickPimListViewTextComponent( vbox );
    d->extraText = new KickPimListViewTextComponent( vbox );
    new KickPimListViewHSpacerComponent( hbox );
    
  }
  else
  {
    KickPimListViewComponent *vbox = new KickPimListViewBoxComponent( hbox, KickPimListViewBoxComponent::Vertical );
    d->nameText = new KickPimListViewTextComponent( vbox );
    d->extraText = new KickPimListViewTextComponent( vbox );
  }
  slotNameChanged();
  slotPhotoChanged();
}

void KickPimContactViewItem::slotPhotoChanged()
{
  if ( d->contactPhoto )
  {
    QPixmap photoPixmap;
    //QPixmap defaultIcon( KGlobal::iconLoader()->loadIcon( "vcard", KIcon::Desktop ) );
    QImage photoImg = d->contact->photo();
    if ( (photoImg.width() > 0) &&  (photoImg.height() > 0) )
    {
      int photoSize = d->photoSize;
      float ratio = float(photoImg.width())/float(photoImg.height());
      int newsize =int(float(photoSize*ratio));
      if ( photoImg.width() > photoImg.height() )
      {
        photoPixmap = photoImg.smoothScale( photoSize, newsize );
      }
      else
      {
        photoPixmap = photoImg.smoothScale( newsize, photoSize );
      }
      QPainter p(&photoPixmap);
      p.setPen(Qt::black);
      p.drawLine(0, 0, photoPixmap.width()-1, 0);
      p.drawLine(0, photoPixmap.height()-1, photoPixmap.width()-1, photoPixmap.height()-1);
      p.drawLine(0, 0, 0, photoPixmap.height()-1);
      p.drawLine(photoPixmap.width()-1, 0, photoPixmap.width()-1, photoPixmap.height()-1);
      d->contactPhoto->setPixmap( photoPixmap, false);
    }
  }
}

void KickPimContactViewItem::slotNameChanged()
{
 if ( d->nameText )
 {
   d->nameText->setText( d->label);
   //d->extraText->setText( d->label);
  // delay the sort if we can
  //if ( KickPimContactView *lv = dynamic_cast<KickPimContactView *>( listView() ) )
  //  lv->delayedSort();
  //else
   listView()->sort();
 }
}


