/*
    List View providing extra support for ListView::Items

    Copyright (c) 2004      by Richard Smith <kde@metafoo.co.uk>

    Kopete    (c) 2004      by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef KICKPIM_LISTVIEW_H
#define KICKPIM_LISTVIEW_H

#include <klistview.h>

/**
 * @author Richard Smith <kde@metafoo.co.uk>
 */
class KickPimListView : public KListView
{
	Q_OBJECT

public:
	KickPimListView( QWidget *parent = 0, const char *name = 0 );
	~KickPimListView();

	/**
	 * Schedule a delayed sort operation. Sorts will be withheld for at most
	 * half a second, after which they will be performed. This way multiple
	 * sort calls can be safely bundled without writing complex code to avoid
	 * the sorts entirely.
	 */
	void delayedSort();

	/**
	 * Set whether to show the lines and +/- boxes in the tree
	 */
	void setShowTreeLines( bool bShowAsTree );

public slots:
	/**
	 * Calls QListView::sort()
	 */
	void slotSort() { sort(); }

protected:
	virtual void keyPressEvent( QKeyEvent *e );

private slots:
	void slotContextMenu(KListView*,QListViewItem *item, const QPoint &point );
	void slotDoubleClicked( QListViewItem *item );

private:
	struct Private;
	Private *d;
};

#endif

// vim: set noet ts=4 sts=4 sw=4:
