/*
    KPKABContactReader

    Copyright (c) 2003      by Bert Speckels <bert.speckels@web.de>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#include <kdebug.h>
#include "kickpim.h"
#include "kpkabcontact.h"
 
#include "kpkabcontactreader.h"

KPKabContactReader::KPKabContactReader()
{
  kdDebug() << endl;
  m_addrBook = StdAddressBook::self(true);
  connect( m_addrBook, SIGNAL( addressBookChanged  (AddressBook*) ),
           this,       SLOT  ( slotAddressBookChanged(AddressBook*) ) );
}

KPKabContactReader::~KPKabContactReader()
{
  kdDebug() << "Skipping an empty addressee" << endl;
  disconnect( this, SLOT  ( slotAddressBookChanged(AddressBook*) ) );
  m_addrBook = 0;
}

void KPKabContactReader::reload()
{
  kdDebug() << endl;

  m_addrBook->load();
  // tell everybody that contacts are ready for display
  slotAddressBookChanged(m_addrBook);
}

void KPKabContactReader::asyncReload()
{
  kdDebug() "requesting async load" << endl;
  m_addrBook->asyncLoad();
  kdDebug() "async load requested" << endl;
}

KPContact* KPKabContactReader::first()
{
  if (!m_addrBook) return 0;
  
  m_itor = m_addrBook->begin();

  // skip empty addresses
  while (m_itor!=m_addrBook->end() && ((Addressee)*m_itor).isEmpty())
  {
    kdDebug() << "Skipping an empty addressee" << endl;
    ++m_itor;
  }
  if (m_itor==m_addrBook->end()) return 0;  

  KPContact* contact = createContact( m_itor );
  m_itor++;
  return contact;
}

KPContact* KPKabContactReader::createContact( AddressBook::Iterator& itor )
{  
  if (itor == m_addrBook->end()) return 0;
        
  Addressee& addr = *itor;
  if (addr.isEmpty()) return 0; // shouldn't happen!
  
  KPKabContact* contact = new KPKabContact(addr);
  return contact;
}
  
KPContact* KPKabContactReader::next()
{
  if (!m_addrBook) return 0;

  // skip empty addresses
  while (m_itor!=m_addrBook->end() && ((Addressee)*m_itor).isEmpty())
  {
    kdDebug() << "Skipping an empty addressee" << endl;
    ++m_itor;
  }
  if (m_itor==m_addrBook->end()) return 0;

  KPContact* contact = createContact( m_itor );
  m_itor++;
  return contact;
}

void KPKabContactReader::slotAddressBookChanged ( AddressBook* book )
{
  kdDebug() << endl;
  m_itor = m_addrBook->begin();
  emit contactsChanged(this);
}

#include "kpkabcontactreader.moc"
