/*
    This file is part of libqopensync.

    Copyright (c) 2005 Tobias Koenig <tokoe@kde.org>
    Copyright (c) 2007 Daniel Gollub <dgollub@suse.de>
    Copyright (c) 2010 Quentin Denis <quentin@links2linux.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include <opensync/opensync.h>
#include <opensync/opensync-plugin.h>

#include "plugin.h"
#include "result.h"

#include "pluginenv.h"

using namespace QSync;

PluginEnv::PluginEnv()
{
  OSyncError *error = 0;
  mPluginEnv = osync_plugin_env_new( &error );
}

PluginEnv::~PluginEnv()
{
  osync_plugin_env_unref( mPluginEnv );
}

Result PluginEnv::initialize()
{
  OSyncError *error = 0;
  if ( !osync_plugin_env_load( mPluginEnv, NULL, &error ) )
    return Result( &error );
  else
    return Result();
}

Result PluginEnv::finalize()
{
  osync_plugin_env_unref( mPluginEnv );
  return Result();
}

int PluginEnv::pluginCount() const
{
  OSyncList *p, *plugins = osync_plugin_env_get_plugins( mPluginEnv );
  QList<void *> pluginlist;
  
  for (p=plugins; p; p = p->next) {
    pluginlist.append(p->data);
  }

  return pluginlist.length();
}

Plugin PluginEnv::pluginAt( int pos ) const
{
  Plugin plugin;
  
  OSyncList *p, *plugins = osync_plugin_env_get_plugins( mPluginEnv );
  QList<void *> pluginlist;
  
  for (p=plugins; p; p = p->next) {
    pluginlist.append(p->data);
  }

  if ( pos < 0 || pos >= pluginCount() )
    return plugin;

  OSyncPlugin *oplugin = static_cast<OSyncPlugin *>(pluginlist[ pos ]);
  plugin.mPlugin = oplugin;

  return plugin;
}

Plugin PluginEnv::pluginByName( const QString &name ) const
{
  Plugin plugin;

  OSyncPlugin *oplugin = osync_plugin_env_find_plugin( mPluginEnv, name.toLatin1() );
  if ( oplugin )
    plugin.mPlugin = oplugin;

  return plugin;
}
