/*
    This file is part of libqopensync.

    Copyright (c) 2005 Tobias Koenig <tokoe@kde.org>
    Copyright (c) 2010 Quentin Denis <quentin@links2linux.de>
    
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include <opensync/opensync.h>
#include <opensync/opensync-engine.h>

#include "syncupdates.h"

using namespace QSync;

SyncMemberUpdate::SyncMemberUpdate()
{
}

SyncMemberUpdate::SyncMemberUpdate( OSyncEngineMemberUpdate *update )
{
  OSyncError *error = osync_engine_member_update_get_error(update);
  //NEW: const char *objtype = osync_engine_member_update_get_objtype(update);
	
  switch ( osync_engine_member_update_get_event(update) ) {
    case OSYNC_ENGINE_MEMBER_EVENT_CONNECT_DONE:
      break;
    case OSYNC_ENGINE_MEMBER_EVENT_CONNECTED:
      mType = Connected;
      break;
    //TODO: implement case OSYNC_ENGINE_MEMBER_EVENT_CONNECT_DONE
    case OSYNC_ENGINE_MEMBER_EVENT_DISCONNECTED:
      mType = Disconnected;
      break;
    case OSYNC_ENGINE_MEMBER_EVENT_READ:
      mType = Read; 
      break;
    case OSYNC_ENGINE_MEMBER_EVENT_WRITTEN:
      mType = Written;
      break;
    case OSYNC_ENGINE_MEMBER_EVENT_SYNC_DONE:
      mType = SyncDone;
      break;
    case OSYNC_ENGINE_MEMBER_EVENT_DISCOVERED:
      mType = Discovered;
      break;
    case OSYNC_ENGINE_MEMBER_EVENT_ERROR:
      mType = Error;
      break;
  }

  if ( error ) {
    mResult = Result( &(error) );
  }

  mMember.mMember = osync_engine_member_update_get_member(update);
}

SyncMemberUpdate::~SyncMemberUpdate()
{
}

SyncMemberUpdate::Type SyncMemberUpdate::type() const
{
  return mType;
}

Result SyncMemberUpdate::result() const
{
  return mResult;
}

Member SyncMemberUpdate::member() const
{
  return mMember;
}

SyncChangeUpdate::SyncChangeUpdate()
{
}

SyncChangeUpdate::SyncChangeUpdate( OSyncEngineChangeUpdate *update )
{
  OSyncError *error = osync_engine_change_update_get_error(update);
  
  switch ( osync_engine_change_update_get_event(update) ) {
    case OSYNC_ENGINE_CHANGE_EVENT_READ:
      mType = Read;
      break;
    case OSYNC_ENGINE_CHANGE_EVENT_WRITTEN:
      mType = Written;
      break;
    case OSYNC_ENGINE_CHANGE_EVENT_ERROR:
      mType = Error;
      break;
  }

  if ( error ) {
    mResult = Result( &(error) );
  }

  mChange = osync_engine_change_update_get_change( update );
  mMember.mMember = osync_engine_change_update_get_member(update);
  
    // TODO PORTING
  //mMappingId = update->mapping_id;
}

SyncChangeUpdate::~SyncChangeUpdate()
{
}

SyncChangeUpdate::Type SyncChangeUpdate::type() const
{
  return mType;
}

Result SyncChangeUpdate::result() const
{
  return mResult;
}

SyncChange SyncChangeUpdate::change() const
{
  return mChange;
}

Member SyncChangeUpdate::member() const
{
  return mMember;
}

int SyncChangeUpdate::mappingId() const
{
  return mMappingId;
}

SyncMappingUpdate::SyncMappingUpdate()
{
}

SyncMappingUpdate::SyncMappingUpdate( OSyncEngineMappingUpdate *update,
                                      OSyncEngine *engine )
{
  OSyncError *error = osync_engine_mapping_update_get_error(update);
  
  switch ( osync_engine_mapping_update_get_event(update) ) {
    case OSYNC_ENGINE_MAPPING_EVENT_SOLVED:
      mType = Solved;
      break;
//    case OSYNC_ENGINE_MAPPING_EVENT_SYNCED:
  //    mType = Synced;
    //  break;
    case OSYNC_ENGINE_MAPPING_EVENT_ERROR:
      mType = Error;
      break;
  }

  if ( error ) {
    mResult = Result( &(error) );
  }

  //FIXME what to do here?: mWinner = update->winner;
  mMapping.mEngine = engine;

  // TODO PORTING
//  mMapping.mMapping = update->mapping;
}

SyncMappingUpdate::~SyncMappingUpdate()
{
}

SyncMappingUpdate::Type SyncMappingUpdate::type() const
{
  return mType;
}

Result SyncMappingUpdate::result() const
{
  return mResult;
}

long long int SyncMappingUpdate::winner() const
{
  return mWinner;
}

SyncMapping SyncMappingUpdate::mapping() const
{
  return mMapping;
}

SyncEngineUpdate::SyncEngineUpdate()
{
}

SyncEngineUpdate::SyncEngineUpdate( OSyncEngineUpdate *update )
{
  OSyncError *error = osync_engine_update_get_error(update);
  
  switch ( osync_engine_update_get_event(update) ) {
    case OSYNC_ENGINE_EVENT_PREPARED_MAP:
//       FIXME
      break;
    case OSYNC_ENGINE_EVENT_PREPARED_WRITE:
//       FIXME
      break;
    case OSYNC_ENGINE_EVENT_MULTIPLIED:
//       FIXME
      break;
    case OSYNC_ENGINE_EVENT_MAPPED:
//       FIXME
      break;
    case OSYNC_ENGINE_MEMBER_EVENT_CONNECT_DONE:
//       FIXME
      break;
    case OSYNC_ENGINE_EVENT_CONNECTED:
      mType = Connected;
      break;
    case OSYNC_ENGINE_EVENT_READ:
      mType = Read;
      break;
    case OSYNC_ENGINE_EVENT_WRITTEN:
      mType = Written;
      break;
    case OSYNC_ENGINE_EVENT_DISCONNECTED: 
      mType = Disconnected;
      break;
    case OSYNC_ENGINE_EVENT_ERROR:
      mType = Error;
      break;
    case OSYNC_ENGINE_EVENT_SUCCESSFUL:
      mType = Successful;
      break;
    case OSYNC_ENGINE_EVENT_PREV_UNCLEAN:
      mType = PrevUnclean;
      break;
    case OSYNC_ENGINE_EVENT_END_CONFLICTS:
      mType = EndConflicts;
      break;
    case OSYNC_ENGINE_EVENT_SYNC_DONE:
      mType = SyncDone;
      break;
  }

  if ( error ) {
    mResult = Result( &(error) );
  }
}

SyncEngineUpdate::~SyncEngineUpdate()
{
}

SyncEngineUpdate::Type SyncEngineUpdate::type() const
{
  return mType;
}

Result SyncEngineUpdate::result() const
{
  return mResult;
}

