
#include "kjotsmodel.h"

#include "datadir.h" 

#include <QDBusConnection>
#include <QMimeData>

#include <kstandarddirs.h>
#include <kdirlister.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kurl.h>

#include "kjotspage.h"
#include "kjotsbook.h"
#include "kjotsentity.h"

#include "datamanageradaptor.h"

#include <kdebug.h>


KJotsModel::KJotsModel( QObject *parent, const QString &rootBook ) : QAbstractItemModel(parent)
{
    // The adaptor automatically connects signals in the dbus interface
    // to signals in this class of the same name and signature.
    (void) new DataManagerAdaptor(this);
    const QString dbusPath = "/KJotsDataManager";
    const QString dbusInterface = "org.kde.KJots.DataManager";
    QDBusConnection dbus = QDBusConnection::sessionBus();
    dbus.registerObject( dbusPath, this );

    // Connect the entityMoved signal from the dbus interface to slotEntityMoved.
    dbus.connect(QString(), dbusPath, dbusInterface, "entityMoved", this,
        SLOT(slotEntityMoved(const QString &, const QString &, const QString &, int)));
    dbus.connect(QString(), dbusPath, dbusInterface, "entityTitleChanged", this,
        SLOT(slotEntityTitleChanged(qlonglong, const QString &)));
    dbus.connect(QString(), dbusPath, dbusInterface, "pageContentChanged", this,
        SLOT(slotPageContentChanged(const QString &, const QString &)));

  m_rootBook = "kjots.bookshelf";
//   m_rootBook = rootBook;

  fillBookshelf();
}

KJotsModel::~KJotsModel()
{

}

int KJotsModel::columnCount(const QModelIndex &index) const
{
  return 1;
}

bool KJotsModel::dropMimeData ( const QMimeData * data, Qt::DropAction action, int row, int column, const QModelIndex & parent )
{
  if ( data->hasFormat( "text/uri-list" ) )
  {

    KUrl::List urls = KUrl::List::fromMimeData( data );
    foreach( const KUrl &url, urls ) {
//       if (m_entries.contains(url.queryString("id")))
//       {
//         KJotsEntity *e = m_entries.value(id);
//         KJotsBook *source = e->parent();
//         KJotsBook *dest = m_entries.value(index.internalId());
//         if (source != dest)
//         {
// //           Remove from source
// //           Add to dest at row or at end.
//
//         } else {
// //           KFile file(source->path());
// //           file.read();
// //           file.remove line for e and replace it at row or at end of list if row is -1
// //           file.write();
//         }
//
//       }
    }
  }
  // Handle drops of text/plain and create new pages?


  return false;
}

QMimeData *KJotsModel::mimeData ( const QModelIndexList & indexes ) const
{
  QMimeData *data = new QMimeData();
  KUrl::List urls;
  foreach( const QModelIndex &index, indexes ) {
    if ( index.column() != 0 )
      continue;

    KJotsEntity *entity = m_entities.value( index.internalId() );
    urls << entity->url();
  }
  urls.populateMimeData( data );

  return data;
}

QStringList KJotsModel::mimeTypes () const
{
  return QStringList() << QLatin1String( "text/uri-list" );
}

QModelIndex KJotsModel::indexForId(qint64 id) const
{
  if ( id <= 0 )
    return QModelIndex();

  qint64 parentId = getParentId(id);
  if ( parentId == -1 )
    return QModelIndex();
  int row = m_childEntities.value(parentId).indexOf(id);
  int column = 0;
  return createIndex(row, column, reinterpret_cast<void*>(id));
}

void KJotsModel::fillBookshelf()
{
  // Nah, that does setTitle, I want to setHeaderdata...
  // addBook(m_rootBook, -1, 0)
  // Get the title and put it in headerdata.
  QString fileName(m_rootBook);
//  QFile bookFile( KStandardDirs::locateLocal("data","kjots/") + fileName );
  QFile bookFile( DATADIR + fileName );
  kDebug() << bookFile.fileName();
  if ( !bookFile.open(QIODevice::ReadOnly | QIODevice::Text) ) {
    KMessageBox::error(0, i18n("<qt>Error opening internal file.</qt>"));
    return;
  }
//   QTextStream inStream ( &bookFile );
//   inStream.setCodec("UTF-8");

  qint64 bookshelfId = 0;

//   insertChildEntities(inStream, bookshelfId);
  insertChildEntities(bookFile, bookshelfId);

  kDebug() << "filled";
}

void KJotsModel::insertChildEntities(QFile &bookFile, qint64 bookId)
{
  int childRow = 0;
  QStringList newEntities;
  while(!bookFile.atEnd())
  {
    QString line = bookFile.readLine();
    QString trimmed = line.trimmed();
    if (!trimmed.isEmpty())
      newEntities << trimmed;
  }

  QModelIndex parentIndex = indexForId(bookId);
  int start = 0;
  int newEntitiesCount = newEntities.size();
  kDebug() << parentIndex << start << newEntitiesCount;
  beginInsertRows(parentIndex, start, start + newEntitiesCount - 1);
  for (int i = 0; i < newEntities.size(); i++)
  {
    QString newEntityName = newEntities[i];
    kDebug() << "adding " << newEntityName << i << newEntities.size();
    if (newEntityName.endsWith(".kjbook"))
    {
      addBook(newEntityName, bookId, i);
    }
    if (newEntityName.endsWith(".kjpage"))
    {
      addPage(newEntityName, bookId, i);
    }
  }
  endInsertRows();
}

void KJotsModel::addBook(const QString &fileName, qint64 parentId, int row)
{
  //QFile bookFile( KStandardDirs::locateLocal("data","kjots/") + fileName );
  QFile bookFile( DATADIR + fileName );
  kDebug() << bookFile.fileName();
  if ( !bookFile.open(QIODevice::ReadOnly | QIODevice::Text) ) {
    KMessageBox::error(0, i18n("<qt>Error opening internal file.</qt>"));
    return;
  }
  KJotsBook *book = new KJotsBook(this);

  book->setFileName(fileName);

  while(!bookFile.atEnd())
  {
    QString line = bookFile.readLine();
    QString trimmed = line.trimmed();

    if (trimmed.startsWith("Title:"))
      book->setTitle(trimmed.mid(6).trimmed());
    // After the empty line, the child entries begin.
    if (trimmed.isEmpty())
      break;
  }

  // This makes the model emit one row at a time. Would be better to do it in groups.
//   QModelIndex parentIndex = indexForId(parentId);

//   beginInsertRows(parentIndex, row, row);
  m_entities.insert(book->id(), book);
  m_childEntities[ parentId ].insert(row, book->id());
//   endInsertRows();

  insertChildEntities(bookFile, book->id());
}

void KJotsModel::addPage(const QString &fileName, qint64 parentId, int row)
{
  //QFile pageFile( KStandardDirs::locateLocal("data","kjots/") + fileName);
  QFile pageFile( DATADIR + fileName );
  kDebug() << pageFile.fileName() << parentId;
  if ( !pageFile.open(QIODevice::ReadOnly | QIODevice::Text) ) {
    KMessageBox::error(0, i18n("<qt>Error opening internal file.</qt>"));
    return;
  }
//   QTextStream inStream ( &pageFile );
//   inStream.setCodec("UTF-8");
  KJotsPage *page = new KJotsPage(this);
  page->setFileName(fileName);

  while(!pageFile.atEnd())
  {
    QString line = pageFile.readLine();
    QString trimmed = line.trimmed();
    if (trimmed.startsWith("Title:"))
      page->setTitle(trimmed.mid(6).trimmed());
    if (trimmed.isEmpty())
      break;
  }

  // The rest is content.
  QString content;
  while(!pageFile.atEnd())
  {
    content += pageFile.readLine();
  }
  page->setContent(content);

  m_entities.insert(page->id(), page);
  m_childEntities[ parentId ].insert(row, page->id());
}

QModelIndex KJotsModel::index( int row, int column, const QModelIndex & parent ) const
{
//   kDebug() << row << column << parent;
  if ( ( row < 0 ) || ( column != 0 ) )
    return QModelIndex();

  QList<qint64> childList = m_childEntities.value(parent.internalId());

//   kDebug() << row << childList;

  if (row >= childList.size())
    return QModelIndex();

  return createIndex(row, column, reinterpret_cast<void*>( childList.at( row ) ) );

}

qint64 KJotsModel::getParentId(qint64 searchId) const
{
  qint64 parentId = -1;
  // TODO: replace 0 with m_rootId or something
  if (searchId == 0)
    return parentId;

  QHash<qint64, QList<qint64> >::const_iterator i;
  for (i = m_childEntities.constBegin(); i != m_childEntities.constEnd(); ++i)
  {
    if (i.value().contains(searchId))
    {
      parentId = i.key();
      break;
    }
  }
  return parentId;
}

QModelIndex KJotsModel::parent( const QModelIndex & index ) const
{

  qint64 parentId = getParentId(index.internalId());
   // parent not found
  if (parentId == -1)
    return QModelIndex();

  // index is a topLevel index.
  if (parentId == 0)
    return QModelIndex();


  qint64 grandParentId = getParentId(parentId);

  int row = m_childEntities.value(grandParentId).indexOf(parentId);

  int column = 0;
  return createIndex(row, column, reinterpret_cast<void *>(parentId));

}

int KJotsModel::rowCount( const QModelIndex & parent ) const
{
//   kDebug() << m_childEntries.value(parent.internalId()).size();
  return m_childEntities.value(parent.internalId()).size();
}

QVariant KJotsModel::data( const QModelIndex & index, int role ) const
{
  if (!index.isValid())
    return QVariant();

  if ((role == Qt::DisplayRole) || (role == Qt::EditRole))
  {
    return m_entities.value(index.internalId())->title();
  }

  if (role == KJotsModel::PageContentRole)
  {
    KJotsEntity *e = m_entities.value(index.internalId());
    KJotsPage *page = dynamic_cast<KJotsPage *>(e);
//     KJotsPageNew *page = qobject_cast<KJotsPageNew *>(e);
    if (!page)
    {
      // Probably a book.
      return QVariant();
    }
    return page->content();

  }

  return QVariant();
}

// Always return false for actions which take place syncronously, eg via a Job.
bool KJotsModel::setData( const QModelIndex &index, const QVariant &value, int role )
{
  if (index.column() != 0)
  {
    return false;
  }

  if ( role == Qt::EditRole ) {
    KJotsEntity *entity = entityForIndex(index);
    QString title = value.toString();
    entity->setTitle(title);
    emit entityTitleChanged(entity->id(), title);
    return false;
  }

  return QAbstractItemModel::setData( index, value, role );
}


KJotsEntity *KJotsModel::entityForIndex(const QModelIndex &index) const
{
  return m_entities.value(index.internalId());
}

Qt::ItemFlags KJotsModel::flags( const QModelIndex &index ) const
{
  Qt::ItemFlags _flags = QAbstractItemModel::flags(index);

  if (!index.isValid())
    return _flags;

  _flags |= Qt::ItemIsEditable;
  return _flags;
}


void KJotsModel::moveEntity(const QString &entity, const QString &source, const QString &dest, int row)
{
  emit entityMoved(entity, source, dest, row);
}


void KJotsModel::slotEntityMoved(const QString &entity, const QString &source, const QString &dest, int row)
{
  kDebug() << entity << source << dest << row;
}


void KJotsModel::slotEntityTitleChanged(qlonglong id, const QString &title)
{
//   KJotsEntity *entity = getEntity(fileName);

  kDebug() << id << title;

  KJotsEntity *entity = m_entities[id];
  entity->setTitle(title);
  m_entities[entity->id()] = entity;
  QModelIndex idx = indexForId(entity->id());
  emit dataChanged(idx, idx);

}

