
#ifndef KJOTSMODEL_H
#define KJOTSMODEL_H

#include <QAbstractItemModel>
#include <QHash>

class QFile;
class QTextStream;
class KJotsEntity;

class KJotsModel : public QAbstractItemModel
{
  Q_OBJECT
public:

  enum Roles {
    PageContentRole = Qt::UserRole,
    EntityIdRole
  };

  KJotsModel(QObject *parent = 0, const QString &rootBook = QString());
  ~KJotsModel();

  // Temporary method:
  void moveEntity(const QString &, const QString &, const QString &, int);

  virtual bool dropMimeData ( const QMimeData * data, Qt::DropAction action, int row, int column, const QModelIndex & parent );

  virtual int columnCount ( const QModelIndex & parent = QModelIndex() ) const;

  virtual QMimeData * mimeData ( const QModelIndexList & indexes ) const;
  virtual Qt::ItemFlags flags( const QModelIndex &index ) const;

  virtual QStringList mimeTypes () const;

  virtual QModelIndex index( int row, int column, const QModelIndex & parent = QModelIndex() ) const;
  virtual QModelIndex parent( const QModelIndex & index ) const;

  virtual int rowCount( const QModelIndex & parent = QModelIndex() ) const;

  virtual QVariant data( const QModelIndex & index, int role = Qt::DisplayRole ) const;
  virtual bool setData( const QModelIndex &index, const QVariant &value, int role = Qt::EditRole );


signals:
  void entityMoved(const QString &, const QString &, const QString &, int);
  void entityTitleChanged(qlonglong, const QString &);
  void pageContentChanged(const QString &, const QString &);

protected:

  qint64 getParentId(qint64 searchId) const;
  QModelIndex indexForId(qint64 id) const;
  KJotsEntity *entityForIndex(const QModelIndex &index) const;
  void fillBookshelf();
  void addBook(const QString &fileName, qint64 parentId, int row);
  void addPage(const QString &fileName, qint64 parentId, int row);
  void insertChildEntities(QFile &bookfile, qint64 bookId);

private:
  QHash<qint64, KJotsEntity*> m_entities;
//   QHash<QString, KJotsEntryNew*> m_entriesByFilename;
  QHash<qint64, QList<qint64> > m_childEntities;
  QString m_rootBook;

private slots:
  void slotEntityMoved(const QString &, const QString &, const QString &, int);
  void slotEntityTitleChanged(qlonglong, const QString &);
//   void slotPageContentChanged(const QString &, const QString &);

};

#endif

