/***************************************************************************
                editeventsdialog.cpp  -  description
                             -------------------
    begin                : Sun Feb 24 22:20:01 EST 2002
    copyright            : (C) 2002-2006 by Buddy Brewer
    email                : kluje-talk@lists.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "editeventsdialog.h"
#include "listboxevent.h"
#include <config.h>
#include <qsize.h>
#include <qcursor.h>
#include <klocale.h>
#include <kmessagebox.h>


EditEventsDialog::EditEventsDialog (KLuJe *parent, const char *name, bool modal) :
    KDialogBase (Plain, i18n ("Edit events"), Apply|Ok|Cancel, Ok,
                (QWidget *) parent, name, modal, true)
{
    activeEvent = 0;
    page = plainPage ();
    QHBoxLayout* topLayout = new QHBoxLayout (page, 0, spacingHint (), "topLayout");

    // LEFT side of dialog
    navigator = new QVBox (page, "navigator");
    datePicker = new KDatePicker (navigator);
    eventPicker = new KListBox (navigator);

    // RIGHT side of dialog
    editor = new QVBox (page, "editor");
    dateEditor = new QDateTimeEdit (editor);
    subjEditor = new QHBox (editor, "subjEditor");
    subjLineLabel = new QLabel (i18n ("Subject:"), subjEditor);
    subjLineEdit = new KLineEdit (subjEditor);
    eventEdit = new KEdit (editor);
    eventEdit->setWordWrap (QMultiLineEdit::WidgetWidth);
    editor->setDisabled (TRUE);

    // Put it all together
    topLayout->addWidget (navigator, 0);
    topLayout->addWidget (editor, 1);
    kapp->config ()->setGroup ("EventEditor");
    setInitialSize (configDialogSize ("EventEditor"));

    // When a date is clicked, download that day's events and display the
    // subjects in a list box.
    connect (datePicker, SIGNAL (dateSelected (QDate)),
            parent->getIO (), SLOT (slotGetevents (QDate)));
    connect (datePicker, SIGNAL (dateSelected (QDate)),
            this, SLOT (slotClearEditor ()));
    connect (datePicker, SIGNAL (dateEntered (QDate)),
            parent->getIO (), SLOT (slotGetevents (QDate)));
    connect (datePicker, SIGNAL (dateEntered (QDate)),
            this, SLOT (slotClearEditor ()));
    connect (parent->getIO (), SIGNAL (newEvents (const QDict<LJEvent>&)),
            this, SLOT (slotUpdateEvents (const QDict<LJEvent>&)));

    // When a subject is clicked, redraw the "editor"  (RIGHT) side of the dialog
    connect (eventPicker, SIGNAL (executed (QListBoxItem*)),
            this, SLOT (slotEventPicked (QListBoxItem*)));
}

void EditEventsDialog::slotShowDialog ()
{
    activeEvent = 0;
    show ();
    KLuJe* main = dynamic_cast<KLuJe*> (this->parent ());

    if (main)
        main->getIO ()->slotGetevents (QDate::currentDate ());
}

void EditEventsDialog::slotEventPicked (QListBoxItem *item)
{
    if (!item)
        return;

    LJEvent* event =  ( (ListBoxEvent*)item)->getEvent ();

    if (event == activeEvent)
        return;

    if (shouldSave ())
        slotApply ();

    subjLineEdit->setText (event->getSubject ());
    eventEdit->setText (event->getText ());
    dateEditor->setDateTime (QDateTime (event->getDate (), event->getTime ()));
    activeEvent = event;
    editor->setEnabled (TRUE);
}


void EditEventsDialog::slotUpdateEvents (const QDict<LJEvent>& events)
{
    eventPicker->clear ();

    for (QDictIterator<LJEvent> it (events); it.current (); ++it)
    {
        ListBoxEvent *event;
        if (!it.current ()->getSubject ().isEmpty ())
            event = new ListBoxEvent (it.current ()->getSubject ());
        else
            event = new ListBoxEvent (it.current ()->getText ());
        event->setEvent (it.current ());
        eventPicker->insertItem (event);
    }

    navigator->setEnabled (TRUE);
}


void EditEventsDialog::slotClearEditor ()
{
    if  (shouldSave ())
        slotApply ();

    activeEvent = 0;
    navigator->setDisabled (TRUE);
    editor->setDisabled (TRUE);
    eventPicker->clear ();
    subjLineEdit->clear ();
    eventEdit->clear ();
    dateEditor->setDateTime (QDateTime::fromString ("1970-01-01 00:00:00", Qt::ISODate));
}


bool EditEventsDialog::shouldSave ()
{
    if (!activeEvent) return false;
    if  (subjLineEdit->text () != activeEvent->getSubject () ||
        eventEdit->text () != activeEvent->getText ())
    {
        int response = KMessageBox::questionYesNo (this,
                                                  i18n ("Changes have been made. Do you want to save them?"),
                                                  i18n ("Save changes?"),
                                                  i18n ("Save Changes"), i18n ("Don't Save Changes"));
        return  (response == KMessageBox::Yes);
    }
    return false;
}


void EditEventsDialog::slotApply ()
{
    if  (!activeEvent)
        return;

    //editor->setDisabled (TRUE);
    activeEvent->setText (eventEdit->text ());
    activeEvent->setSubject (subjLineEdit->text ());
    activeEvent->setDate (dateEditor->dateTime ().date ());
    activeEvent->setTime (dateEditor->dateTime ().time ());
     ( (KLuJe *) this->parent ())->getIO ()->slotSaveEvent (*activeEvent);
    //editor->setEnabled (TRUE);
}


void EditEventsDialog::slotOk ()
{
    slotApply ();
    accept ();
}


void EditEventsDialog::disable ()
{
    this->setEnabled (false);
    this->setCursor (QCursor (Qt::WaitCursor));
    subjLineEdit->setCursor (QCursor (Qt::WaitCursor));
//  eventEdit->setCursor (QCursor (Qt::WaitCursor));
}


void EditEventsDialog::enable ()
{
    this->setEnabled (true);
    this->setCursor (QCursor (Qt::ArrowCursor));
    subjLineEdit->setCursor (QCursor (Qt::ArrowCursor));
//  eventEdit->setCursor (QCursor (Qt::ArrowCursor));
}


EditEventsDialog::~EditEventsDialog ()
{
    kapp->config ()->setGroup ("EventEditor");
    saveDialogSize ("EventEditor");
}

