/***************************************************************************
              kluje.cpp  -  description
               -------------------
  begin                : Mon Jan 28 22:22:24 EST 2002
  copyright            : (C) 2002-2006 by Buddy Brewer
  email                : kluje-talk@lists.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kluje.h"
#include "klujeio.h"
#include "klujesystray.h"
//#include "klujesyntaxhighlighter.h"
#include <qvbox.h>
#include <qhbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qmultilineedit.h>
#include <kmenubar.h>
#include <kapplication.h>
#include <kaction.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kcombobox.h>
#include <kcompletion.h>
#include <kedittoolbar.h>
//#include <khelpmenu.h>
#include <kkeydialog.h>
#include <kkeydialog.h>
#include <klineedit.h>
#include <kmessagebox.h>
#include <krun.h>
#include <kspell.h>
#include <kstatusbar.h>
#include <kstdaccel.h>
#include <kstdaction.h>
#include <kurl.h>
#include <kwin.h>

// richtext support includes
#include <dom/dom_doc.h>
#include <dom/dom_element.h>
#include <dom/html_document.h>
#include <dom/html_base.h>
#include <dom/html_inline.h>
#include <klibloader.h>
#include <kparts/factory.h>
#include <kparts/part.h>
#include <ktextedit.h>

#include <config.h>

#ifdef HAVE_XMMS
// For xmms query
//#include <glib.h>
//#include <xmms/xmmsctrl.h>
#endif


KLuJe::KLuJe(const char *name) :
    KParts::MainWindow(0, name),
  interval(90),
  editpart(0),
  lastMenuIndex(-1)
{
    // Initialize the preferences dialog, display Preferences dialog if neccesary
    prefsDialog = new PreferencesDialog(kapp->config());
    kapp->config()->setGroup("ServerInfo");
    if(kapp->config()->readEntry("Username", "") == "") {
        prefsDialog->exec();
    }

    loadOptions();
    setXMLFile("klujeui.rc");

    moods = new KLuJeMoods(kapp->config());

    connect(prefsDialog, SIGNAL(configChanged(const KConfig &)),
            klujeIO, SLOT(slotConfigChanged(const KConfig &)));

    // Initialize the editevents dialog
    editEventsDialog = new EditEventsDialog(this);

    // Used for polling checkfriends info
    lastUpdate = QString("");
    timer = new QTimer( this );
    connect( timer, SIGNAL(timeout()), klujeIO, SLOT(slotCheckFriends()) );

    // Manually call checkfriends once to initialize polling intervals
    // and last checked time
    klujeIO->slotCheckFriends();


    // set up statusbar
    statusBar()->message("Status: Ready!");

    // Init animation variables
    inverted = false;
    animating = false;

    // Animation signals
    connect(klujeIO, SIGNAL(checkfriendsNotify()), this, SLOT(slotStartAnimation()));

    // Log in to LJ server, get moods, etc.
    initView();
    initActions();
    klujeIO->slotLogin();

    //------------------------ initSystray() ??? -------------------
    sysTray = new KLuJeSysTray(this);
    connect(sysTray, SIGNAL(userClicked()), this, SLOT(slotStopAnimation()));

    kapp->config()->setGroup("MainWindow");
    if (kapp->config()->readBoolEntry("showMenuBar", true) == false)
        menuBar()->hide();
    if (kapp->config()->readBoolEntry("showToolBar", true) == false)
        toolBar()->hide();
    if (kapp->config()->readBoolEntry("showStatusBar", true) == false)
        statusBar()->hide();


    spellConfig = new KSpellConfig ();
}

void KLuJe::initActions()
{
    // File menu
    (void)new KAction(i18n("&Login"), QIconSet(BarIcon ("connect_creating")),
                      CTRL+Key_L, klujeIO, SLOT(slotLogin()), actionCollection(), "login");
    KAction* saveAction = KStdAction::save(klujeIO, SLOT(slotPostevent()), actionCollection());
    saveAction->setText(i18n("&Submit entry"));
    (void)new KAction(i18n("&Edit past events"), QIconSet(BarIcon("edit")), // KDE 4 icon name: document-properties
                      0, editEventsDialog, SLOT(slotShowDialog()), actionCollection(), "show_editevents");
    (void)new KAction(i18n("&Clear"), QIconSet(BarIcon("eraser")), // KDE 4 icon name: draw-eraser
                      CTRL+Key_I, this, SLOT(slotClear()), actionCollection(), "clear");
    (void)KStdAction::quit(kapp, SLOT(closeAllWindows()), actionCollection());

    if (!editpart)
    {
        // Edit menu
        KStdAction::cut(this, SLOT(slotCut()), actionCollection());
        KStdAction::copy(this, SLOT(slotCopy()), actionCollection());
        KStdAction::paste(this, SLOT(slotPaste()), actionCollection());

        // Tools menu
        (void)KStdAction::spelling(this, SLOT(slotSpellCheck()), actionCollection());
    }


    // Settings menu
    kapp->config()->setGroup("MainWindow");
    KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection())
        ->setChecked(kapp->config()->readBoolEntry("showStatusBar", true));
    KStdAction::showMenubar(this, SLOT(slotViewMenuBar()), actionCollection())
        ->setChecked(kapp->config()->readBoolEntry("showMenuBar", true));
    setStandardToolBarMenuEnabled (kapp->config()->readBoolEntry("showToolBar", true));


    KToggleAction* journaledit = new KToggleAction(i18n("Show journal editor"), 0, this,
                                                   SLOT(slotViewJournalEdit()), actionCollection(), "show_journaledit");
    journaledit->setChecked(kapp->config()->readBoolEntry("showJournalEditor", true));
    KToggleAction* musicedit = new KToggleAction(i18n("Show music editor"), 0, this,
                                                 SLOT(slotViewSongEdit()), actionCollection(), "show_musicedit");
    musicedit->setChecked(kapp->config()->readBoolEntry("showMusicEditor", true));
    KToggleAction* subjectedit = new KToggleAction(i18n("Show subject editor"), 0, this,
                                                   SLOT(slotViewSubjectEdit()), actionCollection(), "show_subjectedit");
    subjectedit->setChecked(kapp->config()->readBoolEntry("showSubjectEditor", true));
    KToggleAction* moodedit = new KToggleAction(i18n("Show mood editor"), 0, this,
                                                SLOT(slotViewMoodEdit()), actionCollection(), "show_moodedit");
    moodedit->setChecked(kapp->config()->readBoolEntry("showMoodEditor", true));
    (void)KStdAction::preferences(prefsDialog, SLOT(show()), actionCollection());
    (void)KStdAction::configureToolbars(this, SLOT(slotConfigureToolbars()), actionCollection());
    (void)KStdAction::keyBindings(this, SLOT(slotConfigureShortcuts()), actionCollection());

    kdDebug(6969) << "initActions: calling createGUI" << endl;
    createGUI(editpart);
    connect(getUrlMenu(), SIGNAL(activated(int)), this, SLOT(slotSelectedLink(int)));
    getUrlMenu()->setEnabled(false);
}

void KLuJe::initView()
{
    // set up the main gui stuff
    QWidget* centralWidget = new QWidget(this, "centralWidget");
    setCentralWidget(centralWidget);
    applyMainWindowSettings(kapp->config(), "MainWindow");
    setCaption("KLuJe");
    kapp->setTopWidget(this);
    KWin::setIcons(winId(), DesktopIcon("kluje"), SmallIcon("kluje"));
    this->setIcon(SmallIcon("kluje"));

    // set up main view
    kapp->config()->setGroup("MainWindow");
    QGridLayout *layout = new QGridLayout(centralWidget, 2, 6,
                                          KDialog::marginHint(),
                                          KDialog::spacingHint());
    subjectLabel = new QLabel(i18n("Subject:"), centralWidget);
    subjLineEdit = new KLineEdit(centralWidget);
    layout->addWidget(subjectLabel, 0, 0);
    layout->addWidget(subjLineEdit, 0, 1);
    if (kapp->config()->readBoolEntry("showSubjectEditor", true) == false)
    {
        subjectLabel->hide();
        subjLineEdit->hide();
    }

    QLabel* textLabel = new QLabel(i18n("Event:"), centralWidget);
    KLibFactory *factory = KLibLoader::self()->factory("libkrichtexteditpart");
    if (factory)
    {
        editpart = dynamic_cast<KParts::Part*>
                   (factory->create(centralWidget,
                                    "krichtexteditpart",
                                    "KParts::ReadWritePart"));
    }

    if (editpart)
    {
        QDomNode doc = editpart->domDocument().documentElement();
        QDomNode xmluiItem = doc.firstChild();

        // ugly hack to get rid of unwanted XMLUI mergings
        while (!xmluiItem.isNull())
        {
            QString nodeName = xmluiItem.nodeName();
            QString attrName = xmluiItem.attributes().namedItem("name").toAttr().value();

            if (nodeName == "MenuBar")
            {
                QDomNode singleMenu = xmluiItem.firstChild();
                // weed out unwanted menus
                while  (!singleMenu.isNull())
                {
                    QString name = singleMenu.attributes().namedItem("name").toAttr().value();
                    if (name == "file" || name == "view")
                    {
                        QDomNode next = singleMenu.nextSibling();
                        xmluiItem.removeChild(singleMenu);
                        singleMenu = next;
                    }
                    else
                    {
                        singleMenu = singleMenu.nextSibling();
                    }
                }
            }
            else if (attrName == "mainToolBar")
            {
                QDomNode singleAction = xmluiItem.firstChild();
                // weed out unwanted menus
                while  (!singleAction.isNull())
                {
                    QString name = singleAction.attributes().namedItem("name").toAttr().value();
                    if (name == "file_open" ||
                        name == "file_save" ||
                        name == "file_save_as" ||
                        name == "view_zoom_in" ||
                        name == "view_zoom_out")
                    {
                        QDomNode next = singleAction.nextSibling();
                        xmluiItem.removeChild(singleAction);
                        singleAction = next;
                    }
                    else
                    {
                        singleAction = singleAction.nextSibling();
                    }
                }
            }

            xmluiItem = xmluiItem.nextSibling();
        }
        text = static_cast<KTextEdit*>( editpart->widget() );
    }
    else
    {
        text = new KTextEdit(centralWidget);
        text->setWordWrap(QMultiLineEdit::WidgetWidth);
    }
    layout->addWidget(textLabel, 1, 0, Qt::AlignAuto | Qt::AlignTop);
    layout->addWidget(text, 1, 1);

    // Add background spell checker
//  mSpellChecker = new DictSpellChecker( text );

    moodLabel = new QLabel(i18n("Mood:"), centralWidget);
    moodCombo = new KComboBox(true, centralWidget);
    layout->addWidget(moodLabel, 2, 0);
    layout->addWidget(moodCombo, 2, 1);
    if (kapp->config()->readBoolEntry("showMoodEditor", true) == false)
    {
        moodLabel->hide();
        moodCombo->hide();
    }

    songLabel = new QLabel(i18n("Music:"), centralWidget);
    songLineEdit = new KLineEdit(centralWidget);
    layout->addWidget(songLabel, 3, 0);
    layout->addWidget(songLineEdit, 3, 1);
    if (kapp->config()->readBoolEntry("showMusicEditor", true) == false)
    {
        songLabel->hide();
        songLineEdit->hide();
    }

#ifdef HAVE_XMMS
//  xmmsQueryButton = new QPushButton("XMMS", songEditor);
//  connect(xmmsQueryButton, SIGNAL(clicked()), SLOT(slotQueryXMMS()));
#endif

    journalLabel = new QLabel(i18n("Journal:"), centralWidget);
    journalCombo = new KComboBox(false, centralWidget);
    layout->addWidget(journalLabel, 4, 0);
    layout->addWidget(journalCombo, 4, 1);
    if (kapp->config()->readBoolEntry("showJournalEditor", true) == false)
    {
        journalLabel->hide();
        journalCombo->hide();
    }

    securityLabel = new QLabel(i18n("Security:"), centralWidget);
    securityCombo = new KComboBox(false, centralWidget);
    securityCombo->insertItem(i18n("Public"));
    securityCombo->insertItem(i18n("Private"));
    securityCombo->insertItem(i18n("Friends only"));
    layout->addWidget(securityLabel, 5, 0);
    layout->addWidget(securityCombo, 5, 1);
    if (kapp->config()->readBoolEntry("showSecurityEditor", true) == false)
    {
        securityLabel->hide();
        securityCombo->hide();
    }
}

void KLuJe::loadOptions()
{
    // KConfig *config = KGlobal::config();
    // read in config info
    kapp->config()->setGroup("ServerInfo");
    user = kapp->config()->readEntry("Username", "");
    QString password = kapp->config()->readEntry("Password", "");
    QString serverUrl = kapp->config()->readEntry("ServerUrl",
                                                  "http://www.livejournal.com/interface/flat");
//                                                  "http://www.livejournal.com/cgi-bin/log.cgi");
    klujeIO = new KLuJeIO(this, user, password, serverUrl);
    kapp->config()->setGroup("MainWindow");
}

void KLuJe::slotCut()
{
    if (subjLineEdit->hasFocus()) subjLineEdit->cut();
    if (text->hasFocus()) text->cut();
    if (moodCombo->hasFocus()) moodCombo->lineEdit()->cut();
    if (songLineEdit->hasFocus()) songLineEdit->cut();
}


void KLuJe::slotCopy()
{
    if (subjLineEdit->hasFocus()) subjLineEdit->copy();
    if (text->hasFocus()) text->copy();
    if (moodCombo->hasFocus()) moodCombo->lineEdit()->copy();
    if (songLineEdit->hasFocus()) songLineEdit->copy();
}


void KLuJe::slotPaste()
{
    if (subjLineEdit->hasFocus()) subjLineEdit->paste();
    if (text->hasFocus()) text->paste();
    if (moodCombo->hasFocus()) moodCombo->lineEdit()->paste();
    if (songLineEdit->hasFocus()) songLineEdit->paste();
}

void KLuJe::slotClear()
{
    QString question = i18n("Do you wish to clear the current event?\n\n"
                            "This will result in the loss of all information you "
                            "have not yet submitted!");
    if (KMessageBox::warningYesNo(this, question, i18n("Clear Event"),
                                  i18n("Clear event"), i18n("Don't clear event"),
                                  "clearEventConfirmation") == KMessageBox::Yes)
    {
        slotClearSilently();
    }
}


void KLuJe::slotEnableUI(bool enable)
{
    centralWidget()->setEnabled(enable);
}


void KLuJe::slotClearSilently()
{
    text->clear();
    subjLineEdit->clear();
    moodCombo->setEditText("");
    songLineEdit->clear();
    journalCombo->setCurrentItem(0);
    securityCombo->setCurrentItem(0);
}


void KLuJe::slotSpellCheck()
{
     kdDebug(6969) << "KLuJe::slotSpellCheck() called" << endl;
     QString oldText = text->text();
     KSpell::modalCheck (oldText, spellConfig);
     text->setText(oldText);
}

void KLuJe::slotViewToolBar()
{
    toggleWidget(toolBar(), "showToolBar");
}

void KLuJe::slotViewStatusBar()
{
    toggleWidget(statusBar(), "showStatusBar");
}


void KLuJe::slotViewMenuBar()
{
    toggleWidget(menuBar(), "showMenuBar");
}


void KLuJe::slotViewJournalEdit()
{
    toggleWidget(journalLabel, "showJournalEditor");
    toggleWidget(journalCombo, "showMoodEditor");
}


void KLuJe::slotViewSongEdit()
{
    toggleWidget(songLabel, "showMusicEditor");
    toggleWidget(songLineEdit, "showMoodEditor");
}


void KLuJe::slotViewMoodEdit()
{
    toggleWidget(moodLabel, "showMoodEditor");
    toggleWidget(moodCombo, "showMoodEditor");
}


void KLuJe::slotViewSubjectEdit()
{
    toggleWidget(subjectLabel, "showSubjectEditor");
    toggleWidget(subjLineEdit, "showSubjectEditor");
}

void KLuJe::slotQueryXMMS()
{
#ifdef HAVE_XMMS
//  QString track = xmms_remote_get_playlist_title(0, xmms_remote_get_playlist_pos(0));
//  kdDebug(6969) << "Current track: " << track << endl;
//  songLineEdit->setText(track);
#endif
}


void KLuJe::toggleWidget(QWidget* widget, QString key)
{
    kapp->config()->setGroup("MainWindow");
    if (widget->isVisible())
    {
        kapp->config()->writeEntry(key, false);
        widget->hide();
    }
    else
    {
        kapp->config()->writeEntry(key, true);
        widget->show();
    }
}


void KLuJe::slotStopAnimation()
{
    kdDebug(6969) << "Called stop animate" << endl;
    animating = false;
}


void KLuJe::slotStartAnimation()
{
    animating = true;
    kdDebug(6969) << "Called start animate" << endl;
    slotAnimate();
}

void KLuJe::slotAnimate()
{
    kdDebug(6969) << "Animating" << endl;

    // Swap the icons and check again in half a second as long
    // as the user has not signalled
    // to stop animation by clicking on the system tray
    if( animating )
    {
        kdDebug(6969) << "Animating icon" << endl;
        switchIcon();
        QTimer::singleShot( 500, this, SLOT(slotAnimate()));
    } else
    {
        // Animation is over, so start polling again
        kdDebug(6969) << "User interrupted animation, poll" << endl;
        this->lastUpdate = QString("");
        this->enablePolling();

        // if Icon is inverted, switch back to the normal icon
        // since animation is done
        if(inverted) switchIcon();
    }
}


void KLuJe::switchIcon()
{
    QString newIcon = inverted ? QString("kluje") : QString("kluje_blink");
    KWin::setIcons(this->winId(), SmallIcon( newIcon ), SmallIcon( newIcon ));
    sysTray->setPixmap(SmallIcon( newIcon ));
    inverted = !inverted;
}


void KLuJe::slotExit()
{
    kdDebug(6969) << "Exit called" << endl;
    this->close();
}


bool KLuJe::usesPayServer()
{
    kapp->config()->setGroup("ServerInfo");
    return kapp->config()->readBoolEntry("FastServer", false);
}


void KLuJe::setUsePayServer(bool paid)
{
    kapp->config()->setGroup("ServerInfo");
    kapp->config()->writeEntry("FastServer", paid);
}


KPopupMenu* KLuJe::getUrlMenu()
{
    return static_cast<KPopupMenu*>(factory()->container("urlMenu", this));
}


void KLuJe::slotSelectedLink(int id)
{
    bool found(false);
    KURL url(urls.at(id, &found));

    if (!found)
        return;

    kdDebug(6969) << "User selected link " << url.prettyURL() << endl;

    KRun::runURL(url, "text/html");
}


/** Public entry point to popup menu population **/
void KLuJe::populateMenu(const QMap<QString, QString>& dict)
{
    KPopupMenu* urlMenu = getUrlMenu();

    if (!urlMenu)
        return;

    urlMenu->clear();
    urls.clear();
    populateMenu(urlMenu, dict, 0);
    urlMenu->setEnabled(urls.count() > 0);
    replyDict = dict;
}


/** Internal menu population method.  Does all the work.  Gets none of the love. */
void KLuJe::populateMenu(KPopupMenu * popMenu,
                         const QMap<QString, QString>& dict,
                         int menuNum)
{
    lastMenuIndex = menuNum;

    QString menuCountStr = QString("menu_%1_count").arg(menuNum);
    int count = dict[menuCountStr.latin1()].toInt();
    kdDebug(6969) << "There are " << count << " items in the menu" << endl;
    for (int i = 1; i <= count ; ++i) {

        QString menu = QString("menu_%1_%2_").arg(menuNum).arg(i);

        QString urlStr = dict[menu + "url"];
        kdDebug(6969) << "Found url string " << urlStr << " for menu " << menu << endl;
        QString text = dict[menu + "text"];

        if (!urlStr.isEmpty()) {
            kdDebug(6969) << "Menu " << i << " is " << text << "::" << urlStr << endl;
            urls.push_back(KURL(urlStr));
            popMenu->insertItem(text, urls.count() - 1);
        } else if (dict.find(menu + "sub") != dict.end()) {
            kdDebug(6969) << "Got sub menu " << lastMenuIndex + 1 << endl;
            KPopupMenu * subMenu = new KPopupMenu();
            connect(subMenu, SIGNAL(activated(int)), this, SLOT(slotSelectedLink(int)));
            populateMenu(subMenu, dict, lastMenuIndex + 1);
            popMenu->insertItem(text, subMenu);
        } else {
            if (text == "-") {
                kdDebug(6969) << "Add separator" << endl;
                popMenu->insertSeparator();
            } else {
                kdDebug(6969) << "Add text " << text << endl;
                popMenu->insertItem(text);
            }
        }
    }
}


void KLuJe::slotConfigureToolbars()
{
    saveMainWindowSettings(kapp->config(), "kluje");
    KEditToolbar dlg(factory());
    connect(&dlg, SIGNAL(newToolbarConfig()), SLOT(slotUpdateToolbars()));
    dlg.exec();
    populateMenu (replyDict);
}


void KLuJe::slotUpdateToolbars()
{
    // Max Lapan: this is really not needed at the moment
//    createGUI(editpart);
    applyMainWindowSettings(kapp->config(), "kluje");
}


void KLuJe::slotConfigureShortcuts()
{
    KKeyDialog::configure(actionCollection(), this);
}


KLuJe::~KLuJe()
{
    saveMainWindowSettings(kapp->config(), "MainWindow");
//  delete mSpellChecker;
}

