/***************************************************************************
                            kluje.cpp  -  description
                             -------------------
    begin                : Mon Jan 28 22:22:24 EST 2002
    copyright            : (C) 2002-2006 by Buddy Brewer
    email                : kluje-talk@lists.sourceforge.net
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kluje.h"
#include "editeventsdialog.h"
#include "klujeio.h"

// Needed for encoding strings
#include <kurl.h>

// needed for RIGHT string encoding
#include <ctype.h>

// debug includes
#include <kdebug.h>
//#include <iostream>

// Used for parsing server resposnes
#include <qasciidict.h>
#include <qregexp.h>

#include <keditcl.h>
#include <kcombobox.h>
#include <klineedit.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kstatusbar.h>
#include <ktextedit.h>

#include <qstrlist.h>

#include <kmessagebox.h>

KLuJeIO::KLuJeIO(KLuJe *kluje, QString user, QString pass, QString server)
    : QObject(),
      klujeWin(kluje),
      username(user),
      password(pass),
      serverUrl(server),
      response_counter(0)
{
    response_buffers.setAutoDelete(true);
}

void
KLuJeIO::createJob(const QString& postdata,
                   const QObject* resultReceiver,
                   const char* resultSlot)
{
    kdDebug(6969) << "SENDING: " << postdata << endl;
    // send the request out to the livejournal server
    KIO::TransferJob *job = KIO::http_post(serverUrl, postdata.utf8(), false);
    job->addMetaData("content-type", "Content-Type: application/x-www-form-urlencoded");
    job->setName(getUniqueName().ascii());
//    maybeAddFastServerCookie(job);
    connect(job, SIGNAL(data(KIO::Job*, const QByteArray&)),
            SLOT(slotDataArrived(KIO::Job*, const QByteArray&)));
    connect(job, SIGNAL(result(KIO::Job*)), resultReceiver, resultSlot);
}

void
KLuJeIO::slotPostevent()
{
    kdDebug(6969) << "KLuJeIO::slotPostevent called" << endl;
    QString text = klujeWin->getText()->text();
    QString subject = klujeWin->getSubjectLineEdit()->text();
    text.replace(QRegExp(QString::fromLatin1(".*<body.*>(.*)</body>.*")), QString::fromLatin1("\\1"));
    text.replace(QRegExp(QString::fromLatin1("\n+(<p)")), QString::fromLatin1("\\1"));

    // TODO: prop_opt_preformatted=0, prop_picture_keyword=blah
    QDateTime now = QDateTime::currentDateTime();
    QDate date = now.date();
    QTime time = now.time();

    QString postdata = "mode=postevent&user=" + username +
                       "&hpassword=" + password + "&event=" +
                       encodeForUrl (text) + "&subject=" +
                       encodeForUrl (subject) + "&year=" +
                       QString::number(date.year()) + "&mon=" + QString::number(date.month()) +
                       "&day=" + QString::number(date.day()) +
                       "&min=" + QString::number(time.minute()) +
                       "&hour=" + QString::number(time.hour()) + "&ver=1";

    if (klujeWin->getMoodCombo()->currentText() != "")
    {
        kdDebug(6969) << "KLuJeIO::slotPostevent -- moodCombo->currentText() == " << klujeWin->getMoodCombo()->currentText() << endl;
        if (klujeWin->getMoods()->getId(klujeWin->getMoodCombo()->currentText()) != NULL)
        {
            kdDebug(6969) << "KLuJeIO::slotPostevent -- adding prop_current_moodid=" <<
                klujeWin->getMoods()->getId(klujeWin->getMoodCombo()->currentText()) << endl;
            postdata += "&prop_current_moodid=";
            postdata += klujeWin->getMoods()->getId(klujeWin->getMoodCombo()->currentText());
        } else
        {
            kdDebug(6969) << "KLuJeIO::slotPostevent -- adding prop_current_mood" << endl;
            postdata += "&prop_current_mood=" + klujeWin->getMoodCombo()->currentText();
        }
    }
    if (klujeWin->getSecurityCombo()->currentText() != "Public")
    {
        if (klujeWin->getSecurityCombo()->currentText() == "Private")
            postdata += "&security=private";
        else if (klujeWin->getSecurityCombo()->currentText() == "Friends only")
            postdata += "&security=usemask&allowmask=1";
    }
    if (klujeWin->getSongLineEdit()->text() != "")
    {
        kdDebug(6969) << "KLuJeIO::slotPostevent -- adding song=" << klujeWin->getSongLineEdit()->text() << endl;
        postdata += "&prop_current_music=" + klujeWin->getSongLineEdit()->text();
    }
    if (klujeWin->getJournalCombo()->currentText() != username)
    {
        postdata += "&usejournal=" + klujeWin->getJournalCombo()->currentText();
    }
    // TODO: I have *no* idea why not adding this trailing '&' makes things break
    postdata += "&";
    klujeWin->slotEnableUI(false);
    createJob(postdata, this, SLOT(slotPosteventResult(KIO::Job*)));
}

void
KLuJeIO::slotLogin()
{
    kdDebug(6969) << "KLuJeIO::slotLogin called" << endl;
    QString postdata = "mode=login&user=" +
                       encodeForUrl (username) +
                       "&hpassword=" + encodeForUrl (password) +
                       "&getmoods=" + QString::number(klujeWin->getMoods()->getHighestId()) + "&getmenus=1&getpickws=1" +
                       "&clientversion=KDE-KLuJe/" + VERSION + "&ver=1";
    createJob(postdata, this, SLOT(slotLoginResult(KIO::Job*)));
}

void
KLuJeIO::slotGetdaycounts()
{
    kdDebug(6969) << "KLuJeIO::slotGetdaycounts called" << endl;
    QString postdata = "mode=getdaycounts&user=" +
                       encodeForUrl (username) +
                       "&password=" + encodeForUrl (password);
    // TODO: I have *no* idea why not adding this trailing '&' makes things break
    postdata += "&";
    createJob(postdata, this, SLOT(slotGetdaycountsResult(KIO::Job*)));
}

void
KLuJeIO::slotGetevents(QDate date)
{
    kdDebug(6969) << "KLuJeIO::slotGetevents called" << endl;
    QString postdata = "mode=getevents&user=" +
                       encodeForUrl (username) +
                       "&hpassword=" + encodeForUrl (password) +
                       "&noprops=1&selecttype=day&year=" +
                       QString::number(date.year()) + "&month=" + QString::number(date.month()) +
                       "&day=" + QString::number(date.day()) + "&ver=1";
    // TODO: I have *no* idea why not adding this trailing '&' makes things break
    postdata += "&";
    createJob(postdata, this, SLOT(slotGeteventsByDateResult(KIO::Job*)));
}

void
KLuJeIO::slotGetevents(const QString& id)
{
    kdDebug(6969) << "KLuJeIO::slotGetevents called" << endl;
    QString postdata = "mode=getevents&user=" +
                       encodeForUrl (username) +
                       "&hpassword=" + encodeForUrl (password) +
                       "&selecttype=one&itemid=" + id + "&ver=1";
    createJob(postdata, this, SLOT(slotGeteventsByIdResult(KIO::Job*)));
}

void
KLuJeIO::slotGetFriends()
{
    kdDebug(6969) << "KLuJeIO::slotGetFriends called" << endl;
    QString postdata = "mode=getfriends&user=" +
                       encodeForUrl (username) +
                       "&hpassword=" + encodeForUrl (password) +
                       "&selecttype=one&includefriendof=1&includegroups=1&includebdays=1&ver=1";

    createJob(postdata, this, SLOT(slotGetFriendsResult(KIO::Job*)));
}

void
KLuJeIO::slotDataArrived(KIO::Job *job, const QByteArray& data)
{
//  kdDebug(6969) << "Network data arrived. Progress ID is: " << job->progressId() << endl;
    QPair<QString, bool> *buf = response_buffers.find(job->name());
    if (!buf)
    {
//    kdDebug(6969) << "No data yet for " << job->name() << endl;
        buf = new QPair<QString, bool> ();
        buf->second = FALSE;
        response_buffers.insert(job->name(), buf);
    }

    if (buf->second)
    {
        buf->second = FALSE;
        buf->first = QString::null;
    }

    if (data.size() == 0)
    {
        buf->second = TRUE;
//    kdDebug(6969) << "KLuJeIO::slotDataArrived -- Done. Received:\n" << buf->first << endl;
    }
    else
    {
//         kdDebug(6969) << QString().fill('=', 80) << endl
//                       << "|" << QString(data) << "|" << endl
//                       << QString().fill('=', 80) << endl;
        buf->first.append(data);
    }
}

void
KLuJeIO::slotPosteventResult(KIO::Job *job)
{
    if (job->error() != 0)
    {
        // something has gone wrong
        kdDebug(6969) << "KLuJeIO::slotPosteventResult -- error!" << endl;
        KMessageBox::error(klujeWin, i18n("The event was not submitted due to an IO error!"), i18n("Submission Failed"));
        klujeWin->statusBar()->message(i18n("Status: IO error"));
    } else
    {
        // process response_buffer
        QMap<QString, QString> dict = createResponseTable(response_buffers[job->name()]->first);
        response_buffers.remove(job->name());
        // clear dialog, *IF* submit was successful
        if (QString("OK") == dict.find("success").data())
        {
            kdDebug(6969) << "KLuJeIO::slotPosteventResult -- clearing out the edit boxes" << endl;
            klujeWin->slotClearSilently();
            klujeWin->statusBar()->message(i18n("Status: Entry submission successful"));
            KMessageBox::information(klujeWin, i18n("The event was successfully submitted!"), i18n("Entry Submitted"));
        } else
        {
            klujeWin->statusBar()->message(i18n("Status: %1").arg(dict["errmsg"]));
            KMessageBox::error(klujeWin,
                               i18n("An error occured while attempting to submit the event. "
                                    "The error was:\n\n%1").arg(dict["errmsg"]),
                               i18n("Submission Failed"));
        }
    }

    klujeWin->slotEnableUI(true);
}

void
KLuJeIO::slotLoginResult(KIO::Job *job)
{
    if (job->error())
    {
        // something has gone wrong
        kdDebug(6969) << "KLuJeIO::slotLoginResult -- error!" << endl;
        klujeWin->statusBar()->message("Status: IO error");
    }
    else if (klujeWin)
    {
        // process response_buffer
        QMap<QString, QString> dict = createResponseTable(response_buffers[job->name()]->first);
        response_buffers.remove(job->name());

//         if (dict.find("fastserver") != dict.end())
//         {
//             // If fast server is false, set to true and save config
//             if (!klujeWin->usesPayServer())
//             {
//                 kdDebug(6969) << "Allowed to use fast servers" << endl;
//                 klujeWin->setUsePayServer(true);
//             } else
//             {
//                 kdDebug(6969) << "Already using fast servers" << endl;
//             }
//         } else
//         {
//             if (klujeWin->usesPayServer())
//             {
//                 kdDebug(6969) << "Not using fast server anymore" << endl;
//                 klujeWin->setUsePayServer(false);
//             }
//         }

        klujeWin->statusBar()->message(QString("Status: ") + dict[QString("success")]);

        klujeWin->populateMenu(dict);

        QString message = dict["message"];
        if (!message.isEmpty())
        {
            KMessageBox::information(0, QString(message));
        }

        /**
         * Iterate over all shared journals and add them to the journalcombo
         * in the mainwin.  The nice thing about this is that the journals
         * are pre-sorted for us.
         */
        QStrList journalStrList;
        int sharedJournalCount = dict["access_count"].toInt();
        for(int i=1; i <= sharedJournalCount; ++i)
        {
            QString journal = dict[QString("access_%1").arg(i)];
            kdDebug(6969) << "Allowed to post to journal " << journal << endl;
            journalStrList.append(journal);
        }

        klujeWin->getJournalCombo()->clear();
        klujeWin->getJournalCombo()->insertItem(username);
        klujeWin->getJournalCombo()->insertStrList(journalStrList);

        klujeWin->getMoods()->populate(dict);

        klujeWin->getMoodCombo()->clear();
        klujeWin->getMoodCombo()->insertStrList(klujeWin->getMoods()->getSortedList());
        klujeWin->getMoodCombo()->completionObject()->insertItems(
            QStringList::fromStrList(klujeWin->getMoods()->getSortedList()));
        klujeWin->getMoodCombo()->setEditText("");
    }
}

void
KLuJeIO::slotGetdaycountsResult(KIO::Job *job)
{
    if (job->error())
    {
        // something went wrong
        kdDebug(6969) << "KLuJeIO::slotGetdaycountsResult -- error!" << endl;
        klujeWin->statusBar()->message(i18n("Status: IO error"));
    } else if (klujeWin)
    {
        // process response_buffer
        QMap<QString, QString> dict = createResponseTable(response_buffers[job->name()]->first);
        klujeWin->statusBar()->message(QString("Status: ") + dict[QString("success")]);
    }

    response_buffers.remove(job->name());
}

void
KLuJeIO::slotGeteventsByDateResult(KIO::Job *job)
{
    if (job->error() != 0)
    {
        // something went wrong
        kdDebug(6969) << "KLuJeIO::slotGeteventsByDateResult -- error!" << endl;
        klujeWin->statusBar()->message(i18n("Status: IO error"));
        return;
    }
    // process response_buffer
    QMap<QString, QString> dict = createResponseTable(response_buffers[job->name()]->first);
    response_buffers.remove(job->name());
    QMap<QString, QString>::iterator it = dict.begin();
    QMap<QString, QString>::iterator end = dict.end();
    QRegExp indexRegexp = QRegExp("^(\\w+)_(\\d+)_(\\w+)");
    QRegExp eventRegexp = QRegExp("^events_.*_event$");
    QRegExp itemidRegexp = QRegExp("^events_.*_itemid$");
    QRegExp subjectRegexp = QRegExp("^events_.*_subject$");
    QRegExp timeRegexp = QRegExp("^events_.*_eventtime$");
    QRegExp securityRegexp = QRegExp("^events_.*_security$");
    QRegExp allowmaskRegexp = QRegExp("^events_.*_allowmask$");
    QDict<LJEvent> eventsDict;
    eventsDict.setAutoDelete(true);
    while (it != end)
    {
        if (it.key().contains(indexRegexp))
        {
            indexRegexp.search(it.key());
            if (eventsDict.find(indexRegexp.cap(2)) == NULL) eventsDict.insert(indexRegexp.cap(2), new LJEvent());
            LJEvent* event = eventsDict.find(indexRegexp.cap(2)); // convenience
            if (it.key().contains(eventRegexp))
            {
                event->setText(decodeString(it.data()));
            }
            else if (it.key().contains(itemidRegexp))
            {
                event->setId ((it.data ()).toInt ());
            }
            else if (it.key().contains(subjectRegexp))
            {
                // here we must take in account strange LJ bug:
                // subject text in UTF-8 transmited as is, but post text is encoded
                event->setSubject (QString::fromUtf8 (it.data ().data ()));
            }
            else if(it.key().contains(timeRegexp))
            {
                QDateTime datetime = QDateTime::fromString(it.data(), Qt::ISODate);
                event->setDate(datetime.date());
                event->setTime(datetime.time());
            }
            else if (it.key().contains(securityRegexp))
            {
                event->setSecurity(it.data());
            }
            else if (it.key().contains(allowmaskRegexp))
            {
                event->setAllowMask(it.data());
            }
        }
        ++it;
    }

    emit newEvents(eventsDict);
}

void
KLuJeIO::slotGeteventsByIdResult(KIO::Job *job)
{
    if (job->error())
    {
        // something went wrong
        kdDebug(6969) << "KLuJeIO::slotGeteventsByIdResult -- error!" << endl;
        klujeWin->statusBar()->message(i18n("Status: IO error"));
    }
    else if (klujeWin)
    {
        QMap<QString, QString> dict = createResponseTable(response_buffers[job->name()]->first);
        QMap<QString, QString>::Iterator it = dict.begin();
        QMap<QString, QString>::Iterator end = dict.end();
        while (it != end)
        {
            // process response buffer
            ++it;
        }
    }

    response_buffers.remove(job->name());
}

QMap<QString, QString>
KLuJeIO::createResponseTable(const QString& data)
{
    QMap<QString, QString> dict;
    QStringList list = QStringList::split('\n', data, true);
    QStringList::Iterator it = list.begin();
    QStringList::Iterator end = list.end();

    while (it != end)
    {
        QString name = *it;
        ++it;

        if (it != end)
        {
            dict[name] = *it;
//        kdDebug(6969) << "Adding pair " << name << "::" << *it << endl;
            ++it;
        }
    }

    return dict;
}

void
KLuJeIO::slotConfigChanged(const KConfig &config)
{
    // Max Lapan: this is ugly, I know, but how could we change group?
    KConfig* cfgTmp = const_cast<KConfig*> (&config);
    cfgTmp->setGroup ("ServerInfo");

    serverUrl = config.readEntry("ServerUrl", "");
    username = config.readEntry("Username", "");
    password = config.readEntry("Password", "");
    kdDebug(6969) << "KLuJeIO::slotConfigChanged -- serverUrl == " << serverUrl << endl;
    kdDebug(6969) << "KLuJeIO::slotConfigChanged -- username == " << username << endl;
    kdDebug(6969) << "KLuJeIO::slotConfigChanged -- password == " << password << endl;
}

void
KLuJeIO::slotCheckFriends()
{
    kdDebug(6969) << "KLuJeIO::slotCheckFriends called" << endl;

    QString postdata =
        "mode=checkfriends&user=" + encodeForUrl (username) +
        "&hpassword=" + encodeForUrl (password) + "&lastupdate=" +
        encodeForUrl (klujeWin->getLastUpdate()) + "&ver=1&";

    kdDebug(6969) << "SENDING: " << postdata << endl;
    // send the request out to the livejournal server
    KIO::TransferJob *job = KIO::http_post(serverUrl, postdata.utf8(), false);
    job->addMetaData("content-type", "Content-Type: application/x-www-form-urlencoded");
    job->setName(getUniqueName().ascii());
//    maybeAddFastServerCookie(job);
    connect(job, SIGNAL(data(KIO::Job*, const QByteArray&)), SLOT(slotDataArrived(KIO::Job*, const QByteArray&)));
    connect(job, SIGNAL(result(KIO::Job*)), SLOT(slotCheckFriendsResult(KIO::Job*)));
}

void
KLuJeIO::slotCheckFriendsResult(KIO::Job *job)
{
    if (job->error() != 0)
    {
        // something has gone wrong
        kdDebug(6969) << "KLuJeIO::slotCheckFriendsResult -- error!" << endl;
        klujeWin->enablePolling();
    } else
    {
        kdDebug(6969) << "KLuJeIO::slotCheckFriendsResults==" << endl << response_buffers[job->name()]->first << endl;

        // process response_buffer
        QMap<QString, QString> dict = createResponseTable(response_buffers[job->name()]->first);

//    kdDebug(6969) << "Built response table" << endl;

        QString status = dict["success"];

        if (status != "OK")
        {
            kdDebug(6969) << "Check friends failed.  Status is " << status << ".  Pausing for 15 minutes" << endl;
            klujeWin->enablePolling();
            return;
        }

        QString newUpdate = dict[QString("lastupdate")];
        int newValue = QString((dict[QString("new")])).toInt();
        int interval = QString((dict[QString("interval")])).toInt();

        kdDebug(6969) << "Got checkfriends response: lastupdate=" <<
            newUpdate << " new=" << newValue << " interval=" << interval << endl;

        klujeWin->setInterval(interval);
        klujeWin->setLastUpdate(newUpdate);

        // Check to see if we have a new friend message
        if (newValue > 0)
        {
            kdDebug(6969) << "New friends detected, notifying" << endl;
            emit(checkfriendsNotify());
        } else
        {
            kdDebug(6969) << "No new friends, continuing poll" << endl;
            klujeWin->enablePolling();
        }
    }

    response_buffers.remove(job->name());
}

void KLuJeIO::slotSaveEvent(LJEvent event)
{
    kdDebug(6969) << "KLuJeIO::slotSaveEvent -- entered." << endl;
    QString postdata = "mode=editevent&user=" + encodeForUrl (username) +
                       "&hpassword=" + encodeForUrl (password) + "&itemid=" + QString::number(event.getId()) +
                       "&event=" + encodeForUrl (event.getText()) + "&subject=" +
                       encodeForUrl (event.getSubject()) + "&year=" +
                       QString::number(event.getDate().year()) + "&mon=" + QString::number(event.getDate().month()) +
                       "&day=" + QString::number(event.getDate().day()) +
                       "&min=" + QString::number(event.getTime().minute()) +
                       "&hour=" + QString::number(event.getTime().hour()) +
                       "&security=" + event.getSecurity();
    if (event.getSecurity() == "usemask") postdata += "&allowmask=" + event.getAllowMask();
    postdata += "&ver=1";
    kdDebug(6969) << "SENDING: " << postdata << endl;
    KIO::TransferJob *job = KIO::http_post(serverUrl, postdata.utf8(), false);
    job->addMetaData("content-type", "Content-Type: application/x-www-form-urlencoded");
    job->setName(getUniqueName().ascii());
//    maybeAddFastServerCookie(job);
    connect(job, SIGNAL(data(KIO::Job*, const QByteArray&)), SLOT(slotDataArrived(KIO::Job*, const QByteArray&)));
    connect(job, SIGNAL(result(KIO::Job*)), SLOT(slotSaveEventResult(KIO::Job*)));
}

void KLuJeIO::slotSaveEventResult(KIO::Job *job)
{
    if(job->error() != 0)
    {
        // something has gone wrong
        kdDebug(6969) << "KLuJeIO::slotSaveEventResult -- error!" << endl;
        klujeWin->enablePolling();
    }
    else
    {
        kdDebug(6969) << "KLuJeIO::slotSaveEventResult -- saved event successfully" << endl;
    }

    response_buffers.remove(job->name());
}

void
KLuJeIO::slotGetFriendsResult(KIO::Job* job)
{
    if (job->error() != 0)
    {
        // something has gone wrong
        kdDebug(6969) << "KLuJeIO::slotGetFriendsResult -- error!" << endl;
        KMessageBox::error(klujeWin,
                           i18n("Retrieval of your friends list failed unexpectedly!"),
                           i18n("Request Failed"));
        klujeWin->statusBar()->message(i18n("Status: IO error"));
    }
    else
    {
        QMap<QString, QString> dict = createResponseTable(response_buffers[job->name()]->first);
        response_buffers.remove(job->name());
        QString tmp = dict["success"];
        if (tmp.upper() != "OK")
        {
            QString errorMsg;
            tmp = dict["errmsg"];
            if (tmp.isNull())
            {
                errorMsg = i18n("Retrieval of your friends list failed unexpectedly!");
            }
            else
            {
                errorMsg = i18n("Retrieval of your friends list failed! "
                                "The LiveJournal server sent the following error:\n\n%1")
                           .arg(dict["errmsg"]);
            }
            KMessageBox::error(klujeWin, errorMsg, i18n("Request Failed"));
            klujeWin->statusBar()->message(i18n("Status: IO error"));
            return;
        }

//    uint numFriends = QString(dict["friend_count"]).toUInt();
//    uint numFriendOfs = QString(dict["friendof_count"]).toUInt();
//    uint maxGroupID = QString(dict["frgrp_maxnum"]).toUInt();

        QMap<QString, QString>::iterator it = dict.begin();
        QMap<QString, QString>::iterator end = dict.end();
        for (; it != end; ++it)
        {
            kdDebug(6969) << "KLuJeIO::slotGetFriendsResult: " << it.key() << " " << it.data() << endl;
        }
    }
}

// void KLuJeIO::maybeAddFastServerCookie(KIO::Job *job)
// {
//     if (klujeWin->usesPayServer())
//     {
//         kdDebug(6969) << "Using fast server for request" << endl;
//         job->addMetaData(QString("Cookie"), QString("Cookie: ljfastserver=1"));
//     }
// }

QString KLuJeIO::decodeString(const QString &str)
{
    QString retval = str;
    retval = retval.replace(QRegExp("\\+"), " ");
//    retval = KURL::decode_string(retval);
    kdDebug(6969) << "Decode: before: " << retval << endl;
    QUrl::decode (retval);
    kdDebug(6969) << "Decode: after: " << retval << endl;
    return retval;
}

KLuJeIO::~KLuJeIO()
{
}


QString KLuJeIO::encodeForUrl (const QString& str) const
{
    // QUrl::encode doesn't make sense. See:
    // http://www.livejournal.com/doc/server/ljp.csp.guide.intro.html
    QCString buf (str.utf8 ());
    QString res;

    for (unsigned int i = 0; i < buf.length (); i++) {
        unsigned char c = buf[i];

        if ((c >= '0' && c <= '9') || (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z'))
            res += c;
        else
            if (c == ' ')
                res += '+';
            else
                res += QString ("%") + QString ().sprintf ("%02X", c);
    }

    return res;
}

