/*
This file is part of KDE.

Copyright (c) 2008 Stephen Kelly <steveire@gmail.com>

This library is free software; you can redistribute it and/or modify it
under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This library is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.

You should have received a copy of the GNU Library General Public License
along with this library; see the file COPYING.LIB.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/


#include "attachmentstrategy.h"

#include <kmime/kmime_content.h>
#include <kdebug.h>

#include <QPair>

AttachmentStrategy::AttachmentStrategy ( QList< QPair< ContentStrategy*, Content* > > list, QObject *parent, ContentStrategy *s )
        : ContentStrategy ( parent, s )
{
    m_list = list;
    contentsProcessedCount = 0;
}

AttachmentStrategy::~AttachmentStrategy()
{

}

AttachmentStrategy::AttachmentStrategy ( QList< QPair< ContentStrategy*, Content* > > list, QObject *parent ) : ContentStrategy ( parent )
{
    m_list = list;
}

void AttachmentStrategy::slotProcessContent ( Content* content )
{
    mixedContent = new Content();
    mixedContent->contentType ( true )->setMimeType ( "multipart/mixed" );
    mixedContent->contentType()->setBoundary ( KMime::multiPartBoundary() );

    mixedContent->setContent ( content );

    contentsProcessedCount = 0;
//     foreach(QPair < ContentStrategy*, Content *>a, m_list)
    int listSize = m_list.size();
    if ( listSize > 0 ) {
        for ( int i = 0; i < listSize; i++ ) {
            QPair< ContentStrategy*, Content * > a = m_list.at ( i );
            connect ( a.first, SIGNAL ( contentsReady ( Content* ) ),
                      SLOT ( attachmentDone ( Content* ) ) );
            a.first->processContent ( a.second );
        }
        return;
    }
    kDebug() << "Error. No attachments.";
    emit contentsReady ( content );

}

void AttachmentStrategy::attachmentDone ( Content* content )
{
    mixedContent->addContent ( content );

    contentsProcessedCount++;
    if ( contentsProcessedCount == m_list.size() ) {
        emit contentsReady ( mixedContent );
    }

}

