/*
This file is part of KDE.

Copyright (c) 2008 Stephen Kelly <steveire@gmail.com>

This library is free software; you can redistribute it and/or modify it
under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This library is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.

You should have received a copy of the GNU Library General Public License
along with this library; see the file COPYING.LIB.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/

#ifndef ATTACHMENTSTRATEGY_H
#define ATTACHMENTSTRATEGY_H

#include "contentstrategy.h"

namespace KMime
{
class Content;
}

using namespace KMime;

#include <QPair>


/**
@brief Strategy for attachments to Content.
*/
class AttachmentStrategy : public ContentStrategy
{
    Q_OBJECT
public:
    /**
    @p list is a list of pairs. The pairs are consist of a Content and a Strategy to process it.
    Individual Content might be processed by different strategies.
    */

    AttachmentStrategy ( QList< QPair< ContentStrategy*, Content* > > list, QObject *parent = 0 );
    AttachmentStrategy ( QList< QPair< ContentStrategy*, Content* > > list, QObject *parent = 0, ContentStrategy *s = 0 );
    virtual ~AttachmentStrategy();


protected Q_SLOTS:
    virtual void slotProcessContent ( Content* content );
    void attachmentDone ( Content* );

private:
    QList< QPair< ContentStrategy*, Content* > > m_list;
    int contentsProcessedCount;
    Content *mixedContent;
};

#endif
