/*
This file is part of KDE.

Copyright (c) 2008 Stephen Kelly <steveire@gmail.com>

This library is free software; you can redistribute it and/or modify it
under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This library is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.

You should have received a copy of the GNU Library General Public License
along with this library; see the file COPYING.LIB.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/


#include "mailclient.h"

#include <kmime/kmime_message.h>
#include <kmime/kmime_content.h>

#include <kdatetime.h>

#include <KApplication>

#include <QTextBrowser>
#include <QFont>

// #include <QVBoxLayout>
#include <QGridLayout>
#include <QLabel>
#include <QPushButton>

#include <kdebug.h>

#include "contentstrategy.h"
#include "alternativestrategy.h"
#include "attachmentstrategy.h"
#include "gpgsigningstrategy.h"
#include "gpgencryptionstrategy.h"
#include "relatedstrategy.h"
#include "nullstrategy.h"


using namespace KMime;

MailClient::MailClient ( QWidget *parent ) : QWidget ( parent )
{
    QGridLayout *mainLayout = new QGridLayout( this );

    QGridLayout *layout = new QGridLayout( this );

    int row = 0;
    int column = 0;

    QPushButton *b0 = new QPushButton ( "Mail0", this );
    layout->addWidget ( b0, row, column );
    connect ( b0, SIGNAL ( clicked() ), SLOT ( composeMail0() ) );

    QLabel *l0 = new QLabel( "A plain text email", this );
    layout->addWidget ( l0, row++, column+1 );

    QPushButton *b1 = new QPushButton ( "Mail1", this );
    layout->addWidget ( b1, row, column );
    layout->setRowStretch(row, 0);
    connect ( b1, SIGNAL ( clicked() ), SLOT ( composeMail1() ) );

    QLabel *l1 = new QLabel( "A signed plain text email", this );
    layout->addWidget ( l1, row++, column+1 );

    QPushButton *b2 = new QPushButton ( "Mail2", this );
    layout->addWidget ( b2, row, column );
    layout->setRowStretch(row, 0);
    connect ( b2, SIGNAL ( clicked() ), SLOT ( composeMail2() ) );

    QLabel *l2 = new QLabel( "A plain text and html alternative email", this );
    layout->addWidget ( l2, row++, column+1 );

    QPushButton *b3 = new QPushButton ( "Mail3", this );
    layout->addWidget ( b3, row, column );
    layout->setRowStretch(row, 0);
    connect ( b3, SIGNAL ( clicked() ), SLOT ( composeMail3() ) );

    QLabel *l3 = new QLabel( "A signed encrypted plain text email", this );
    layout->addWidget ( l3, row++, column+1 );

    QPushButton *b4 = new QPushButton ( "Mail4", this );
    layout->addWidget ( b4, row, column );
    layout->setRowStretch(row, 0);
    connect ( b4, SIGNAL ( clicked() ), SLOT ( composeMail4() ) );

    QLabel *l4 = new QLabel( "A plain text and html alternative email with an <br /> encrypted plain text attachment and an <br /> encrypted+signed plain text attachment", this );
    layout->addWidget ( l4, row++, column+1 );

    QPushButton *b5 = new QPushButton ( "Mail5", this );
    layout->addWidget ( b5, row, column );
    layout->setRowStretch(row, 0);
    connect ( b5, SIGNAL ( clicked() ), SLOT ( composeMail5() ) );

    QLabel *l5 = new QLabel( "An html email with related image and<br /> related html document [RFC2557]", this );
    layout->addWidget ( l5, row++, column+1 );

    QPushButton *b6 = new QPushButton ( "Mail6", this );
    layout->addWidget ( b6, row, column );
    layout->setRowStretch(row, 0);
    connect ( b6, SIGNAL ( clicked() ), SLOT ( composeMail6() ) );

    QLabel *l6 = new QLabel( "A plain text email with RFC2557 alternative <br /> which has one related image", this );
    layout->addWidget ( l6, row++, column+1 );

    QPushButton *b7 = new QPushButton ( "Mail7", this );
    layout->addWidget ( b7, row, column );
    layout->setRowStretch(row, 0);
    connect ( b7, SIGNAL ( clicked() ), SLOT ( composeMail7() ) );

    QLabel *l7 = new QLabel( "Multipart related with nested related content", this );
    layout->addWidget ( l7, row++, column+1 );

    mainLayout->addLayout(layout, 0, 0);


    qtb = new QTextBrowser( this );
    qtb->setFontFamily( "Courier" );
    mainLayout->addWidget( qtb, 0, 1);

    QPushButton *quit = new QPushButton ( "Quit", this );
    mainLayout->addWidget ( quit, 1, 0, 1, 2);
    connect ( quit, SIGNAL ( clicked() ), SLOT ( onQuit() ) );

    setLayout ( mainLayout );

}



void MailClient::composeMail0()
{

    Content *c1 = new Content();
    c1->contentType ( true )->setMimeType ( "text/plain" );
    c1->setBody ( "plain text body" );

    sendMail ( c1 );

}


void MailClient::composeMail1()
{

    Content *c1 = new Content();
    c1->contentType ( true )->setMimeType ( "text/plain" );
    c1->setBody ( "plain text body" );

    ContentStrategy *cs = new GpgSigningStrategy ( this );

    connect ( cs, SIGNAL ( contentsReady ( Content* ) ), SLOT ( sendMail ( Content* ) ) );

    cs->processContent ( c1 );

}


void MailClient::composeMail2()
{
    Content *c1 = new Content();
    c1->contentType ( true )->setMimeType ( "text/plain" );
    c1->setBody ( "plain text body" );

    Content *c2 = new Content();
    c2->contentType ( true )->setMimeType ( "text/html" );
    c2->setBody ( "rich <b>text</b> body" );

    QList< QPair< ContentStrategy*, Content* > > alternatives;

    QPair< ContentStrategy*, Content* > al1;
    al1.first = new NullStrategy ( this );
    al1.second = c2;

    alternatives << al1;


    ContentStrategy *cs = new AlternativeStrategy ( alternatives, this );

    connect ( cs, SIGNAL ( contentsReady ( Content* ) ), SLOT ( sendMail ( Content* ) ) );

    cs->processContent ( c1 );

}

void MailClient::composeMail3()
{
    Content *c1 = new Content();
    c1->contentType ( true )->setMimeType ( "text/plain" );
    c1->setBody ( "plain text body" );

    ContentStrategy *cs = new GpgSigningStrategy ( this, new GpgEncryptionStrategy ( this ) );

    connect ( cs, SIGNAL ( contentsReady ( Content* ) ), SLOT ( sendMail ( Content* ) ) );

    cs->processContent ( c1 );

}

void MailClient::composeMail4()
{
    Content *c1 = new Content();
    c1->contentType ( true )->setMimeType ( "text/plain" );
    c1->setBody ( "plain text body" );

    Content *c2 = new Content();
    c2->contentType ( true )->setMimeType ( "text/html" );
    c2->setBody ( "rich <b>text</b> body" );

    QList< QPair< ContentStrategy*, Content* > > alternatives;

    QPair< ContentStrategy*, Content* > al1;
    al1.first = new NullStrategy ( this );
    al1.second = c2;

    alternatives << al1;

    ContentStrategy *mainContentStrategy = new AlternativeStrategy ( alternatives, this );

    Content *c3 = new Content();
    c3->contentType ( true )->setMimeType ( "text/plain" );
    c3->setBody ( "attached content" );

    Content *c4 = new Content();
    c4->contentType ( true )->setMimeType ( "text/plain" );
    c4->setBody ( "more attached content" );

    QList< QPair< ContentStrategy*, Content* > > attachments;

    QPair< ContentStrategy*, Content* > a1;
    a1.first = new GpgEncryptionStrategy ( this );
    a1.second = c3;

    QPair< ContentStrategy*, Content* > a2;
    a2.first = new GpgSigningStrategy ( this, new GpgEncryptionStrategy ( this ) );
    a2.second = c4;

    attachments << a1 << a2;

    ContentStrategy *cs = new AttachmentStrategy ( attachments, this, mainContentStrategy );

    connect ( cs, SIGNAL ( contentsReady ( Content* ) ), SLOT ( sendMail ( Content* ) ) );

    cs->processContent ( c1 );

}

void MailClient::composeMail5()
{

    Content *c1 = new Content();
    c1->contentType ( true )->setMimeType ( "text/html" );
    c1->setBody ( "html body with inline images <img src=\"images/12fd3.png\" /> and links to <a href=\"related/other.html\">related</a> content" );

    Content *c2 = new Content();
    c2->contentType ( true )->setMimeType ( "image/png" );
    //TODO: implement this in kmime:
    c2->contentLocation ( true )->from7BitString ( "images/12fd3.png" );
    c2->setBody ( "base64 encoded image" );

    Content *c3 = new Content();
    c3->contentType ( true )->setMimeType ( "text/html" );
    //TODO: implement this in kmime:
//     c3->contentLocation(true)->setLocation( "related/other.html" );
    c3->setBody ( "related content" );


    QList< QPair< ContentStrategy*, Content* > > related;

    QPair< ContentStrategy*, Content* > re1;
    re1.first = new NullStrategy ( this );
    re1.second = c2;

    QPair< ContentStrategy*, Content* > re2;
    re2.first = new NullStrategy ( this );
    re2.second = c3;

    related << re1 << re2;

    ContentStrategy *cs = new RelatedStrategy ( related, this );

    connect ( cs, SIGNAL ( contentsReady ( Content* ) ), SLOT ( sendMail ( Content* ) ) );

    cs->processContent ( c1 );

}

void MailClient::composeMail6()
{
    Content *c1 = new Content();
    c1->contentType ( true )->setMimeType ( "text/plain" );
    c1->setBody ( "plain text body" );

    Content *c2 = new Content();
    c2->contentType ( true )->setMimeType ( "text/html" );
    c2->setBody ( "rich <b>text</b> body with related image: <img src=\"images/foo.png\" />" );

    Content *c3 = new Content();
    c3->contentType ( true )->setMimeType ( "image/png" );
    //TODO: implement this in kmime:
    c3->contentLocation ( true )->from7BitString ( "images/foo.png" );
    c3->setBody ( "base64 encoded image" );


    QList< QPair< ContentStrategy*, Content* > > alternatives;


    QList< QPair< ContentStrategy*, Content* > > related;

    QPair< ContentStrategy*, Content* > re1;
    re1.first = new NullStrategy ( this );
    re1.second = c3;

    related << re1;

    QPair< ContentStrategy*, Content* > al1;
    al1.first = new RelatedStrategy ( related, this ); // QList< Content* >() << c3
    al1.second = c2;

    alternatives << al1;


    ContentStrategy *cs = new AlternativeStrategy ( alternatives, this );

    connect ( cs, SIGNAL ( contentsReady ( Content* ) ), SLOT ( sendMail ( Content* ) ) );

    cs->processContent ( c1 );

}


void MailClient::composeMail7()
{
    Content *c2 = new Content();
    c2->contentType ( true )->setMimeType ( "text/html" );
    c2->setBody ( "rich <b>text</b> body with related <a href=\"related/other.html\">document</a>." );

    Content *c3 = new Content();
    c3->contentType ( true )->setMimeType ( "image/png" );
    //TODO: implement this in kmime:
    c3->contentLocation ( true )->from7BitString ( "images/foo.png" );
    c3->setBody ( "base64 encoded image" );

    Content *c4 = new Content();
    c4->contentType ( true )->setMimeType ( "text/html" );
    c4->setBody ( "related content with inline <img src=\"images/foo.png\" /> image." );



    QList< QPair< ContentStrategy*, Content* > > related;


    QList< QPair< ContentStrategy*, Content* > > relatedOther;

    QPair< ContentStrategy*, Content* > re0;
    re0.first = new NullStrategy ( this );
    re0.second = c3;

    relatedOther << re0;


    QPair< ContentStrategy*, Content* > re1;
    RelatedStrategy *rs = new RelatedStrategy (relatedOther, this );
    rs->setLocation( "related/other.html" );
    re1.first = rs;
    re1.second = c4;

    related << re1;

    ContentStrategy *cs = new RelatedStrategy ( related, this );

    connect ( cs, SIGNAL ( contentsReady ( Content* ) ), SLOT ( sendMail ( Content* ) ) );

    cs->processContent ( c2 );

}




void MailClient::sendMail ( Content* content )
{
//     kDebug() << content->encodedContent();
    Message *message = new Message();
    KMime::Headers::From *fromAddress = new KMime::Headers::From();
    fromAddress->addAddress ( "konqi@kde.org" );
    message->setHeader ( fromAddress );

    KMime::Headers::To *toAddress = new KMime::Headers::To();
    toAddress->addAddress ( "katie@kde.org" );
    message->setHeader ( toAddress );

    message->date()->setDateTime ( KDateTime::currentLocalDateTime() );

    message->subject()->from7BitString ( "Topic" );

    message->setContent ( content );
    message->assemble();

//     kDebug() << endl << message->encodedContent();

//     kDebug() << qtb->currentFont().family();

    qtb->setPlainText( message->encodedContent() );

}

void MailClient::onQuit()
{
    KApplication::kApplication()->quit();
}

