/*
This file is part of KDE.

Copyright (c) 2008 Stephen Kelly <steveire@gmail.com>

This library is free software; you can redistribute it and/or modify it
under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This library is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.

You should have received a copy of the GNU Library General Public License
along with this library; see the file COPYING.LIB.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/

#include "relatedstrategy.h"

#include <kmime/kmime_content.h>
#include <kdebug.h>

RelatedStrategy::RelatedStrategy ( QList< QPair< ContentStrategy*, Content* > > list, ContentStrategy *s, QObject *parent )
        : ContentStrategy ( parent, s )
{
    m_list = list;
    contentLocation = QString();
}

RelatedStrategy::~RelatedStrategy()
{

}

RelatedStrategy::RelatedStrategy ( QList< QPair< ContentStrategy*, Content* > > list, QObject *parent )
        : ContentStrategy ( parent )
{
    m_list = list;
    contentLocation = QString();
}

void RelatedStrategy::setLocation( const QString &loc )
{
    contentLocation = loc;
}

void RelatedStrategy::slotProcessContent ( Content* content )
{
    relatedContent = new Content();
    relatedContent->contentType ( true )->setMimeType ( "multipart/related" );
    relatedContent->contentType()->setBoundary ( KMime::multiPartBoundary() );

    if (!contentLocation.isEmpty())
    {
        relatedContent->contentLocation( true )->fromUnicodeString( contentLocation, "UTF8" );
    }

    relatedContent->addContent ( content );

//     foreach ( Content *c, m_list ) {
//         relatedContent->addContent ( c );
//     }
//     emit contentsReady ( relatedContent );

    contentsProcessedCount = 0;
    int listSize = m_list.size();
    for ( int i = 0; i < listSize; i++ ) {
        QPair< ContentStrategy*, Content * > a = m_list.at ( i );
        connect ( a.first, SIGNAL ( contentsReady ( Content* ) ),
                SLOT ( relatedDone ( Content* ) ) );
                a.first->processContent ( a.second );
    }
}

void RelatedStrategy::relatedDone ( Content* content )
{
    relatedContent->addContent ( content );

    contentsProcessedCount++;
    if ( contentsProcessedCount == m_list.size() ) {
        emit contentsReady ( relatedContent );
    }

}

