/***************************************************************************
   Copyright (C) 2005 by Marco Gulino <marco.gulino@gmail.com>
   Copyright (C) 2005 by Stefan Bogner <bochi@kmobiletools.org>
   Copyright (C) 2007 by Matthias Lechner <matthias@lmme.de>
   Copyright (C) 2010 by Quentin Denis <quentin@links2linux.de>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "kmobiletoolsgammu_engine.h"

#include <QtGui/QVBoxLayout>
#include <QtGui/QLabel>
#include <QtCore/QMutexLocker>
#include <QtCore/QTimer>
#include <KDebug>
#include <kgenericfactory.h>

// job includes
//#include "initphonejob.h"
/*#include "phoneinfosjob.h"
#include "pollstatusjob.h"*/
#include "fetchaddressbookjob.h"
// #include "editaddressbookjob.h"
// #include "fetchsmsjob.h"
// #include "deletesmsjob.h"
// #include "storesmsjob.h"
// #include "sendsmsjob.h"

#include "gammusms.h"

#include <kdebug.h>

kmobiletoolsGammu_engine::kmobiletoolsGammu_engine(QObject *parent, const QString& deviceName)
 : EngineXP(parent, deviceName)
{
    kDebug() <<"Initialized Gammu engine";

    m_device = new Device( this->deviceName() );
}

kmobiletoolsGammu_engine::~kmobiletoolsGammu_engine()
{
}

#include "kmobiletoolsgammu_engine.moc"

// QString kmobiletoolsGammu_engine::engineDesktopName() const {
//     return "Gammu Engine";
// }
// 
// 
// QString kmobiletoolsGammu_engine::engineLibName() const {
//     return "kmobiletools_gammu";
// }
// 
// 
// bool kmobiletoolsGammu_engine::pdu() {
//     return true;
// }





QList<QWizardPage*> kmobiletoolsGammu_engine::pageList() const {
    QList<QWizardPage*> wizardPages;

    QWizardPage* newPage = new QWizardPage;
    newPage->setTitle( QString( "Gammu page ;-)" ) );
    newPage->setSubTitle( QString( "This is all gammu" ) );
    QLabel *goOn = new QLabel( newPage );
    goOn->setText( "Nothing to see here, go on!" );

    wizardPages.append( newPage );

    return wizardPages;
}

void kmobiletoolsGammu_engine::connectDevice()
{
    kDebug() <<  QString( "Initialized device %1" ).arg( deviceName() );
    kDebug() <<"Gammu engine: initPhone() called";
//emit jobCreated( new InitPhoneJob( m_device, this, this->name() ) );
m_device->initPhone();
    m_initialized = true;
    emit deviceConnected();
}

void kmobiletoolsGammu_engine::disconnectDevice()
{
    if( !m_initialized ) {
        status( QString( "Engine is not initialized yet." ) );
        emit deviceDisconnected();
        return;
    }

    m_initialized = false;
    status( QString( "Shut down succeeded ;-)" ));
    QTimer::singleShot( 5*1000, this, SIGNAL(deviceDisconnected()) );
}


int kmobiletoolsGammu_engine::signalStrength() const
{
    if( !m_statusInformationFetched )
        return -1;

    return 99;
}

int kmobiletoolsGammu_engine::charge() const
{
    if( !m_statusInformationFetched )
        return -1;

    return 70;
}

KMobileTools::Status::PowerSupplyType kmobiletoolsGammu_engine::powerSupplyType() const
{
    if( !m_statusInformationFetched )
        return KMobileTools::Status::Unknown;

    return KMobileTools::Status::ACAdaptor;
}

bool kmobiletoolsGammu_engine::ringing() const
{
    return false;
}

QString kmobiletoolsGammu_engine::networkName() const
{
    if( !m_informationFetched )
        return QString( "Unknown" );

    return "Vodafone";
}

QString kmobiletoolsGammu_engine::manufacturer() const
{
    if( !m_informationFetched )
        return QString( "Unknown" );

    return "Nokia";
}

KMobileTools::Information::Manufacturer kmobiletoolsGammu_engine::manufacturerID() const
{
    if( !m_informationFetched )
        return KMobileTools::Information::Unknown;

    return KMobileTools::Information::Nokia;
}

QString kmobiletoolsGammu_engine::model() const
{
    if( !m_informationFetched )
        return QString( "Unknown" );

    return "3650";
}

QString kmobiletoolsGammu_engine::imei() const
{
    if( !m_informationFetched )
        return QString( "Unknown" );

    return "123456789";
}

QString kmobiletoolsGammu_engine::revision() const
{
    if( !m_informationFetched )
        return QString( "Unknown" );

    return "V4.3";
}

void kmobiletoolsGammu_engine::fetchStatusInformation()
{
    if( !m_initialized ) {
        status( QString( "Engine is not initialized yet." ) );
        return;
    }

    if( !m_statusInformationFetched ) {
        m_statusInformationFetched = true;

        emit signalStrengthChanged( signalStrength() );
        emit chargeChanged( charge() );
        emit chargeTypeChanged( powerSupplyType() );
        emit phoneRinging( ringing() );
    }

    status( "Status information fetched." );
    emit statusInformationFetched();
}

void kmobiletoolsGammu_engine::fetchInformation()
{
    if( !m_initialized ) {
        status( QString( "Engine is not initialized yet." ) );
        return;
    }

    if( !m_informationFetched ) {
        m_informationFetched = true;

        emit networkNameChanged( networkName() );
    }

    status( "Mobile phone information fetched." );
    emit informationFetched();
}

void kmobiletoolsGammu_engine::populateAddressbook()
{
    // building sim contacts
    for( int i=1; i<=10; i++ ) {
        KMobileTools::AddressbookEntry addressee;
        addressee.setName( QString( "Sim contact %1" ).arg( QString::number( i ) ) );
        addressee.insertEmail( QString( "dummy@kmobiletools.org" ) );
        addressee.setMemorySlot( KMobileTools::AddressbookEntry::Sim );
        addAddressee( addressee );
    }

    // building phone contacts
    for( int i=1; i<=5; i++ ) {
        KMobileTools::AddressbookEntry addressee;
        addressee.setName( QString( "Phone contact %1" ).arg( QString::number( i ) ) );
        addressee.insertEmail( QString( "dummy@kmobiletools.org" ) );
        addressee.setMemorySlot( KMobileTools::AddressbookEntry::Phone );
        addAddressee( addressee );
    }
}

void kmobiletoolsGammu_engine::fetchAddressbook()
{
    if( !m_initialized ) {
        status( QString( "Engine is not initialized yet." ) );
        return;
    }

    // first fetch?
    if( !m_addressbookFetched )
        populateAddressbook();

    for( int i=0; i<m_addedAddressees.size(); i++ ) {
        m_addressbook.append( m_addedAddressees.at( i ) );
        emit addresseeAdded( m_addedAddressees.at( i ) );
    }
    m_addedAddressees.clear();

    for( int i=0; i<m_addressbook.size(); i++ ) {
        for( int j=0; j<m_removedAddressees.size(); j++ ) {
            if( m_addressbook.at( i ).uid() == m_removedAddressees.at( j ) ) {
                emit addresseeRemoved( m_addressbook.at( i ) );
                m_addressbook.removeAt( i );
            }
        }
    }
    m_removedAddressees.clear();

    emit jobCreated( new FetchAddressbookJob( this ) );

    m_addressbookFetched = true;
    status( "Address book fetched." );
    emit addressbookFetched();
}

KMobileTools::AddressbookEntry::MemorySlots kmobiletoolsGammu_engine::availableMemorySlots() const
{
    if( !m_addressbookFetched )
        return KMobileTools::AddressbookEntry::Unknown;

    return ( KMobileTools::AddressbookEntry::Phone | KMobileTools::AddressbookEntry::Sim );
}

KMobileTools::Addressbook kmobiletoolsGammu_engine::addressbook() const
{
    if( !m_addressbookFetched )
        return KMobileTools::Addressbook();

    return m_addressbook;
}

void kmobiletoolsGammu_engine::addAddressee( const KMobileTools::AddressbookEntry& addressee )
{
    QMutexLocker locker( &m_mutex );

    m_addedAddressees.append( addressee );

    status( QString( "Enqueued addressee \"%1\" to be added." ).arg( addressee.name() ) );
}

void kmobiletoolsGammu_engine::editAddressee( const KMobileTools::AddressbookEntry& oldAddressee,
                                const KMobileTools::AddressbookEntry& newAddressee )
{
    QMutexLocker locker( &m_mutex );

    m_addedAddressees.append( newAddressee );
    m_removedAddressees << oldAddressee.uid();

    status( QString( "Enqueued addressee \"%1\" to be edited." ).arg( newAddressee.name() ) );
}

void kmobiletoolsGammu_engine::removeAddressee( const KMobileTools::AddressbookEntry& addressee )
{
    QMutexLocker locker( &m_mutex );

    m_removedAddressees << addressee.uid();
    status( QString( "Enqueued addressee \"%1\" to be removed." ).arg( addressee.name() ) );
}





// int kmobiletoolsGammu_engine::availPbSlots()
// {
//     /// @todo make a job out of this
//     QStringList phonebookSlots = m_device->phonebookSlots();
// 
//     int retval = 0;
//     if ( phonebookSlots.contains("ME") )
//         retval+= PB_Phone;
// 
//     if ( phonebookSlots.contains("SM") )
//         retval+= PB_SIM;
// 
//     if ( phonebookSlots.contains("TA") )
//         retval+= PB_DataCard;
// 
//     return retval;
// }
// 
// 
// void kmobiletoolsGammu_engine::queryClose() {
//     m_device->terminatePhone();
// }


// void kmobiletoolsGammu_engine::processSlot( Job* p_job ) {
//     kmobiletoolsEngine::processSlot(p_job);
// 
//     KMobileTools::JobXP *job=(kmobiletoolsJob *) p_job;
// 
//     int unreadSMS = 0;
//     int totalSMS = 0;
// 
//     switch( job->jobType() ){
//         //
//         // Misc phone jobs
//         //
//         case kmobiletoolsJob::initPhone:
//             b_connected = m_device->phoneConnected();
//             if( b_connected )
//                 emit connected();
//             else
//                 emit disconnected();
//             break;
// 
//         case kmobiletoolsJob::fetchPhoneInfos:
//             s_manufacturer=( (PhoneInfosJob*) job )->manufacturer();
//             if( s_manufacturer.contains( "Nokia", false ) )
//                 i_manufacturer = Nokia;
//             s_model=( (PhoneInfosJob*) job )->model();
//             s_revision=( (PhoneInfosJob*) job )->revision();
//             s_imei=( (PhoneInfosJob*) job )->imei();
//             break;
// 
//         case kmobiletoolsJob::pollStatus:
//             emit charge( ( (PollStatusJob*) job )->phoneCharge() );
//             emit signal( ( (PollStatusJob*) job )->phoneSignal() );
//             emit networkName( ( (PollStatusJob*) job )->networkName() );
//             emit isRinging(  ( (PollStatusJob*) job )->ringing() );
// 
//             unreadSMS = ( (PollStatusJob*) job )->unreadSMS();
//             totalSMS = ( (PollStatusJob*) job )->totalSMS();
// 
//             if( unreadSMS != s_newSMS || totalSMS != s_totalSMS ) {
//                 s_newSMS = unreadSMS;
//                 s_totalSMS = totalSMS;
//                 emit updateInfoPage(0);
//             }
//             break;
// 
//         case kmobiletoolsJob::testPhoneFeatures:
//             break;
// 
//         case kmobiletoolsJob::syncDateTimeJob:
//             break;
// 
//         case kmobiletoolsJob::selectCharacterSet:
//             break;
// 
//         //
//         // Address book jobs
//         //
//         case kmobiletoolsJob::fetchAddressBook:
//             m_addresseeList = ((FetchAddressBookJob*) job)->addresseeList();
//             p_addresseeList= (ContactPtrList*) &m_addresseeList;
//             emit phoneBookUpdated();
//             break;
// 
//         case kmobiletoolsJob::addAddressee:
//             // nothing to do here
//             break;
// 
//         case kmobiletoolsJob::delAddressee:
//             // nothing to do here
//             break;
// 
//         case kmobiletoolsJob::editAddressee:
//             // nothing to do here
//             break;

        //
        // SMS jobs
        //
//         case kmobiletoolsJob::fetchSMS:
//             if( ((FetchSMSJob*) job)->smsList() == 0 )
//                 break;
// 
//             p_smsList->sync( ((FetchSMSJob*) job)->smsList() );
//             emit smsListUpdated();
//             break;
// 
//         case kmobiletoolsJob::smsFolders:
//             break;
// 
//         case kmobiletoolsJob::sendSMS:
//             // nothing to do here
//             break;
// 
//         case kmobiletoolsJob::storeSMS:
//             // nothing to do here
//             break;
// 
//         case kmobiletoolsJob::sendStoredSMS:
//             // nothing to do here
//             break;
// 
//         case kmobiletoolsJob::delSMS:
//             // nothing to do here
//             break;
// 
//         case kmobiletoolsJob::selectSMSSlot:
//             break;

        //
        // Calendar jobs
        //
//         case kmobiletoolsJob::fetchKCal:
//             break;
//    }
//}


// void kmobiletoolsGammu_engine::probePhone() {
//     kDebug() <<"Gammu engine: probePhone() called";
//     b_connected = false;
// 
//     // since we don't use DeviceList, we need to call initPhone() ourselves
//     initPhone();
// 
//     /// @todo implement me
// }


// void kmobiletoolsGammu_engine::initPhone()
// {
//     kDebug() <<"Gammu engine: initPhone() called";
//     emit jobCreated( new InitPhoneJob( m_device, this, this->name() ) );
// }

// DeviceInfos kmobiletoolsGammu_engine::probeDevice( ThreadWeaver::Job *job, bool fullprobe,
//                             const QString &deviceName,
//                             const QStringList &params ) const {
//     Q_UNUSED(job)
//     Q_UNUSED(fullprobe)
//     Q_UNUSED(params)
// 
//     bool bluetooth = false;
//     if( deviceName.contains( "bluetooth" ) )
//         bluetooth = true;
// 
// 
// 
//     DeviceInfos deviceInfos;
//     if( !m_device->phoneConnected() ) {
//         // try to initialize phone
//         m_device->initPhone();
//         if( !m_device->phoneConnected() )
//             return deviceInfos;
//     }
// 
//     kDebug() <<"Gammu engine: device name:" << deviceName;
// 
// 
//     deviceInfos.setFoundPath( deviceName );
// 
//     deviceInfos.setModel( m_device->model() );
//     deviceInfos.setRevision( m_device->version() );
//     deviceInfos.setImei( m_device->imei() );
//     deviceInfos.setManufacturer( m_device->manufacturer() );
//     deviceInfos.setPbSlots( m_device->phonebookSlots() );
//     deviceInfos.setSMSCenter( m_device->smsc() );
// 
//     QValueList<QString> utf8;
//     utf8 << "utf8";
//     deviceInfos.setCharsets( utf8 );
// 
//     return deviceInfos;
// }


// void kmobiletoolsGammu_engine::dial( DialActions dialActions, const QString &number ) {
//     switch( dialActions ) {
//         case DIAL_DIAL:
//             m_device->dial( number );
//             break;
// 
//         case DIAL_HANGUP:
//             m_device->hangup();
//             break;
// 
//         default:
//             kDebug() <<"Gammu engine: dialActions ==" << dialActions;
//             break;
//     }
// }


// QStringList kmobiletoolsGammu_engine::encodings() {
//     QStringList encodings;
//     encodings << "utf8";
//     return encodings;
// }


    /******************************************************
     * Phone information related methods                  *
     ******************************************************/

// void kmobiletoolsGammu_engine::pollPhoneStatus() {
//     if( m_device->phoneConnected() )
//         enqueueJob( new PollStatusJob( m_device, this, this->name() ) );
// }
// 
// 
// void kmobiletoolsGammu_engine::getPhoneInfos()
// {
//     if( m_device->phoneConnected() )
//         enqueueJob( new PhoneInfosJob( m_device, this, this->name() ) );
// }


    /******************************************************
     * SMS related methods                                *
     ******************************************************/

// void kmobiletoolsGammu_engine::retrieveSMSList()
// {
//     enqueueJob( new FetchSMSJob( m_device, this, this->name() ) );
// }
// 
// 
// void kmobiletoolsGammu_engine::slotDelSMS( SMS* sms ) {
//     enqueueJob( new DeleteSMSJob( (GammuSMS*) sms, m_device, this, this->name() ) );
// }
// 
// 
// void kmobiletoolsGammu_engine::slotStoreSMS( SMS* sms ) {
//     enqueueJob( new StoreSMSJob( sms, m_device, this, this->name() ) );
// }
// 
// 
// void kmobiletoolsGammu_engine::slotStoreSMS( const QString &number,
//                                              const QString &text ) {
//     enqueueJob( new StoreSMSJob( number, text, m_device, this, this->name() ) );
// }
// 
// 
// void kmobiletoolsGammu_engine::slotSendSMS( const QString &number,
//                                             const QString &text ) {
//     enqueueJob( new SendSMSJob( kmobiletoolsJob::sendSMS, number, text,
//                                 m_device, this, this->name() ) );
// }
// 
// 
// void kmobiletoolsGammu_engine::slotSendSMS( SMS* sms ) {
//     enqueueJob( new SendSMSJob( kmobiletoolsJob::sendSMS, (GammuSMS*) sms,
//                                 m_device, this, this->name() ) );
// }
// 
// 
// void kmobiletoolsGammu_engine::slotSendStoredSMS( SMS* sms ) {
//     enqueueJob( new SendSMSJob( kmobiletoolsJob::sendStoredSMS, (GammuSMS*) sms,
//                                 m_device, this, this->name() ) );
// }

    /******************************************************
     * Phone book related methods                         *
     ******************************************************/

// void kmobiletoolsGammu_engine::fetchAddressbook() {
//     emit jobCreated( new FetchAddressBookJob( m_device, this, this->name() ) );
// }
void kmobiletoolsGammu_engine::fetchAddressbook()
{
    if( !m_initialized ) {
        kDebug() << "Engine is not initialized yet.";
        return;
    }

    // first fetch?
    if( !m_addressbookFetched )
        populateAddressbook();

    for( int i=0; i<m_addedAddressees.size(); i++ ) {
        m_addressbook.append( m_addedAddressees.at( i ) );
        emit addresseeAdded( m_addedAddressees.at( i ) );
    }
    m_addedAddressees.clear();

    for( int i=0; i<m_addressbook.size(); i++ ) {
        for( int j=0; j<m_removedAddressees.size(); j++ ) {
            if( m_addressbook.at( i ).uid() == m_removedAddressees.at( j ) ) {
                emit addresseeRemoved( m_addressbook.at( i ) );
                m_addressbook.removeAt( i );
            }
        }
    }
    m_removedAddressees.clear();

    emit jobCreated( new FetchAddressbookJob( m_device, this ) );

    m_addressbookFetched = true;
    kDebug() << "Address book fetched.";
    emit addressbookFetched();
}


// void kmobiletoolsGammu_engine::slotAddAddressee( QValueList<KABC::Addressee>* list ) {
//     emit jobCreated( new EditAddressBookJob
//                 (  kmobiletoolsJob::addAddressee, list, m_device, this, this->name() )
//               );
// 
//     emit jobCreated( new FetchAddressBookJob( m_device, this, this->name() ) );
// }
// 
// 
// void kmobiletoolsGammu_engine::slotDelAddressee( QValueList<KABC::Addressee>* list ) {
//     emit jobCreated( new EditAddressBookJob( kmobiletoolsJob::delAddressee,
//                                         list,
//                                         m_device,
//                                         this,
//                                         this->name() ) );
// 
//     emit jobCreated( new FetchAddressBookJob( m_device, this, this->name() ) );
// 
// }
// 
// 
// void kmobiletoolsGammu_engine::slotEditAddressee(KABC::Addressee* oldAddressee, KABC::Addressee* newAddressee) {
//     emit jobCreated( new EditAddressBookJob( oldAddressee,
//                                         newAddressee,
//                                         m_device,
//                                         this,
//                                         this->name() ) );
// 
//     emit jobCreated( new FetchAddressBookJob( m_device, this, this->name() ) );
// }


    /******************************************************
     * Calendar related methods                           *
     ******************************************************/
/*
void kmobiletoolsGammu_engine::fetchCalendar() {
    /// @todo implement me
}*/


/**************************************************************************/


extern "C"
{
    void* init_kmobiletools_gammu()
    {
      return new kmobiletoolsGammu_engineFactory;
    }
};

K_EXPORT_PLUGIN( kmobiletoolsGammu_engineFactory )

kmobiletoolsGammu_engineFactory::kmobiletoolsGammu_engineFactory()
 : KPluginFactory("kmobiletools_gammu")
{
}

kmobiletoolsGammu_engineFactory::~kmobiletoolsGammu_engineFactory()
{
}

QObject *kmobiletoolsGammu_engineFactory::create(
  const char *iface, QWidget *parentWidget, QObject *parent, const QVariantList &args, const QString &keyword )
{
    Q_UNUSED(iface)
    Q_UNUSED(parentWidget)
    Q_UNUSED(keyword)
    QString deviceName = args.at( 0 ).toString();
    return new kmobiletoolsGammu_engine( parent, deviceName );
}

#include "kmobiletoolsgammu_engine.moc"
