/*
 *  This file is part of kremail.
 *  Copyright (C) 2008 Thomas Gillespie <tomjamesgillespie@googlemail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "kremailPage.h"

#include <QMouseEvent>
#include <QPainter>
#include <QPen>

#include <KTemporaryFile>
#include <KProcess>

#include <KDebug>

KRemailPage::KRemailPage(QString file)
{
    m_pageImage = QImage(file);
    m_pagePixmap = QPixmap::fromImage(m_pageImage);
}

KRemailPage::KRemailPage(QImage img)
{
    m_pageImage = img;
    m_pagePixmap = QPixmap::fromImage(m_pageImage);
}

KRemailPage::~KRemailPage()
{
}

void KRemailPage::mousePressEvent (QMouseEvent * event)
{
    m_startMouse = m_currentMouse = event->posF();
    m_mouseDown = true;
}

void KRemailPage::mouseReleaseEvent (QMouseEvent * event)
{
    m_currentMouse = event->posF();
    m_mouseDown = false;
    repaint();

    int startX, startY, endX, endY;
    if(m_currentMouse.x() < m_startMouse.x())
    {
        startX = (int) m_currentMouse.x();
	endX = (int) m_startMouse.x();
    }
    else
    {
        endX = (int) m_currentMouse.x();
        startX = (int) m_startMouse.x();
    }
    if(m_currentMouse.y() < m_startMouse.y())
    {
        startY = (int) m_currentMouse.y();
        endY = (int) m_startMouse.y();
    }
    else
    {
        endY = (int) m_currentMouse.y();
        startY = (int) m_startMouse.y();
    }

    QRectF selectedArea = QRectF(QPoint(startX, startY), QPoint(endX, endY));
    double scaleX = m_pageImage.width()/(double)this->width();
    double scaleY = m_pageImage.height()/(double)this->height();
    selectedArea.setX(startX*scaleX);
    selectedArea.setY(startY*scaleY);
    selectedArea.setWidth((endX-startX)*scaleX);
    selectedArea.setHeight((endY-startY)*scaleY);
    QImage selection = m_pageImage.copy(selectedArea.toRect());

    KTemporaryFile tempImageFile1;
    tempImageFile1.setSuffix(".tif");
    tempImageFile1.open();
    KTemporaryFile tempImageFile2;
    tempImageFile2.setSuffix(".tif");
    tempImageFile2.open();
    KTemporaryFile tempOutputFile;
    tempOutputFile.setSuffix(".txt");
    tempOutputFile.open();

    selection.save(tempImageFile1.fileName(), "tif", 100);

    //<hack> - qt's tiff exporter fails
    KProcess proc;
    proc << "convert";
    proc << "-depth" << "8" << "-compress" << "none" << "-monochrome" << "-resize" << "200%" << "-density" << "150x150";
    proc << tempImageFile1.fileName();
    proc << tempImageFile2.fileName();
    proc.execute();
    //</hack>
    KProcess proc2;
    proc2 << "tesseract";
    proc2 << tempImageFile2.fileName();
    proc2 << tempOutputFile.fileName().remove(".txt");
    proc2.execute();
    QString text(tempOutputFile.readAll());
    emit textReady(text);
}

void KRemailPage::mouseMoveEvent (QMouseEvent * event)
{
    m_currentMouse = event->posF();
    repaint();
}

void KRemailPage::paintEvent (QPaintEvent * event)
{
    QPainter p(this);
    p.drawPixmap(rect(), m_pagePixmap, m_pagePixmap.rect());
    if(m_mouseDown)
    {
        QPen pen;
        pen.setWidth(3);
        p.setPen(pen);
        p.drawRect(QRectF(m_startMouse, m_currentMouse));
    }
    p.end();
}

QImage KRemailPage::image()
{
    return m_pageImage;
}
