/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "feedlistfilterproxymodel.h"

#include <KDebug>
#include <KLocale>

FeedlistFilterProxyModel::FeedlistFilterProxyModel( QObject *parent )
        : QSortFilterProxyModel( parent )
{
        setDynamicSortFilter( true );
}

FeedlistFilterProxyModel::~FeedlistFilterProxyModel()
{
}

void FeedlistFilterProxyModel::setSubscriptionLabel( const QString &subscriptionLabel )
{
        m_subscriptionLabel = subscriptionLabel;
        invalidateFilter();
}

QString FeedlistFilterProxyModel::subscriptionLabel() const
{
        return m_subscriptionLabel;
}

bool FeedlistFilterProxyModel::filterAcceptsRow( int sourceRow, const QModelIndex &sourceParent ) const
{
        if ( m_subscriptionLabel.isEmpty() )
                return true;

        QModelIndex sourceIndex = sourceModel()->index( sourceRow, 0, sourceParent );

        if ( sourceModel()->data( sourceIndex, FeedlistModel::TierRole ) == FeedlistModel::TagTier ) {
                if ( sourceModel()->rowCount( sourceIndex ) > 0 ) {
                        int rows = sourceModel()->rowCount( sourceIndex );
                        for ( int i = 0; i < rows; ++i ) {
                                QModelIndex feedSourceIndex = sourceModel()->index( i, 0, sourceIndex );
                                if ( sourceModel()->data( feedSourceIndex, FeedlistModel::SubscriptionLabelsRole ).toStringList().
                                                          contains( m_subscriptionLabel ) )
                                        return true;
                        }
                }

                return false;
        }

        return sourceModel()->data( sourceIndex, FeedlistModel::SubscriptionLabelsRole ).toStringList().contains( m_subscriptionLabel );
}
