/*
    Copyright (C) 2008    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KRSSREADER_MAINWIDGET_H
#define KRSSREADER_MAINWIDGET_H

#include <QtCore/QPointer>
#include <QtGui/QWidget>
#include <krss/treenode.h>

#include <boost/shared_ptr.hpp>

namespace KRss {
    class Item;
    class ItemListing;
    class ItemModel;
    class ItemView;
    class FeedList;
    class FeedListModel;
    class FeedListView;
    class Feed;
    class TagProvider;
    class StandardActionManager;
}

class KJob;
class KXmlGuiWindow;
class KLineEdit;
class KActionCollection;

class QSplitter;
class QWebView;

class MainWidget: public QWidget
{
    Q_OBJECT
public:
    explicit MainWidget( KXmlGuiWindow *parent = 0 );
    ~MainWidget();

private:
    void setCollection( const boost::shared_ptr<KRss::ItemListing>& coll );

private Q_SLOTS:

    void importAkregatorFeeds();
    void importOpml();
    void importOpmlFinished(KJob*);
    void importItems();
    void importItemsFinished(KJob*);

    void setCurrentNode( const boost::shared_ptr<KRss::TreeNode>& node );
    void setCurrentItem( const KRss::Item& item );

    void feedListRetrieved( KJob* );
    void itemListingDone( KJob* );
    void tagProviderRetrieved( KJob * );
    void fullItemFetched( KJob *job );

private:
    QSplitter *m_mainSplitter;
    QSplitter *m_itemSplitter;

    KLineEdit *m_searchEdit;
    KRss::ItemView *m_itemView;
    QWebView *m_itemViewer;
    QPointer<KRss::Feed> m_currentFeed;
    boost::shared_ptr<KRss::TreeNode> m_currentNode;
    KRss::FeedListView *m_feedListView;
    boost::shared_ptr<KRss::FeedList> m_feedList;
    boost::shared_ptr<const KRss::TagProvider> m_tagProvider;
    KRss::StandardActionManager *m_actionManager;
    QPointer<KActionCollection> m_actionCollection;
};

#endif // KRSSREADER_MAINWIDGET_H
