/*
    Copyright (c) 2009 Tobias Koenig <tokoe@kde.org>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#ifndef CONFIGDIALOG_H
#define CONFIGDIALOG_H

#include <kdialog.h>

#include "contactattributes.h"

namespace KLDAP {
class LdapConfigWidget;
}

class QCheckBox;
class QPushButton;
class QScrollArea;

class KComboBox;
class KLineEdit;

class ConfigDialog : public KDialog
{
  Q_OBJECT

  public:
    ConfigDialog( WId windowId );

  private Q_SLOTS:
    void save();
    void editAttributes();

  private:
    void load();

    QPushButton *mEditButton;
    KLDAP::LdapConfigWidget *mConfigWidget;
    QCheckBox *mSubTree;
    ContactAttributes mAttributes;
    int mRdnPrefix;
};

class AttributesDialog : public KDialog
{
  Q_OBJECT

  public:
    AttributesDialog( const ContactAttributes &attributes, int rdnPrefix, QWidget *parent = 0 );
    ~AttributesDialog();

    ContactAttributes attributes() const;
    int rdnPrefix() const;

  private Q_SLOTS:
    void schemaChanged( int );

  private:
    void updateAttributeFields();

    KComboBox *mSchemaCombo;
    QScrollArea *mScrollArea;
    KComboBox *mRdnCombo;
    QList<KLineEdit*> mLineEdits;
    ContactAttributes mAttributes;
};

#endif
