/*
    Copyright (c) 2009 Tobias Koenig <tokoe@kde.org>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#ifndef CONTACTATTRIBUTES_H
#define CONTACTATTRIBUTES_H

#include <QtCore/QMap>
#include <QtCore/QString>

class ContactAttributes
{
  public:
    enum Type
    {
      ObjectClass,
      CommonName,
      Uid,

      FormattedName,
      Title,
      GivenName,
      FamilyName,

      Birthday,

      GpgKey,
      SMimeCertificate,

      WorkMail,
      HomeMail,
      Url,

      HomePhoneNumber,
      WorkPhoneNumber,
      MobilePhoneNumber,
      FaxNumber,
      PagerNumber,
      IsdnNumber,

      WorkAddressPostOfficeBox,
      WorkAddressStreet,
      WorkAddressPostalCode,
      WorkAddressCity,
      WorkAddressState,

      HomeAddressPostOfficeBox,
      HomeAddressStreet,
      HomeAddressPostalCode,
      HomeAddressCity,
      HomeAddressState,

      Profession,
      Organization,
      Department,
      Office,

      Photo,
      Sound,
      Note
    };
    typedef QList<Type> TypeList;

    /**
     * Returns a list of all available attribute types.
     */
    static TypeList allTypes();

    /**
     * Returns the i18n'ed label for the given @p type.
     */
    static QString label( Type type );

    /**
     * Returns the attribute types that are available for
     * the contact attributes.
     */
    TypeList types() const;

    /**
     * Returns the LDAP attribute name that shall be used for
     * the given attribute @p type.
     *
     * E.g. 'sn' for FamilyName
     */
    QString name( Type type ) const;

    /**
     * Sets the LDAP attribute @p name for the given attribute @p type.
     */
    void setName( Type type, const QString &name );

    /**
     * Creates a new contact attributes object from a string @p value.
     */
    static ContactAttributes fromString( const QString &value );

    /**
     * Marshalls the contact attributes object to a string value.
     */
    QString toString() const;

    /**
     * Returns a contact attributes as defined in inetOrgPerson LDAP schema.
     */
    static ContactAttributes defaultAttributes();

  private:
    QMap<Type, QString> mAttributes;
};

#endif
