/*
    This file is part of KDE playground.
    Copyright (c) 2007 Mathias Soeken <msoeken@tzi.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "personfilterproxymodel.h"

#include <KDebug>

#include "personmodel.h"

PersonFilterProxyModel::PersonFilterProxyModel( QObject *parent ) : QSortFilterProxyModel( parent ),
  mMarkedItemsOnly( false ), mCategoryFilter()
{
}

void PersonFilterProxyModel::setMarkedItemsOnly( bool markedItemsOnly )
{
  mMarkedItemsOnly = markedItemsOnly;
}

bool PersonFilterProxyModel::isMarkedItemsOnly() const
{
  return mMarkedItemsOnly;
}

void PersonFilterProxyModel::setCategoryFilter( QString categoryFilter )
{
  mCategoryFilter = categoryFilter;
}

QString PersonFilterProxyModel::categoryFilter() const
{
  return mCategoryFilter;
}

bool PersonFilterProxyModel::filterAcceptsRow( int sourceRow, const QModelIndex& sourceParent ) const
{
  bool preFilter = true;
  
  if (mMarkedItemsOnly) {
    QModelIndex index = sourceModel()->index( sourceRow, 2, sourceParent );
    preFilter = sourceModel()->data( index ).toString() != "";
  }
  
  if (!preFilter) return false;
  
  if (!mCategoryFilter.isEmpty()) {
    QModelIndex index = sourceModel()->index( sourceRow, PersonModel::FieldsCount + 1, sourceParent );
    QStringList categories = sourceModel()->data( index, Qt::UserRole ).toStringList();
    preFilter = categories.contains( mCategoryFilter );
  }
  
  if (!preFilter) return false;
  
  bool match = false;
  for (int i = 0; i <= PersonModel::FieldsCount; ++i) {
    if (i == PersonModel::StatusColumn) continue;
    
    match = match || (sourceModel()->data( sourceModel()->index(sourceRow, i, sourceParent), Qt::UserRole ).toString().contains( filterRegExp() ));
  }
  
  return match;
}
