/*
    This file is part of KDE playground.
    Copyright (c) 2007 Mathias Soeken <msoeken@tzi.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef PERSONMODEL_H
#define PERSONMODEL_H

#include <QAbstractItemModel>
#include <QStringList>

class PersonModelItem;

namespace KABC {
  class StdAddressBook;
  class AddressBook;
}

using namespace KABC;

class PersonModel : public QAbstractItemModel 
{
  Q_OBJECT
      
  friend class PersonTreeView;
      
public:
  PersonModel( QObject *parent = 0 );
  virtual ~PersonModel();

private:
  KABC::StdAddressBook *mStdAddressBook;
  PersonModelItem *mRootItem;
  QStringList mCategories;
  bool mBasicInterface : true;
  
public:
  static const int FieldsCount = 5;
  static const int SpecialFieldsCount = 1;
  static const int StatusColumn = 2;    
  
  QStringList categories() const { return mCategories; }

public:
  virtual int columnCount( const QModelIndex& parent = QModelIndex () ) const;
  virtual QVariant headerData( int section, Qt::Orientation orientation, int role = Qt::DisplayRole ) const;
  virtual int rowCount( const QModelIndex& parent = QModelIndex () ) const;
  virtual QVariant data( const QModelIndex& index, int role = Qt::DisplayRole ) const;
  virtual QModelIndex index( int row, int column, const QModelIndex& parent = QModelIndex () ) const;
  virtual QModelIndex parent( const QModelIndex& index ) const;
  virtual bool setData( const QModelIndex& index, const QVariant& value, int role = Qt::EditRole );
  virtual Qt::ItemFlags flags( const QModelIndex& index ) const;
  
private slots:
  void addressBookChanged( AddressBook *addressBook );
};

#endif
