/*
    This file is part of KDE playground.
    Copyright (c) 2007 Mathias Soeken <msoeken@tzi.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef PERSONTREEVIEW_H
#define PERSONTREEVIEW_H

#include <QTreeView>

class QToolButton;

class PersonModel;
class PersonFilterProxyModel;

class PersonTreeView : public QTreeView {
  Q_OBJECT
  
public:
  PersonTreeView( QWidget* parent = 0 );      
  
public:
  PersonFilterProxyModel* proxyModel() const { return mProxyModel; }
  PersonModel* model() const { return mPersonModel; }
  
  void toggleSettings( bool basicInterface );
  
protected:
  void mouseMoveEvent( QMouseEvent* e );
  void leaveEvent( QEvent* e );
  void keyPressEvent( QKeyEvent* e );
  void paintEvent( QPaintEvent* e );
  
private:
  void setButtonIndex( const QModelIndex& index );      
  
private:
  PersonFilterProxyModel *mProxyModel;
  PersonModel *mPersonModel;
  
  QToolButton *mBtnTo, *mBtnCC, *mBtnBCC;
  int mSelectedIndex;
  QModelIndex mSelectedModelIndex;
  
private slots:
  void slotModelReset();
  void slotDoubleClicked( const QModelIndex& index );
  void btnToClicked( bool checked = false );
  void btnCCClicked( bool checked = false );
  void btnBCCClicked( bool checked = false );
};

#endif
