/*
   Copyright (c) 2009 Sascha Peilicke <sasch.pe@gmx.de>

   This application is free software; you can redistribute it and/or modify it
   under the terms of the GNU Library General Public License as published by
   the Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This application is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
   License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this application; see the file COPYING.LIB.  If not, write to the
   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA.
*/

#ifndef TIMETRACKINGOBSERVER_H
#define TIMETRACKINGOBSERVER_H

#include <Akonadi/AgentBase>
#include <Akonadi/Item>

#include <QHash>

class QDateTime;

/**
 * This class implements the Observer interface and allows to query
 * which items where added, modified or removed over time. The query can
 * be restricted to distinct collections.
 *
 * @author Sascha Peilicke <sasch.pe@gmx.de>
 */
class TimeTrackingObserver : public QObject
                           , public Akonadi::AgentBase::Observer
{
    Q_OBJECT

public:
    Akonadi::Item::List allItems(const QDateTime &timestamp, Akonadi::Entity::Id collectionId = -1);
    Akonadi::Item::List addedItems(const QDateTime &timestamp, Akonadi::Entity::Id collectionId = -1);
    Akonadi::Item::List changedItems(const QDateTime &timestamp, Akonadi::Entity::Id collectionId = -1);
    Akonadi::Item::List removedItems(const QDateTime &timestamp, Akonadi::Entity::Id collectionId = -1);

protected:
    virtual void itemAdded(const Akonadi::Item &item, const Akonadi::Collection &collection);
    virtual void itemChanged(const Akonadi::Item &item, const QSet<QByteArray> &partIdentifiers);
    virtual void itemRemoved(const Akonadi::Item &item);

private:
    Akonadi::Item::List items(const QHash<Akonadi::Entity::Id, Akonadi::Item> &items, const QDateTime &timestamp, Akonadi::Entity::Id collectionId);

    QList<Akonadi::Entity::Id> m_allItemsFetchedCollectionIds;

    QHash<Akonadi::Entity::Id, Akonadi::Item> m_allItems;
    QHash<Akonadi::Entity::Id, Akonadi::Item> m_addedItems;
    QHash<Akonadi::Entity::Id, Akonadi::Item> m_changedItems;
    QHash<Akonadi::Entity::Id, Akonadi::Item> m_removedItems;
};

#endif
