/*
    Copyright (c) 2009 Sascha Peilicke <sasch.pe@gmx.de>

    This application is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This application is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this application; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "clientagent.h"
#include "clientadaptor.h"              // D-Bus adaptor
#include "settings.h"                   // KConfigXT stuff
#include "settingsadaptor.h"            // D-Bus adaptor
#include "settingsdialog.h"

#include <Akonadi/ChangeRecorder>
#include <Akonadi/CollectionFetchJob>

#include <KWindowSystem>

#include <QDBusConnection>

#include <syncevo/declarations.h>
#include <syncevo/SyncContext.h>

using namespace Akonadi;

ClientAgent::ClientAgent(const QString &id)
    : AgentBase(id)
{
    // Setup D-Bus related stuff
    new ClientAdaptor(this);
    new SettingsAdaptor(Settings::self());
    QDBusConnection::sessionBus().registerObject(QLatin1String("/"), this);
    QDBusConnection::sessionBus().registerObject(QLatin1String("/Settings"), Settings::self());

    //TODO: Investigate AgentManager, AgentManagerInstance and friends to
    //      also have a visual change in the AgentInstanceWidget
    setAgentName(Settings::self()->name());

    init();
}

ClientAgent::~ClientAgent()
{
    // Unregister D-Bus objects
    QDBusConnection::sessionBus().unregisterObject("/", QDBusConnection::UnregisterTree);
    QDBusConnection::sessionBus().unregisterObject("/Settings", QDBusConnection::UnregisterTree);

    // Make sure that all custom settings are also saved
    Settings::self()->writeConfig();
}

void ClientAgent::configure(WId windowId)
{
    SettingsDialog dialog;

    if (windowId) {
        KWindowSystem::setMainWindow(&dialog, windowId);
    }
    if (dialog.exec() == QDialog::Accepted) {
        init();
    } else {
        if (Settings::self()->firstRun()) {
            const QString message = i18nc("@info:status", "Please configure the agent");
            emit status(Broken, message);
        }
    }
    Settings::self()->setFirstRun(false);
}

bool ClientAgent::sync()
{
    int index = 0;
    QString message;

    message = i18nc("@info:status", "Sync started");
    kDebug() << message;
    emit status(Running, message);

    //TODO: Complete synthesis sync stuff

    message = i18nc("@info:status", "Sync finished");
    kDebug() << message;
    emit status(Idle, message);
    return true;
}

void ClientAgent::init()
{
    // Every SyncML clients needs a globally unique identifier.
    QUuid id = QUuid(Settings::self()->uuid());
    if (id.isNull()) {
        id = QUuid::createUuid();
        kDebug() << "Created UUID:" << id;
        Settings::self()->setUuid(id.toString());
    }

    kDebug() << "Initializing...";

    /*// Unset all previously monitored collections
    foreach (const Collection &collection, changeRecorder()->collectionsMonitored()) {
        changeRecorder()->setCollectionMonitored(collection, false);
    }

    // Set new monitored collections. It's easier here to fetch all collections and
    // to see which ones we need actually.
    CollectionFetchJob *job = new CollectionFetchJob(Collection::root(), CollectionFetchJob::Recursive);
    if (job->exec()) {
        foreach (const Collection &collection, job->collections()) {
            if (Settings::self()->contactsEnabled() &&
                collection.id() == Settings::self()->contactsCollectionId()) {
                changeRecorder()->setCollectionMonitored(collection);
            }
            if (Settings::self()->eventsEnabled() &&
                collection.id() == Settings::self()->eventsCollectionId()) {
                changeRecorder()->setCollectionMonitored(collection);
            }
            if (Settings::self()->todosEnabled() &&
                collection.id() == Settings::self()->todosCollectionId()) {
                changeRecorder()->setCollectionMonitored(collection);
            }
            if (Settings::self()->notesEnabled() &&
                collection.id() == Settings::self()->notesCollectionId()) {
                changeRecorder()->setCollectionMonitored(collection);
            }
        }
    }

    kDebug() << "Monitored collections:";
    foreach (const Collection &collection, changeRecorder()->collectionsMonitored()) {
        kDebug() << "\tid:" << collection.id() << "name:" << collection.name();
    }*/

    if (!Settings::self()->firstRun()) {
        const QString message = i18nc("@info:status", "Ready");
        emit status(Idle, message);
    }
}

AKONADI_AGENT_MAIN(ClientAgent)

#include "clientagent.moc"
