# - Find funambol
# Find the native Funambol C++ Client SDK headers and libraries.
#
#  FUNAMBOL_FOUND           True if funambol got found
#  FUNAMBOL_INCLUDE_DIRS    Location of funambol headers 
#  FUNAMBOL_LIBRARIES       List of libraries to use funambol

# Look for the header file.
FIND_PATH(FUNAMBOL_COMMON_INCLUDE_DIR NAMES base/errors.h PATH_SUFFIXES funambol/common)
IF(DEFINED UNIX)
    FIND_PATH(FUNAMBOL_POSIX_INCLUDE_DIR NAMES base/posixadapter.h PATH_SUFFIXES funambol/posix)
    MARK_AS_ADVANCED(FUNAMBOL_COMMON_INCLUDE_DIR FUNAMBOL_POSIX_INCLUDE_DIR)
ENDIF(DEFINED UNIX)
IF(DEFINED WIN32)
    FIND_PATH(FUNAMBOL_WINDOWS_INCLUDE_DIR NAMES base/winadapter.h PATH_SUFFIXES funambol/windows)
    MARK_AS_ADVANCED(FUNAMBOL_COMMON_INCLUDE_DIR FUNAMBOL_WINDOWS_INCLUDE_DIR)
ENDIF(DEFINED WIN32)

# Look for the library.
FIND_LIBRARY(FUNAMBOL_LIBRARY NAMES funambol)
MARK_AS_ADVANCED(FUNAMBOL_LIBRARY)

# handle the QUIETLY and REQUIRED arguments and set FUNAMBOL_FOUND to TRUE if 
# all listed variables are TRUE
INCLUDE(FindPackageHandleStandardArgs)
IF(DEFINED UNIX)
    FIND_PACKAGE_HANDLE_STANDARD_ARGS(FUNAMBOL DEFAULT_MSG FUNAMBOL_LIBRARY FUNAMBOL_COMMON_INCLUDE_DIR FUNAMBOL_POSIX_INCLUDE_DIR)
ENDIF(DEFINED UNIX)
IF(DEFINED WIN32)
    FIND_PACKAGE_HANDLE_STANDARD_ARGS(FUNAMBOL DEFAULT_MSG FUNAMBOL_LIBRARY FUNAMBOL_COMMON_INCLUDE_DIR FUNAMBOL_WINDOWS_INCLUDE_DIR)
ENDIF(DEFINED WIN32)

IF(FUNAMBOL_FOUND)
    SET(FUNAMBOL_LIBRARIES ${FUNAMBOL_LIBRARY})
    IF(DEFINED UNIX)
        SET(FUNAMBOL_INCLUDE_DIRS ${FUNAMBOL_COMMON_INCLUDE_DIR} ${FUNAMBOL_POSIX_INCLUDE_DIR})
    ENDIF(DEFINED UNIX)
    IF(DEFINED WIN32)
        SET(FUNAMBOL_INCLUDE_DIRS ${FUNAMBOL_COMMON_INCLUDE_DIR} ${FUNAMBOL_WINDOWS_INCLUDE_DIR})
    ENDIF(DEFINED WIN32)
ELSE(FUNAMBOL_FOUND)
    SET(FUNAMBOL_LIBRARIES)
    SET(FUNAMBOL_INCLUDE_DIRS)
ENDIF(FUNAMBOL_FOUND)
