/*
    Copyright (c) 2009 Sascha Peilicke <sasch.pe@gmx.de>

    This application is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This application is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this application; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#ifndef SYNCML_DATASTORE_H
#define SYNCML_DATASTORE_H

#include <QDateTime>
#include <QDebug>
#include <QString>

/**
 * This is a helper object for the ServerAgent class.
 */
class DataStore : public QObject
{
    Q_OBJECT

public:
    DataStore(const QString &resourceName, QObject *parent = 0);

    QString contentType() const { return m_contentType; }
    QString source() const { return m_source; }
    QString target() const { return m_target; }

    void setUseSlowSync(bool useSlowSync);
    bool useSlowSync() const { return m_useSlowSync; }

    void setLastSyncTime(const QDateTime &lastSyncTime);
    QDateTime lastSyncTime() const { return m_lastSyncTime; }

    void setLastSyncSuccessful(bool lastSyncSuccessfull);
    bool lastSyncSuccessfull() const { return m_lastSyncSuccessfull; }

    bool isValid() const { return m_isValid; }

private:
    QString m_contentType;
    QString m_source;
    QString m_target;

    QDateTime m_lastSyncTime;
    bool m_lastSyncSuccessfull;

    bool m_useSlowSync;
    bool m_isValid;
};

QDebug &operator<<(QDebug &dbg, const DataStore &dataStore);

#endif
