/*
    Copyright (c) 2009 Sascha Peilicke <sasch.pe@gmx.de>

    This application is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This application is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this application; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#ifndef SYNCML_SERVERAGENT_H
#define SYNCML_SERVERAGENT_H

#include "datastore.h"

#include <Akonadi/AgentBase>

struct SmlDataSyncObject;
struct SmlError;

/**
 *
 */
class ServerAgent : public Akonadi::AgentBase
                  , public Akonadi::AgentBase::Observer
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.kde.Akonadi.SyncML.Server")

public:
    ServerAgent(const QString &id);
    virtual ~ServerAgent();

    DataStore *dataStore(const QString &source);
    bool sendChanges(SmlDataSyncObject *dsObject, SmlError **localError);

public Q_SLOTS:
    virtual void configure(WId windowId);
    bool startServer();
    bool restartServer();
    void stopServer();

protected:
    virtual void itemAdded(const Akonadi::Item &item, const Akonadi::Collection &collection);
    virtual void itemChanged(const Akonadi::Item &item, const QSet<QByteArray> &partIdentifiers);
    virtual void itemRemoved(const Akonadi::Item &item);

private:
    bool initSettings();
    bool initDataStores();

    SmlDataSyncObject *m_smlDataSyncObject;
    SmlError *m_smlError;

    QList<DataStore *> *m_dataStores;
    bool m_isServerRunning;
};

#endif
