/*
    Copyright (c) 2009 Sascha Peilicke <sasch.pe@gmx.de>

    This application is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This application is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this application; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#include "settingsdialog.h"
#include "settings.h"

#include <KConfigDialogManager>

SettingsDialog::SettingsDialog(QWidget *parent)
    : KDialog(parent)
{
    m_ui.setupUi(mainWidget());
    setCaption(Settings::self()->name());

    m_manager = new KConfigDialogManager(this, Settings::self());
    m_manager->updateWidgets();
    m_proxyModel.addMimeTypeFilters(QStringList() << "text/calendar"
                                                  << "text/directory"
                                                  << "application/x-vnd.kde.notes");
    m_proxyModel.setSourceModel(&m_model);
    m_ui.collectionView->setModel(&m_proxyModel);

    connect(this, SIGNAL(okClicked()), SLOT(save()));
    connect(&m_proxyModel, SIGNAL(rowsInserted(const QModelIndex &, int, int)),
            this, SLOT(applyResourceSelection(const QModelIndex &, int, int)));
    connect(m_ui.kcfg_Transport, SIGNAL(currentIndexChanged(int)), this, SLOT(changeSslWidgets()));

    changeSslWidgets();
}

SettingsDialog::~SettingsDialog()
{
    delete m_manager;
}

void SettingsDialog::save()
{
    m_manager->updateSettings();

    QStringList selectedResourceNames;
    QStringList selectedResourceIdentifiers;
    foreach (const QModelIndex &index, m_ui.collectionView->selectionModel()->selectedIndexes()) {
        selectedResourceNames << index.data().toString();
    }
    Settings::self()->setMonitoredResourceNames(selectedResourceNames);
    Settings::self()->writeConfig();
}

void SettingsDialog::applyResourceSelection(const QModelIndex &, int start, int end)
{
    // Set reasonable defaults when the server agent is run the first time
    // and the config dialog is shown. Otherwise use the user-configured values.
    if (Settings::self()->firstRun()) {
        for (int i = start; i <= end; i++) {
            const QModelIndex index = m_proxyModel.index(i, 0);
            m_ui.collectionView->selectionModel()->select(index, QItemSelectionModel::Select);
        }
    } else {
        QStringList resourcesToSelect = Settings::self()->monitoredResourceNames();

        for (int i = start; i <= end; i++) {
            const QModelIndex index = m_proxyModel.index(i, 0);

            if (resourcesToSelect.contains(index.data().toString())) {
                m_ui.collectionView->selectionModel()->select(index, QItemSelectionModel::Select);
            }
        }
    }
}

void SettingsDialog::changeSslWidgets()
{
    if (m_ui.kcfg_Transport->currentIndex() == 0) { /* Means HTTP */
        m_ui.sslLabel1->setEnabled(true);
        m_ui.sslLabel2->setEnabled(true);
        m_ui.kcfg_SslKeyFile->setEnabled(true);
        m_ui.kcfg_SslCertificateFile->setEnabled(true);
    } else {
        m_ui.sslLabel1->setEnabled(false);
        m_ui.sslLabel2->setEnabled(false);
        m_ui.kcfg_SslKeyFile->setEnabled(false);
        m_ui.kcfg_SslCertificateFile->setEnabled(false);
    }
}

#include "settingsdialog.moc"
